@@ -27,6 +27,10 @@
  *
  *
  */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 
 #include "sctp.h"
 #include <signal.h>
@@ -41,7 +45,6 @@
 #define POLLOUT    0x004
 #define POLLERR    0x008
 
-
 #define CHAT_PORT                          2345
 #define MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES    10
 #define MAXIMUM_NUMBER_OF_IN_STREAMS         10
@@ -50,9 +53,7 @@
 
 #define MAX_BUFFER_LENGTH                   1400	
 
-#ifndef min
 #define min(x,y)            (x)<(y)?(x):(y)
-#endif
 
 static unsigned char localAddressList[MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES][SCTP_MAX_IP_LEN];
 static unsigned char destinationAddress[SCTP_MAX_IP_LEN];
@@ -108,7 +109,7 @@ void getArgs(int argc, char **argv)
         switch (c) {
         case 'd':
             if (strlen(optarg) < SCTP_MAX_IP_LEN) {
-                strcpy((char *)destinationAddress, optarg);
+                strcpy(destinationAddress, optarg);
 		client = 1;
             }
             break;
@@ -121,7 +122,7 @@ void getArgs(int argc, char **argv)
         case 's':
             if ((noOfLocalAddresses < MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES) &&
                 (strlen(optarg) < SCTP_MAX_IP_LEN  )) {
-                strcpy((char *)localAddressList[noOfLocalAddresses], optarg);
+                strcpy(localAddressList[noOfLocalAddresses], optarg);
                 noOfLocalAddresses++;
             }
             break;
@@ -178,7 +179,7 @@ void getDestinationIPaddr(char paddr[SCTP_MAX_IP_LEN])
 
    pathID = sctp_getPrimary(associationID);
    sctp_getPathStatus(associationID, pathID, &pathStatus);
-   strcpy((char *)paddr, (const char *)pathStatus.destinationAddress);
+   strcpy(paddr,pathStatus.destinationAddress);
 
    /* refreshes the ncurses window with the new IP address */
    mvwaddstr(peerWinStatus, 0, 0, paddr);
@@ -225,7 +226,7 @@ void initializecurses()
     selfWinStatus = newwin(1,selfX2,0,0);
     peerWinStatus = newwin(1,peerX2,peerY1-1,0);
     
-    getDestinationIPaddr((char *)peeraddr);
+    getDestinationIPaddr(peeraddr);
 
     waddstr(selfWinStatus,usrid);
  
@@ -249,7 +250,7 @@ void dataArriveNotif(unsigned int assocID, unsigned int streamID, unsigned int l
                      unsigned short streamSN,unsigned int TSN, unsigned int protoID, unsigned int unordered, void* ulpDataPtr)
 {
     unsigned char chunk[SCTP_MAXIMUM_DATA_LENGTH];
-    unsigned int length;
+    int length;
     unsigned int tsn;
     unsigned short ssn;
  
@@ -266,7 +267,7 @@ void dataArriveNotif(unsigned int assocID, unsigned int streamID, unsigned int l
     if (!(len>length)) {
       chunk[len] = 0;
     }
-    waddstr(peerWin, (char *)chunk);
+    waddstr(peerWin,chunk);
     wrefresh(peerWin);
 
 }
@@ -281,7 +282,7 @@ void sendFailureNotif(unsigned int assocID,
   }
 }
 
-void networkStatusChangeNotif(unsigned int assocID, short destAddrIndex, unsigned short newState, void* ulpDataPtr)
+void networkStatusChangeNotif(unsigned int assocID, unsigned int destAddrIndex, int newState, void* ulpDataPtr)
 {
     SCTP_AssociationStatus assocStatus;
     SCTP_PathStatus pathStatus;
@@ -300,25 +301,26 @@ void networkStatusChangeNotif(unsigned int assocID, short destAddrIndex, unsigne
         
         /* select a new one */ 
         sctp_getAssocStatus(assocID, &assocStatus);
-        for (pathID=0; pathID < assocStatus.numberOfAddresses; pathID++){
+        for (pathID=0; pathID < assocStatus.numberOfDestinationPaths; pathID++){
             sctp_getPathStatus(assocID, pathID, &pathStatus);
             if (pathStatus.state == SCTP_PATH_OK)
                 break;
         }
         
         /* and use it */
-        if (pathID < assocStatus.numberOfAddresses) {
+        if (pathID < assocStatus.numberOfDestinationPaths) {
             sctp_setPrimary(assocID, pathID);
-	    getDestinationIPaddr((char *)peeraddr);
+	    getDestinationIPaddr(peeraddr);
 	   
         }
     }
 }
 
 void* communicationUpNotif(unsigned int assocID, int status,
-                           unsigned int noOfDestinations,
-                           unsigned short noOfInStreams, unsigned short noOfOutStreams,
-                           int associationSupportsPRSCTP,void* dummy)
+                            unsigned int noOfDestinations,
+                            unsigned short noOfInStreams, unsigned short noOfOutStreams,
+                            int associationSupportsPRSCTP, int adLayerIndLen,
+                            void* adLayerInd,void* dummy)
 {	
 
     associationID=assocID;
@@ -328,7 +330,7 @@ void* communicationUpNotif(unsigned int assocID, int status,
     return NULL;
 }
 
-void communicationLostNotif(unsigned int assocID, unsigned short status, void* ulpDataPtr)
+void communicationLostNotif(unsigned int assocID, int status, void* ulpDataPtr)
 {	
     unsigned char buffer[SCTP_MAXIMUM_DATA_LENGTH];
     unsigned int bufferLength;
@@ -365,7 +367,7 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
     sctp_deleteAssociation(assocID);
 }
 
-void communicationErrorNotif(unsigned int assocID, unsigned short status, void* dummy)
+void communicationErrorNotif(unsigned int assocID, int status, void* dummy)
 {
   if (verbose) {  
     sprintf(tstr, "%-8x: Communication error (status %u)\n", assocID, status);
@@ -405,14 +407,13 @@ void shutdownCompleteNotif(unsigned int assocID, void* ulpDataPtr)
     
 }
 
-void stdinCallback(int fd,short int revent, short int* gotEvents, void* dummy)
+void stdinCallback(int fd, short int revent, short int* gotEvent, void* dummy)
 {
 /*   this function gets triggered by the ncurses library upon *any* keystroke 
      from user (for handling of Ctrl-C and Ctrl-\ see function finish()) */
     
     static int t;
-    unsigned char c;
-    char buffer[MAX_BUFFER_LENGTH];
+    unsigned char c, buffer[MAX_BUFFER_LENGTH];
  
     /* call ncurses function wgetch() to read in one user keystroke. 
        Did not use the wgetstr() function because it blocks until the user#
@@ -436,13 +437,13 @@ void stdinCallback(int fd,short int revent, short int* gotEvents, void* dummy)
 	strcpy(buffer,usrid);        /* adds the user ID into buffer */
 	buffer[usrid_len] = '\0';
 	strcat(buffer,"# ");
-	strcat(buffer, (char *)buf);          /* adds the contents of buf into buffer */
+	strcat(buffer,buf);          /* adds the contents of buf into buffer */
 	sctp_send(associationID,
                   0,
-                  (unsigned char *)buffer, strlen(buffer),
+                  buffer, strlen(buffer),
                   SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
-                  SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, 
-                  SCTP_INFINITE_LIFETIME, SCTP_ORDERED_DELIVERY,
+                  SCTP_USE_PRIMARY,  
+                  SCTP_INFINITE_LIFETIME, SCTP_NO_CONTEXT, SCTP_ORDERED_DELIVERY,
                   SCTP_BUNDLING_DISABLED);
 	bufCount=0;
 	waddch(selfWin,'\n');
@@ -492,7 +493,7 @@ static void finish(int sig)
 
 int main(int argc, char **argv)
 {
-    int sctpInstance;
+    unsigned short sctpInstance;
     SCTP_ulpCallbacks terminalUlp;
     SCTP_InstanceParameters instanceParameters;
 
