@@ -1,74 +1,60 @@
 /*
- * --------------------------------------------------------------------------
- *
- *           //=====   //===== ===//=== //===//  //       //   //===//
- *          //        //         //    //    // //       //   //    //
- *         //====//  //         //    //===//  //       //   //===<<
- *              //  //         //    //       //       //   //    //
- *       ======//  //=====    //    //       //=====  //   //===//
- *
- * -------------- An SCTP implementation according to RFC 4960 --------------
+ *  $Id: combined_server.c,v 1.5 2004/01/13 16:13:14 ajung Exp $
  *
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
- * Copyright (C) 2001-2004 Andreas Jungmaier
- * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Acknowledgements:
- * Realized in co-operation between Siemens AG and the University of
- * Duisburg-Essen, Institute for Experimental Mathematics, Computer
- * Networking Technology group.
- * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany
- * (Förderkennzeichen 01AK045).
- * The authors alone are responsible for the contents.
+ * Realized in co-operation between Siemens AG
+ * and University of Essen, Institute of Computer Networking Technology.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * There are two mailinglists available at http://www.sctp.de which should be
+ * used for any discussion related to this implementation.
+ *
+ * Contact: discussion@sctp.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
+ *
+ * NO THIS IS NOT THE COMBINED SERVER, YET. IT IS A TEST APPLICATION FOR
+ * ME - Andreas
  *
- * Contact: sctp-discussion@sctp.de
- *          thomas.dreibholz@gmail.com
- *          tuexen@fh-muenster.de
- *          andreas.jungmaier@web.de
  */
 
-#ifndef WIN32
+
 #include <unistd.h>
-#endif
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>         /* for atoi() under Linux */
 #include <time.h>
 #include "sctp_wrapper.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
 
 #define ECHO_PORT                             7
 #define DISCARD_PORT                          9
 #define DAYTIME_PORT                         13
 #define CHARGEN_PORT                         19
 
-#define MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES    10
+#define MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES    15
 #define MAXIMUM_NUMBER_OF_IN_STREAMS         17
 #define MAXIMUM_NUMBER_OF_OUT_STREAMS        17
 #define MAXIMUM_PAYLOAD_LENGTH             8192
 #define BUFFER_LENGTH                      1024
 #define SEND_QUEUE_SIZE                     100
 
-#ifndef min
 #define min(x,y)            (x)<(y)?(x):(y)
-#endif
 
 struct ulp_data {
     int maximumStreamID;
@@ -84,33 +70,6 @@ static int unknownCommand                     = 0;
 static int sendOOTBAborts                     = 1;
 static int timeToLive                         = SCTP_INFINITE_LIFETIME;
 
-const char *
-pathStateName(unsigned int state)
-{
-    switch (state) {
-        case SCTP_PATH_OK:
-            return "REACHABLE";
-            break;
-        case SCTP_PATH_UNREACHABLE:
-            return "UNREACHABLE";
-            break;
-        case SCTP_PATH_ADDED:
-            return "ADDED";
-            break;
-        case SCTP_PATH_REMOVED:
-            return "REMOVED";
-            break;
-        case SCTP_PATH_CONFIRMED:
-            return "CONFIRMED";
-            break;
-        case SCTP_PATH_UNCONFIRMED:
-            return "UNCONFIRMED";
-            break;
-        default:
-            return "UNKNOWN";
-            break;
-    }
-}
 
 void printUsage(void)
 {
@@ -120,56 +79,45 @@ void printUsage(void)
     printf("-s       source address\n");
     printf("-t       time to live in ms\n");
     printf("-v       verbose mode\n");
-    printf("-V       very verbose mode\n");
+    printf("-V       very verbose mode\n");   
 }
 
 void getArgs(int argc, char **argv)
 {
-    int i;
-    char *opt;
+    int c;
+    extern char *optarg;
+    extern int optind;
 
-    for(i=1; i < argc ;i++) {
-        if (argv[i][0] == '-') {
-            switch (argv[i][1]) {
-                case 'h':
-                  printUsage();
-                  exit(0);
-                case 's':
-                  if (i+1 >= argc) {
-                     printUsage();
-                     exit(0);
-                  }
-                  opt = argv[++i];
-                  if ((noOfLocalAddresses < MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES) && (strlen(opt) < SCTP_MAX_IP_LEN  )) {
-                     strcpy((char *)localAddressList[noOfLocalAddresses], opt);
-                     noOfLocalAddresses++;
-                  };
-                break;
-                case 'i':
-                  sendOOTBAborts = 0;
-                  break;
-                case 't':
-                  if (i+1 >= argc) {
-                     printUsage();
-                     exit(0);
-                  }
-                  opt=argv[++i];
-                  timeToLive = atoi(opt);
-                break;
-                case 'v':
-                  verbose = 1;
-                break;
-                case 'V':
-                  verbose = 1;
-                  vverbose = 1;
-                break;
-                default:
-                  unknownCommand = 1;
-                break;
-         }
-        }
-      else
-         unknownCommand = 1;
+    while ((c = getopt(argc, argv, "hs:it:vV")) != -1)
+    {
+        switch (c) {
+        case 'h':
+            printUsage();
+            exit(0);
+        case 's':
+            if ((noOfLocalAddresses < MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES) &&
+                (strlen(optarg) < SCTP_MAX_IP_LEN  )) {
+                strcpy(localAddressList[noOfLocalAddresses], optarg);
+                noOfLocalAddresses++;
+            };
+            break;
+        case 'i':
+            sendOOTBAborts = 0;
+            break;
+        case 't':
+            timeToLive = atoi(optarg);
+            break;
+        case 'v':
+            verbose = 1;
+            break;
+        case 'V':
+            verbose = 1;
+            vverbose = 1;
+            break;
+        default:
+            unknownCommand = 1;
+            break;
+       }
     }
 }
 
@@ -177,15 +125,15 @@ void checkArgs(void)
 {
     int abortProgram;
     int printUsageInfo;
-
+    
     abortProgram = 0;
     printUsageInfo = 0;
-
+    
     if (noOfLocalAddresses == 0) {
 #ifdef HAVE_IPV6
-        strcpy((char *)localAddressList[noOfLocalAddresses], "::0");
+        strcpy(localAddressList[noOfLocalAddresses], "::0");
 #else
-        strcpy((char *)localAddressList[noOfLocalAddresses], "0.0.0.0");
+        strcpy(localAddressList[noOfLocalAddresses], "0.0.0.0");
 #endif
         noOfLocalAddresses++;
     }
@@ -193,47 +141,47 @@ void checkArgs(void)
          printf("Error:   Unkown options in command.\n");
          printUsageInfo = 1;
     }
-
+    
     if (printUsageInfo == 1)
         printUsage();
     if (abortProgram == 1)
         exit(-1);
 }
 
-void echoDataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int len,
+void echoDataArriveNotif(unsigned int assocID, unsigned int streamID, unsigned int len,
                          unsigned short streamSN, unsigned int TSN, unsigned int protoID, unsigned int unordered, void* ulpDataPtr)
 {
     unsigned char chunk[MAXIMUM_PAYLOAD_LENGTH];
-    unsigned int length;
+    int length;
     unsigned short ssn;
     unsigned int tsn;
 
-    if (vverbose) {
+    if (vverbose) {  
       fprintf(stdout, "%-8x: Data arrived (%u bytes on stream %u, %s)\n",
                       assocID, len, streamID, (unordered==SCTP_ORDERED_DELIVERY)?"ordered":"unordered");
       fflush(stdout);
     }
     /* read it */
     length = MAXIMUM_PAYLOAD_LENGTH;
-    SCTP_receive(assocID, streamID, chunk, &length, &ssn, &tsn, SCTP_MSG_DEFAULT);
+    SCTP_receive(assocID, streamID, chunk, &length,&ssn, &tsn, SCTP_MSG_DEFAULT);
     /* and send it */
     SCTP_send(assocID,
-              (unsigned short)min(streamID, ((struct ulp_data *) ulpDataPtr)->maximumStreamID),
+              min(streamID, ((struct ulp_data *) ulpDataPtr)->maximumStreamID),
               chunk, length,
               protoID,
               SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive, unordered, SCTP_BUNDLING_DISABLED);
 }
 
-void dataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int len,
+void dataArriveNotif(unsigned int assocID, unsigned int streamID, unsigned int len,
                      unsigned short streamSN, unsigned int TSN, unsigned int protoID,
                      unsigned int unordered, void* ulpDataPtr)
 {
     unsigned char chunk[MAXIMUM_PAYLOAD_LENGTH];
-    unsigned int length;
+    int length;
     unsigned short ssn;
     unsigned int tsn;
 
-    if (vverbose) {
+    if (vverbose) {  
       fprintf(stdout, "%-8x: Data arrived (%u bytes on stream %u, %s)\n",
                       assocID, len, streamID, (unordered==SCTP_ORDERED_DELIVERY)?"ordered":"unordered");
       fflush(stdout);
@@ -246,42 +194,53 @@ void dataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int
 void sendFailureNotif(unsigned int assocID,
                       unsigned char *unsent_data, unsigned int dataLength, unsigned int *context, void* dummy)
 {
-  if (verbose) {
+  if (verbose) {  
     fprintf(stdout, "%-8x: Send failure\n", assocID);
     fflush(stdout);
   }
 }
 
-void networkStatusChangeNotif(unsigned int assocID, short affectedPathID, unsigned short newState, void* ulpDataPtr)
+void networkStatusChangeNotif(unsigned int assocID, unsigned int affectedPathID, int newState, void* ulpDataPtr)
 {
     SCTP_AssociationStatus assocStatus;
     SCTP_PathStatus pathStatus;
-    unsigned short pathID;
+    unsigned int pathKey;
+    int pathIndex;
+    unsigned char status[40];
+
+    if (newState == SCTP_PATH_OK) strcpy(status, "ACTIVE STATE");
+    if (newState == SCTP_PATH_UNREACHABLE) strcpy(status,"INACTIVE");
+    if (newState == SCTP_PATH_ADDED) strcpy(status, "PATH ADDED");
+    if (newState == SCTP_PATH_REMOVED) strcpy(status, "PATH DELETED");
+    if (newState == SCTP_ASCONF_SUCCEEDED) strcpy(status, "ASCONF SUCCEEDED");
+    if (newState == SCTP_ASCONF_FAILED) strcpy(status, "ASCONF FAILED");
+    if (newState == SCTP_PATH_CONFIRMED) strcpy(status, "PATH CONFIRMED");
+    if (newState == SCTP_PATH_UNCONFIRMED) strcpy(status, "PATH UNCONFIRMED");    
 
     if (verbose) {
         SCTP_getPathStatus(assocID, affectedPathID, &pathStatus);
-        fprintf(stdout, "%-8x: Network status change: path %u (towards %s) is now %s\n",
+        fprintf(stdout, "%-8x: Network status change: path %u (towards %s) is now %s\n", 
                         assocID, affectedPathID,
                         pathStatus.destinationAddress,
-                        pathStateName(newState));
+                        status);
         fflush(stdout);
     }
-
+    
     /* if the primary path has become inactive */
-    if ((newState == SCTP_PATH_UNREACHABLE) &&
-        (affectedPathID == SCTP_getPrimary(assocID))) {
-
+    if ((newState == SCTP_PATH_UNREACHABLE) && (affectedPathID == SCTP_getPrimary(assocID))) {
+        
         /* select a new one */
         SCTP_getAssocStatus(assocID, &assocStatus);
-        for (pathID=0; pathID < assocStatus.numberOfAddresses; pathID++){
-            SCTP_getPathStatus(assocID, pathID, &pathStatus);
+        for (pathIndex=0; pathIndex < assocStatus.numberOfDestinationPaths ; pathIndex++){
+            pathKey = assocStatus.destinationPathIDs[pathIndex];
+            SCTP_getPathStatus(assocID, pathKey, &pathStatus);
             if (pathStatus.state == SCTP_PATH_OK)
                 break;
         }
-
+        
         /* and use it */
-        if (pathID < assocStatus.numberOfAddresses) {
-            SCTP_setPrimary(assocID, pathID);
+        if (pathIndex < assocStatus.numberOfDestinationPaths) {
+            SCTP_setPrimary(assocID, pathKey);
         }
     }
 }
@@ -290,142 +249,104 @@ void networkStatusChangeNotif(unsigned int assocID, short affectedPathID, unsign
 void* echoCommunicationUpNotif(unsigned int assocID, int status,
                                unsigned int noOfPaths,
                                unsigned short noOfInStreams, unsigned short noOfOutStreams,
-                               int associationSupportsPRSCTP, void* dummy)
-{
+                               int associationSupportsPRSCTP, int adLayerIndLen, void* adLayerInd, void* dummy)
+{	
     struct ulp_data *ulpDataPtr;
-    SCTP_PathStatus pathStatus;
-    unsigned int i;
 
-    if (verbose) {
+    if (verbose) {  
         fprintf(stdout, "%-8x: Communication up (%u paths)\n", assocID, noOfPaths);
         fflush(stdout);
     }
-
-    if (vverbose) {
-        for (i=0; i < noOfPaths; i++) {
-            SCTP_getPathStatus(assocID, (short)i, &pathStatus);
-            fprintf(stdout, "%-8x: Path Status of path %u (towards %s): %s.\n", assocID, i, pathStatus.destinationAddress, pathStateName(pathStatus.state));
-        }
-    }
-
-    ulpDataPtr                   = (struct ulp_data*)malloc(sizeof(struct ulp_data));
+  
+    ulpDataPtr                   = malloc(sizeof(struct ulp_data));
     ulpDataPtr->maximumStreamID  = noOfOutStreams - 1;
     ulpDataPtr->ShutdownReceived = 0;
     return((void *) ulpDataPtr);
 }
 
 void* daytimeCommunicationUpNotif(unsigned int assocID, int status,
-                                  unsigned int noOfPaths,
+                                  unsigned int noOfDestinations,
                                   unsigned short noOfInStreams, unsigned short noOfOutStreams,
-                                  int associationSupportsPRSCTP, void* dummy)
-{
+                                  int associationSupportsPRSCTP, int adLayerIndLen, void* adLayerInd, void* dummy)
+{	
     char *timeAsString;
     time_t now;
-    SCTP_PathStatus pathStatus;
-    unsigned int i;
-
-    if (verbose) {
-        fprintf(stdout, "%-8x: Communication up (%u paths)\n", assocID, noOfPaths);
+ 
+    if (verbose) {  
+        fprintf(stdout, "%-8x: Communication up (%u paths)\n", assocID, noOfDestinations);
         fflush(stdout);
     }
-
-    if (vverbose) {
-        for (i=0; i < noOfPaths; i++) {
-            SCTP_getPathStatus(assocID, (short)i, &pathStatus);
-            fprintf(stdout, "%-8x: Path Status of path %u (towards %s): %s.\n", assocID, i, pathStatus.destinationAddress, pathStateName(pathStatus.state));
-        }
-    }
-
+   
     /* get the current time and convert to string */
     time(&now);
     timeAsString = ctime(&now);
 
-    if (vverbose) {
+    if (vverbose) {  
         fprintf(stdout, "%-8x: Current Time: %s", assocID, timeAsString);
         fflush(stdout);
     }
-
+   
     SCTP_send(assocID,
               0,
-              (unsigned char *)timeAsString, strlen(timeAsString),
+              timeAsString, strlen(timeAsString),
               SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
-              SCTP_USE_PRIMARY, SCTP_NO_CONTEXT,
-             timeToLive, SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED);
-
+              SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, 
+	          timeToLive, SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED);
+    
     SCTP_shutdown(assocID);
 
     return NULL;
 }
 
 void* chargenCommunicationUpNotif(unsigned int assocID, int status,
-                                  unsigned int noOfPaths,
+                                  unsigned int noOfDestinations,
                                   unsigned short noOfInStreams, unsigned short noOfOutStreams,
-                                  int associationSupportsPRSCTP, void* dummy)
-{
+                                  int associationSupportsPRSCTP, int adLayerIndLen, void* adLayerInd, void* dummy)
+{	
     int length;
-    unsigned char buffer[BUFFER_LENGTH];
+    char buffer[BUFFER_LENGTH];
     struct ulp_data *ulpDataPtr;
-    SCTP_PathStatus pathStatus;
-    unsigned int i;
 
-    if (verbose) {
-        fprintf(stdout, "%-8x: Communication up (%u paths)\n", assocID, noOfPaths);
+    if (verbose) {  
+        fprintf(stdout, "%-8x: Communication up (%u paths)\n", assocID, noOfDestinations);
         fflush(stdout);
     }
-
-    if (vverbose) {
-        for (i=0; i < noOfPaths; i++) {
-            SCTP_getPathStatus(assocID, (short)i, &pathStatus);
-            fprintf(stdout, "%-8x: Path Status of path %u (towards %s): %s.\n", assocID, i, pathStatus.destinationAddress, pathStateName(pathStatus.state));
-        }
-    }
-
-    length = 1 + (rand() % 512);
-    memset((void *)buffer, 'A', length);
+    
+    length = 1 + (random() % 512);
+    memset(buffer, 'A', length);
     buffer[length-1] = '\n';
-
+    
     while(SCTP_send(assocID, 0, buffer, length, SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
-                    SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive,
+                    SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive, 
                     SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED) == SCTP_SUCCESS) {
       if (vverbose) {
           fprintf(stdout, "%-8x: %u bytes sent.\n", assocID, length);
           fflush(stdout);
       }
-      length = 1 + (rand() % 512);
+      length = 1 + (random() % 512);
       memset(buffer, 'A', length);
       buffer[length-1] = '\n';
     }
-    ulpDataPtr                   = (struct ulp_data*)malloc(sizeof(struct ulp_data));
+    ulpDataPtr                   = malloc(sizeof(struct ulp_data));
     ulpDataPtr->maximumStreamID  = noOfOutStreams - 1;
     ulpDataPtr->ShutdownReceived = 0;
     return((void *) ulpDataPtr);
 }
 
 void* discardCommunicationUpNotif(unsigned int assocID, int status,
-                                  unsigned int noOfPaths,
+                                  unsigned int noOfDestinations,
                                   unsigned short noOfInStreams, unsigned short noOfOutStreams,
-                                  int associationSupportsPRSCTP, void* dummy)
-{
-    SCTP_PathStatus pathStatus;
-    unsigned int i;
-
-    if (verbose) {
-        fprintf(stdout, "%-8x: Communication up (%u paths)\n", assocID, noOfPaths);
+                                  int associationSupportsPRSCTP, int adLayerIndLen, void* adLayerInd, void* dummy)
+{	
+    if (verbose) {  
+        fprintf(stdout, "%-8x: Communication up (%u paths)\n", assocID, noOfDestinations);
         fflush(stdout);
     }
-
-    if (vverbose) {
-        for (i=0; i < noOfPaths; i++) {
-            SCTP_getPathStatus(assocID, (short)i, &pathStatus);
-            fprintf(stdout, "%-8x: Path Status of path %u (towards %s): %s.\n", assocID, i, pathStatus.destinationAddress, pathStateName(pathStatus.state));
-        }
-    }
-
     return NULL;
 }
 
-void communicationLostNotif(unsigned int assocID, unsigned short status, void* ulpDataPtr)
-{
+void communicationLostNotif(unsigned int assocID, int status, void* ulpDataPtr)
+{	
     unsigned char buffer[SCTP_MAXIMUM_DATA_LENGTH];
     unsigned int bufferLength;
     unsigned short streamID, streamSN;
@@ -436,7 +357,7 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
         fprintf(stdout, "%-8x: Communication lost (status %u)\n", assocID, status);
         fflush(stdout);
     }
-
+  
     /* retrieve data */
     bufferLength = sizeof(buffer);
     while (SCTP_receiveUnsent(assocID, buffer, &bufferLength, &tsn,
@@ -449,10 +370,10 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
         /* after that, reset bufferLength */
         bufferLength = sizeof(buffer);
     }
-
+    
     bufferLength = sizeof(buffer);
     while (SCTP_receiveUnacked(assocID, buffer, &bufferLength, &tsn,
-                               &streamID, &streamSN, &protoID) >= 0){
+			       &streamID, &streamSN, &protoID) >= 0){
         /* do something with the retrieved data */
         if (vverbose) {
             fprintf(stdout, "%-8x: Retrieved unacked chunk with %u bytes (SSN %u, SID %u)\n", assocID,bufferLength,streamSN,streamID );
@@ -461,17 +382,17 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
         /* after that, reset bufferLength */
         bufferLength = sizeof(buffer);
     }
-
+                          
     /* delete the association */
     if (ulpDataPtr)
         free((struct ulp_data *) ulpDataPtr);
-
+        
     SCTP_deleteAssociation(assocID);
 }
 
-void communicationErrorNotif(unsigned int assocID, unsigned short status, void* dummy)
+void communicationErrorNotif(unsigned int assocID, int status, void* dummy)
 {
-  if (verbose) {
+  if (verbose) {  
     fprintf(stdout, "%-8x: Communication error (status %u)\n", assocID, status);
     fflush(stdout);
   }
@@ -479,27 +400,27 @@ void communicationErrorNotif(unsigned int assocID, unsigned short status, void*
 
 void chargenRestartNotif(unsigned int assocID, void* ulpDataPtr)
 {
-    unsigned int length;
-    unsigned char buffer[BUFFER_LENGTH];
-
-    if (verbose) {
+    int length;
+    char buffer[BUFFER_LENGTH];
+   
+    if (verbose) {  
         fprintf(stdout, "%-8x: Restart\n", assocID);
         fflush(stdout);
     }
 
-    length = 1 + (rand() % 512);
-    memset((void *) buffer, 'A', length);
+    length = 1 + (random() % 512);
+    memset(buffer, 'A', length);
     buffer[length-1] = '\n';
-
+    
     while((!(((struct ulp_data *)ulpDataPtr)->ShutdownReceived)) &&
           (SCTP_send(assocID, 0, buffer, length, SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
-                     SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive,
+                     SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive, 
                      SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED) == SCTP_SUCCESS)) {
       if (vverbose) {
           fprintf(stdout, "%-8x: %u bytes sent.\n", assocID, length);
           fflush(stdout);
       }
-      length = 1 + (rand() % 512);
+      length = 1 + (random() % 512);
       memset(buffer, 'A', length);
       buffer[length-1] = '\n';
     }
@@ -508,21 +429,21 @@ void chargenRestartNotif(unsigned int assocID, void* ulpDataPtr)
 void echoRestartNotif(unsigned int assocID, void* ulpDataPtr)
 {
     SCTP_AssociationStatus assocStatus;
-
-    if (verbose) {
+    
+    if (verbose) {  
         fprintf(stdout, "%-8x: Restart\n", assocID);
         fflush(stdout);
     }
-
+    
     SCTP_getAssocStatus(assocID, &assocStatus);
-
+    
     /* update ULP data */
     ((struct ulp_data *) ulpDataPtr)->maximumStreamID = assocStatus.outStreams - 1;
 }
 
 void restartNotif(unsigned int assocID, void* ulpDataPtr)
-{
-    if (verbose) {
+{    
+    if (verbose) {  
         fprintf(stdout, "%-8x: Restart\n", assocID);
         fflush(stdout);
     }
@@ -530,18 +451,18 @@ void restartNotif(unsigned int assocID, void* ulpDataPtr)
 
 void shutdownCompleteNotif(unsigned int assocID, void* ulpDataPtr)
 {
-  if (verbose) {
+  if (verbose) {  
     fprintf(stdout, "%-8x: Shutdown complete\n", assocID);
     fflush(stdout);
   }
   if (ulpDataPtr)
     free((struct ulp_data *) ulpDataPtr);
-
+    
   SCTP_deleteAssociation(assocID);
 }
 
 void queueStatusChangeNotif(unsigned int assocID, int queueType, int queueID, int queueLength, void* ulpDataPtr)
-{
+{	
     if (vverbose) {
         fprintf(stdout, "%-8x: Queue status change notification: Type %d, ID %d, Length %d\n",
                         assocID, queueType, queueID, queueLength);
@@ -550,30 +471,30 @@ void queueStatusChangeNotif(unsigned int assocID, int queueType, int queueID, in
 }
 
 void chargenQueueStatusChangeNotif(unsigned int assocID, int queueType, int queueID, int queueLength, void* ulpDataPtr)
-{
-    unsigned int length;
-    unsigned char buffer[BUFFER_LENGTH];
+{	
+    int length;
+    char buffer[BUFFER_LENGTH];
 
     if (vverbose) {
         fprintf(stdout, "%-8x: Queue status change notification: Type %d, ID %d, Length %d\n",
                         assocID, queueType, queueID, queueLength);
         fflush(stdout);
     }
-
+    
     if (queueType == SCTP_SEND_QUEUE) {
-      length = 1 + (rand() % 512);
-      memset((void *)buffer, 'A', length);
+      length = 1 + (random() % 512);
+      memset(buffer, 'A', length);
       buffer[length-1] = '\n';
-
+      
       while((!(((struct ulp_data *)ulpDataPtr)->ShutdownReceived)) &&
             (SCTP_send(assocID, 0, buffer, length, SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
-                       SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive,
+                       SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive, 
                        SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED) == SCTP_SUCCESS)) {
         if (vverbose) {
             fprintf(stdout, "%-8x: %u bytes sent.\n", assocID, length);
             fflush(stdout);
         }
-        length = 1 + (rand() % 512);
+        length = 1 + (random() % 512);
         memset(buffer, 'A', length);
         buffer[length-1] = '\n';
       }
@@ -582,7 +503,7 @@ void chargenQueueStatusChangeNotif(unsigned int assocID, int queueType, int queu
 
 void shutdownReceivedNotif(unsigned int assocID, void* ulpDataPtr)
 {
-    if (verbose) {
+    if (verbose) {  
         fprintf(stdout, "%-8x: Shutdown received\n", assocID);
         fflush(stdout);
     }
@@ -595,7 +516,7 @@ int main(int argc, char **argv)
     SCTP_ulpCallbacks echoUlp, discardUlp, daytimeUlp, chargenUlp;
     SCTP_LibraryParameters params;
     SCTP_InstanceParameters instanceParameters;
-    int sctpInstance;
+    short sctpInstance;
 
     /* initialize the echo_ulp variable */
     echoUlp.dataArriveNotif              = &echoDataArriveNotif;
@@ -648,7 +569,7 @@ int main(int argc, char **argv)
     /* handle all command line options */
     getArgs(argc, argv);
     checkArgs();
-
+    
     SCTP_initLibrary();
     SCTP_getLibraryParameters(&params);
     params.sendOotbAborts    = sendOOTBAborts;
@@ -660,7 +581,7 @@ int main(int argc, char **argv)
                           MAXIMUM_NUMBER_OF_IN_STREAMS, MAXIMUM_NUMBER_OF_OUT_STREAMS,
                           noOfLocalAddresses, localAddressList,
                           echoUlp);
-
+    
     SCTP_registerInstance(DISCARD_PORT,
                           MAXIMUM_NUMBER_OF_IN_STREAMS, MAXIMUM_NUMBER_OF_OUT_STREAMS,
                           noOfLocalAddresses, localAddressList,
@@ -675,10 +596,10 @@ int main(int argc, char **argv)
                                          MAXIMUM_NUMBER_OF_IN_STREAMS, MAXIMUM_NUMBER_OF_OUT_STREAMS,
                                          noOfLocalAddresses, localAddressList,
                                          chargenUlp);
-
-    SCTP_getAssocDefaults((unsigned short)sctpInstance, &instanceParameters);
+    
+    SCTP_getAssocDefaults(sctpInstance, &instanceParameters);
     instanceParameters.maxSendQueue = SEND_QUEUE_SIZE;
-    SCTP_setAssocDefaults((unsigned short)sctpInstance, &instanceParameters);
+    SCTP_setAssocDefaults(sctpInstance, &instanceParameters);
 
     /* run the event handler forever */
     while (1) {
@@ -686,5 +607,6 @@ int main(int argc, char **argv)
     }
 
     /* this will never be reached */
-    return 0;
+    exit(0);
 }
+
