@@ -1,60 +1,44 @@
 /*
- * --------------------------------------------------------------------------
- *
- *           //=====   //===== ===//=== //===//  //       //   //===//
- *          //        //         //    //    // //       //   //    //
- *         //====//  //         //    //===//  //       //   //===<<
- *              //  //         //    //       //       //   //    //
- *       ======//  //=====    //    //       //=====  //   //===//
- *
- * -------------- An SCTP implementation according to RFC 4960 --------------
+ *  $Id: daytime_server.c,v 1.2 2004/01/13 16:13:14 ajung Exp $
  *
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
- * Copyright (C) 2001-2004 Andreas Jungmaier
- * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Acknowledgements:
- * Realized in co-operation between Siemens AG and the University of
- * Duisburg-Essen, Institute for Experimental Mathematics, Computer
- * Networking Technology group.
- * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany
- * (Förderkennzeichen 01AK045).
- * The authors alone are responsible for the contents.
+ * Realized in co-operation between Siemens AG
+ * and University of Essen, Institute of Computer Networking Technology.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * There are two mailinglists available at http://www.sctp.de which should be
+ * used for any discussion related to this implementation.
+ *
+ * Contact: discussion@sctp.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
+ *
+ * daytime_server.c  - main program module
  *
- * Contact: sctp-discussion@sctp.de
- *          thomas.dreibholz@gmail.com
- *          tuexen@fh-muenster.de
- *          andreas.jungmaier@web.de
  */
 
-#ifndef WIN32
 #include <unistd.h>
-#endif
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <time.h>
 
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include "sctp_wrapper.h"
+#include <sctp_wrapper.h>
 
 #define DAYTIME_PORT                         13
 #define MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES    10
@@ -71,34 +55,6 @@ static int unknownCommand = 0;
 static int sendOOTBAborts = 1;
 static int timeToLive     = SCTP_INFINITE_LIFETIME;
 
-const char *
-pathStateName(unsigned int state)
-{
-    switch (state) {
-        case SCTP_PATH_OK:
-            return "REACHABLE";
-            break;
-        case SCTP_PATH_UNREACHABLE:
-            return "UNREACHABLE";
-            break;
-        case SCTP_PATH_ADDED:
-            return "ADDED";
-            break;
-        case SCTP_PATH_REMOVED:
-            return "REMOVED";
-            break;
-        case SCTP_PATH_CONFIRMED:
-            return "CONFIRMED";
-            break;
-        case SCTP_PATH_UNCONFIRMED:
-            return "UNCONFIRMED";
-            break;
-        default:
-            return "UNKNOWN";
-            break;
-    }
-}
-
 void printUsage(void)
 {
     printf("Usage:   daytime_server [options]\n");
@@ -106,57 +62,46 @@ void printUsage(void)
     printf("-i       ignore OOTB packets\n");
     printf("-s       source address\n");
     printf("-t       time to live in ms\n");
-    printf("-v       verbose mode\n");
-    printf("-V       very verbose mode\n");
+    printf("-v       verbose mode\n");   
+    printf("-V       very verbose mode\n");   
 }
 
 void getArgs(int argc, char **argv)
 {
-    int i;
-    char *opt;
+    int c;
+    extern char *optarg;
+    extern int optind;
 
-    for(i=1; i < argc ;i++) {
-        if (argv[i][0] == '-') {
-            switch (argv[i][1]) {
-                case 'h':
-                    printUsage();
-                    exit(0);
-                case 'i':
-                    sendOOTBAborts = 0;
-                    break;
-                case 's':
-                    if (i+1 >= argc) {
-                        printUsage();
-				        exit(0);
-				    }
-				    opt = argv[++i];
-                    if ((noOfLocalAddresses < MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES) &&
-                        (strlen(opt) < SCTP_MAX_IP_LEN  )) {
-                        strcpy((char *)localAddressList[noOfLocalAddresses], opt);
-                        noOfLocalAddresses++;
-                    };
-                    break;
-                case 't':
-                    if (i+1 >= argc) {
-                        printUsage();
-				        exit(0);
-				    }
-				    opt = argv[++i];
-                    timeToLive = atoi(opt);
-                    break;
-                case 'v':
-                    verbose = 1;
-                    break;
-                case 'V':
-                    verbose = 1;
-                    vverbose = 1;
-                    break;
-                default:
-                    unknownCommand = 1;
-                    break;
-            }
-        } else
-			unknownCommand = 1;
+    while ((c = getopt(argc, argv, "his:t:vV")) != -1)
+    {
+        switch (c) {
+        case 'h':
+            printUsage();
+            exit(0);
+        case 'i':
+            sendOOTBAborts = 0;
+            break;
+        case 's':
+            if ((noOfLocalAddresses < MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES) &&
+                (strlen(optarg) < SCTP_MAX_IP_LEN  )) {
+                strcpy(localAddressList[noOfLocalAddresses], optarg);
+                noOfLocalAddresses++;
+            };
+            break;  
+        case 't':
+            timeToLive = atoi(optarg);
+            break;
+        case 'v':
+            verbose = 1;
+            break;
+        case 'V':
+            verbose = 1;
+            vverbose = 1;
+            break;
+        default:
+            unknownCommand = 1;
+            break;
+       }
     }
 }
 
@@ -164,15 +109,15 @@ void checkArgs(void)
 {
     int abortProgram;
     int printUsageInfo;
-
+    
     abortProgram = 0;
     printUsageInfo = 0;
-
+    
     if (noOfLocalAddresses == 0) {
 #ifdef HAVE_IPV6
-        strcpy((char *)localAddressList[noOfLocalAddresses], "::0");
+        strcpy(localAddressList[noOfLocalAddresses], "::0");
 #else
-        strcpy((char *)localAddressList[noOfLocalAddresses], "0.0.0.0");
+        strcpy(localAddressList[noOfLocalAddresses], "0.0.0.0");
 #endif
         noOfLocalAddresses++;
     }
@@ -180,24 +125,24 @@ void checkArgs(void)
          printf("Error:   Unkown options in command.\n");
          printUsageInfo = 1;
     }
-
+    
     if (printUsageInfo == 1)
         printUsage();
     if (abortProgram == 1)
         exit(-1);
 }
 
-void dataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int len,
+void dataArriveNotif(unsigned int assocID, unsigned int streamID, unsigned int len,
                      unsigned short streamSN, unsigned int TSN, unsigned int protoID,
                      unsigned int unordered, void* ulpDataPtr)
 {
     unsigned char chunk[MAXIMUM_PAYLOAD_LENGTH];
-    unsigned int length;
+    int length;
     unsigned int tsn;
     unsigned short ssn;
 
-
-    if (vverbose) {
+ 
+    if (vverbose) {  
       fprintf(stdout, "%-8x: Data arrived (%u bytes on stream %u, %s)\n",
                       assocID, len, streamID, (unordered==SCTP_ORDERED_DELIVERY)?"ordered":"unordered");
       fflush(stdout);
@@ -210,17 +155,28 @@ void dataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int
 void sendFailureNotif(unsigned int assocID,
                       unsigned char *unsent_data, unsigned int dataLength, unsigned int *context, void* dummy)
 {
-  if (verbose) {
+  if (verbose) {  
     fprintf(stdout, "%-8x: Send failure\n", assocID);
     fflush(stdout);
   }
 }
 
-void networkStatusChangeNotif(unsigned int assocID, short destAddrIndex, unsigned short newState, void* ulpDataPtr)
+void networkStatusChangeNotif(unsigned int assocID, unsigned int destAddrIndex, int newState, void* ulpDataPtr)
 {
-    if (verbose) {
-        fprintf(stdout, "%-8x: Network status change: path %u is now %s\n",
-                assocID, destAddrIndex, pathStateName(newState));
+    unsigned char status[40];
+
+    if (newState == SCTP_PATH_OK) strcpy(status, "ACTIVE STATE");
+    if (newState == SCTP_PATH_UNREACHABLE) strcpy(status,"INACTIVE");
+    if (newState == SCTP_PATH_ADDED) strcpy(status, "PATH ADDED");
+    if (newState == SCTP_PATH_REMOVED) strcpy(status, "PATH DELETED");
+    if (newState == SCTP_ASCONF_SUCCEEDED) strcpy(status, "ASCONF SUCCEEDED");
+    if (newState == SCTP_ASCONF_FAILED) strcpy(status, "ASCONF FAILED");
+    if (newState == SCTP_PATH_CONFIRMED) strcpy(status, "PATH CONFIRMED");
+    if (newState == SCTP_PATH_UNCONFIRMED) strcpy(status, "PATH UNCONFIRMED");
+
+    if (verbose) {  
+        fprintf(stdout, "%-8x: Network status change: path %u is now %s\n", 
+                assocID, destAddrIndex, status);
         fflush(stdout);
     }
 }
@@ -228,57 +184,57 @@ void networkStatusChangeNotif(unsigned int assocID, short destAddrIndex, unsigne
 void* communicationUpNotif(unsigned int assocID, int status,
                            unsigned int noOfDestinations,
                            unsigned short noOfInStreams, unsigned short noOfOutStreams,
-                           int associationSupportsPRSCTP, void* dummy)
-{
+                           int associationSupportsPRSCTP, int adLayerIndLen, void* adLayerInd, void* dummy)
+{	
     char *timeAsString;
     time_t now;
-
-    if (verbose) {
+ 
+    if (verbose) {  
         fprintf(stdout, "%-8x: Communication up (%u paths)\n", assocID, noOfDestinations);
         fflush(stdout);
     }
-
+   
     /* get the current time and convert to string */
     time(&now);
     timeAsString = ctime(&now);
 
-    if (vverbose) {
+    if (vverbose) {  
         fprintf(stdout, "%-8x: Current Time: %s", assocID, timeAsString);
         fflush(stdout);
     }
-
+   
     SCTP_send(assocID,
               0,
-              (unsigned char *)timeAsString, strlen(timeAsString),
+              timeAsString, strlen(timeAsString),
               SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
-              SCTP_USE_PRIMARY, SCTP_NO_CONTEXT,
+              SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, 
 	          timeToLive, SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED);
-
+    
     SCTP_shutdown(assocID);
 
     return NULL;
 }
 
-void communicationLostNotif(unsigned int assocID, unsigned short status, void* ulpDataPtr)
-{
+void communicationLostNotif(unsigned int assocID, int status, void* ulpDataPtr)
+{	
     if (verbose) {
-        fprintf(stdout, "%-8x: Communication lost (status %u)\n", assocID, status);
+        fprintf(stdout, "%-8x: Communication lost (status %d)\n", assocID, status);
         fflush(stdout);
     }
     SCTP_deleteAssociation(assocID);
 }
 
-void communicationErrorNotif(unsigned int assocID, unsigned short status, void* dummy)
+void communicationErrorNotif(unsigned int assocID, int status, void* dummy)
 {
-  if (verbose) {
-    fprintf(stdout, "%-8x: Communication error (status %u)\n", assocID, status);
+  if (verbose) {  
+    fprintf(stdout, "%-8x: Communication error (status %d)\n", assocID, status);
     fflush(stdout);
   }
 }
 
 void restartNotif(unsigned int assocID, void* ulpDataPtr)
-{
-    if (verbose) {
+{    
+    if (verbose) {  
         fprintf(stdout, "%-8x: Restart\n", assocID);
         fflush(stdout);
     }
@@ -286,7 +242,7 @@ void restartNotif(unsigned int assocID, void* ulpDataPtr)
 
 void shutdownCompleteNotif(unsigned int assocID, void* ulpDataPtr)
 {
-  if (verbose) {
+  if (verbose) {  
     fprintf(stdout, "%-8x: Shutdown complete\n", assocID);
     fflush(stdout);
   }
@@ -295,7 +251,7 @@ void shutdownCompleteNotif(unsigned int assocID, void* ulpDataPtr)
 
 void shutdownReceivedNotif(unsigned int assocID, void* ulpDataPtr)
 {
-    if (verbose) {
+    if (verbose) {  
         fprintf(stdout, "%-8x: Shutdown received\n", assocID);
         fflush(stdout);
     }
@@ -321,13 +277,13 @@ int main(int argc, char **argv)
     /* handle all command line options */
     getArgs(argc, argv);
     checkArgs();
-
+    
     SCTP_initLibrary();
     SCTP_getLibraryParameters(&params);
     params.sendOotbAborts    = sendOOTBAborts;
     params.checksumAlgorithm = SCTP_CHECKSUM_ALGORITHM_CRC32C;
     SCTP_setLibraryParameters(&params);
-
+    
     /* set up the "server" */
     SCTP_registerInstance(DAYTIME_PORT,
                           MAXIMUM_NUMBER_OF_IN_STREAMS, MAXIMUM_NUMBER_OF_OUT_STREAMS,
@@ -338,7 +294,8 @@ int main(int argc, char **argv)
     while (1) {
         SCTP_eventLoop();
     }
-
+    
     /* this will never be reached */
-    return 0;
+    exit(0);
 }
+
