@@ -1,69 +1,52 @@
 /*
- * --------------------------------------------------------------------------
- *
- *           //=====   //===== ===//=== //===//  //       //   //===//
- *          //        //         //    //    // //       //   //    //
- *         //====//  //         //    //===//  //       //   //===<<
- *              //  //         //    //       //       //   //    //
- *       ======//  //=====    //    //       //=====  //   //===//
- *
- * -------------- An SCTP implementation according to RFC 4960 --------------
+ *  $Id: echo_tool.c,v 1.4 2004/01/22 15:10:01 rohde Exp $
  *
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
- * Copyright (C) 2001-2004 Andreas Jungmaier
- * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Acknowledgements:
- * Realized in co-operation between Siemens AG and the University of
- * Duisburg-Essen, Institute for Experimental Mathematics, Computer
- * Networking Technology group.
- * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany
- * (Förderkennzeichen 01AK045).
- * The authors alone are responsible for the contents.
+ * Realized in co-operation between Siemens AG
+ * and University of Essen, Institute of Computer Networking Technology.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * There are two mailinglists available at http://www.sctp.de which should be
+ * used for any discussion related to this implementation.
+ *
+ * Contact: discussion@sctp.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
+ *
+ * echo_tool.c  - main program module
  *
- * Contact: sctp-discussion@sctp.de
- *          thomas.dreibholz@gmail.com
- *          tuexen@fh-muenster.de
- *          andreas.jungmaier@web.de
  */
 
 #include "sctp_wrapper.h"
 
-#ifndef WIN32
 #include <unistd.h>
-#endif
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>         /* for atoi() under Linux */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #define ECHO_PORT                             7
 #define MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES    10
 #define MAXIMUM_NUMBER_OF_ASSOCIATIONS        5
 #define MAXIMUM_NUMBER_OF_IN_STREAMS         17
 #define MAXIMUM_NUMBER_OF_OUT_STREAMS        17
-#define MAXIMUM_PAYLOAD_LENGTH             8192
+#define MAXIMUM_PAYLOAD_LENGTH             30192
 
-#ifndef min
 #define min(x,y)            (x)<(y)?(x):(y)
-#endif
 
 struct ulp_data {
     int maximumStreamID;
@@ -71,6 +54,8 @@ struct ulp_data {
     unsigned long nrOfReceivedChunks;
     unsigned long nrOfReceivedBytes;
     int ShutdownReceived;
+    unsigned short highestStream;
+    unsigned short lastStream;
 };
 
 static struct ulp_data ulpData[MAXIMUM_NUMBER_OF_ASSOCIATIONS];
@@ -89,224 +74,153 @@ static int verbose                            = 0;
 static int vverbose                           = 0;
 static int unknownCommand                     = 0;
 static unsigned int deltaT                    = 1000;
-static int rotateStreams                      = 0;
 static int sendToAll                          = 0;
+static int rotateStreams                      = 0;
 static int startAssociation                   = 0;
 static int sendUnordered                      = 0;
+static int sendUnorderedAndOrdered            = 0;
 static int sendOOTBAborts                     = 1;
 static int timeToLive                         = SCTP_INFINITE_LIFETIME;
 
-const char *
-pathStateName(unsigned int state)
-{
-    switch (state) {
-        case SCTP_PATH_OK:
-            return "REACHABLE";
-            break;
-        case SCTP_PATH_UNREACHABLE:
-            return "UNREACHABLE";
-            break;
-        case SCTP_PATH_ADDED:
-            return "ADDED";
-            break;
-        case SCTP_PATH_REMOVED:
-            return "REMOVED";
-            break;
-        case SCTP_PATH_CONFIRMED:
-            return "CONFIRMED";
-            break;
-        case SCTP_PATH_UNCONFIRMED:
-            return "UNCONFIRMED";
-            break;
-        default:
-            return "UNKNOWN";
-            break;
-    }
-}
-
 void printUsage(void)
 {
     printf("usage:   echo_tool [options]\n");
     printf("options:\n");
-    printf("-a                  use all streams in a round robin fashion when injecting packets\n");
     printf("-b                  send back incoming data on all existing associations\n");
-    printf("-d destination_addr establish a association with the specified address\n");
-    printf("-l length           number of bytes of the payload when generating traffic (default 512)\n");
+    printf("-c                  change stream on each send event\n");
+    printf("-d destination_addr establish a association with the specified address\n");   
+    printf("-l length           number of bytes of the payload when generating traffic (default 512)\n");   
     printf("-m                  print number of received bytes and chunks per period (see -p)\n");
-    printf("-M                  print number of received bytes, chunks per period (see -p) and flow control info\n");
-    printf("-n number           number of packets initially send out (default 0)\n");
-    printf("-p period           period for the measurements in milliseconds (default 1000)\n");
+    printf("-M                  print number of received bytes, chunks per period (see -p) and flow control info\n");    
+    printf("-n number           number of packets initially send out (default 0)\n");   
+    printf("-o local port       use this port as local port when setting up the assoc (default 7)\n");
+    printf("-p period           period for the measurements in milliseconds (default 1000)\n");   
     printf("-q byte             TOS byte used by all assiciations (default 0x10)\n");
     printf("-r port             remote port number (default echo port\n");
     printf("-s                  source address\n");
     printf("-t                  time to live in ms\n");
     printf("-i                  ignore OOTB packets\n");
     printf("-u                  inject the initial packets unordered\n");
-    printf("-v                  verbose mode\n");
-    printf("-V                  very verbose mode\n");
+    printf("-U                  inject the initial packets ordered and unordered\n");
+    printf("-v                  verbose mode\n");   
+    printf("-V                  very verbose mode\n");   
 }
 
 void getArgs(int argc, char **argv)
 {
-    int i;
-    char *opt;
+    int c;
+    extern char *optarg;
+    extern int optind;
 
-    for(i=1; i < argc ;i++) {
-        if (argv[i][0] == '-') {
-            switch (argv[i][1]) {
-                case 'a':
-                    rotateStreams = 1;
-                    break;
-                case 'b':
-                    sendToAll = 1;
-                    break;
-                case 'd':
-                    if (i+1 >= argc) {
-                        printUsage();
-                        exit(0);
-                    }
-                    opt = argv[++i];
-                    if (strlen(opt) < SCTP_MAX_IP_LEN) {
-                        strcpy((char *)destinationAddress, opt);
-                        startAssociation = 1;
-                    }
-                    break;
-                case 'l':
-                    if (i+1 >= argc) {
-                        printUsage();
-                        exit(0);
-                    }
-                    opt = argv[++i];
-                    chunkLength = min(atoi(opt),MAXIMUM_PAYLOAD_LENGTH);
-                    break;
-                case 'm':
-                    doMeasurements = 1;
-                    break;
-                case 'M':
-                    doMeasurements = 1;
-                    doAllMeasurements = 1;
-                    break;
-                case 'n':
-                    if (i+1 >= argc) {
-                        printUsage();
-                        exit(0);
-                    }
-                    opt = argv[++i];
-                    numberOfInitialPackets = atoi(opt);
-                    break;
-                case 'o':
-                    if (i+1 >= argc) {
-                        printUsage();
-                        exit(0);
-                    }
-                    opt = argv[++i];
-                    localPort = (unsigned short)atoi(opt);
-                    break;
-                case 'p':
-                    if (i+1 >= argc) {
-                        printUsage();
-                        exit(0);
-                    }
-                    opt = argv[++i];
-                    deltaT = atoi(opt);
-                    break;
-                case 'q':
-                    if (i+1 >= argc) {
-                        printUsage();
-                        exit(0);
-                    }
-                    opt = argv[++i];
-                    tosByte = (unsigned char) atoi(opt);
-                    break;
-                case 'r':
-                    if (i+1 >= argc) {
-                        printUsage();
-                        exit(0);
-                    }
-                    opt = argv[++i];
-                    remotePort =  atoi(opt);
-                    break;
-                case 's':
-                    if (i+1 >= argc) {
-                        printUsage();
-                        exit(0);
-                    }
-                    opt = argv[++i];
-                    if ((noOfLocalAddresses < MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES) &&
-                        (strlen(opt) < SCTP_MAX_IP_LEN  )) {
-                        strcpy((char *)localAddressList[noOfLocalAddresses], opt);
-                        noOfLocalAddresses++;
-                    }
-                    break;
-                case 't':
-                    if (i+1 >= argc) {
-                        printUsage();
-                        exit(0);
-                    }
-                    opt = argv[++i];
-                    timeToLive = atoi(opt);
-                    break;
-                case 'i':
-                    sendOOTBAborts = 0;
-                    break;
-                case 'u':
-                    sendUnordered = 1;
-                    break;
-                case 'v':
-                    verbose = 1;
-                    break;
-                case 'V':
-                    verbose = 1;
-                    vverbose = 1;
-                    break;
-                default:
-                    unknownCommand = 1;
-                    break;
+    while ((c = getopt(argc, argv, "bd:l:mMn:p:q:r:s:t:o:icuUvV")) != -1)
+    {
+        switch (c) {
+        case 'b':
+            sendToAll = 1;
+            break;
+        case 'c':
+            rotateStreams = 1;
+            break;
+        case 'd':
+            if (strlen(optarg) < SCTP_MAX_IP_LEN) {
+                strcpy(destinationAddress, optarg);
+                startAssociation = 1;
             }
-        } else
-          unknownCommand = 1;
+            break;
+        case 'l':
+            chunkLength = min(atoi(optarg),MAXIMUM_PAYLOAD_LENGTH);
+            break;
+        case 'm':
+            doMeasurements = 1;
+            break;
+        case 'M':
+            doMeasurements = 1;
+            doAllMeasurements = 1;
+            break;
+        case 'n':
+            numberOfInitialPackets = atoi(optarg);
+            break;
+        case 'o':
+            localPort = (unsigned short)atoi(optarg);
+            break;
+        case 'p':
+            deltaT = atoi(optarg);
+            break;
+        case 'q':
+            tosByte = (unsigned char) atoi(optarg);
+            break;
+        case 'r':
+            remotePort =  atoi(optarg);
+            break;
+        case 's':
+            if ((noOfLocalAddresses < MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES) &&
+                (strlen(optarg) < SCTP_MAX_IP_LEN  )) {
+                strcpy(localAddressList[noOfLocalAddresses], optarg);
+                noOfLocalAddresses++;
+            }
+            break;  
+        case 't':
+            timeToLive = atoi(optarg);
+            break;
+        case 'i':
+            sendOOTBAborts = 0;
+            break;
+        case 'u':
+            sendUnordered = 1;
+            break;
+        case 'U':
+            sendUnorderedAndOrdered = 1;
+            break;
+        case 'v':
+            verbose = 1;
+            break;
+        case 'V':
+            verbose = 1;
+            vverbose = 1;
+            break;
+        default:
+            unknownCommand = 1;
+            break;
+        }
     }
 }
-
 void checkArgs(void)
 {
     int abortProgram;
     int printUsageInfo;
-
+    
     abortProgram = 0;
     printUsageInfo = 0;
-
+    
     if (noOfLocalAddresses == 0) {
 #ifdef HAVE_IPV6
-        strcpy((char *)localAddressList[noOfLocalAddresses], "::0");
+        strcpy(localAddressList[noOfLocalAddresses], "::0");
 #else
-        strcpy((char *)localAddressList[noOfLocalAddresses], "0.0.0.0");
+        strcpy(localAddressList[noOfLocalAddresses], "0.0.0.0");
 #endif
         noOfLocalAddresses++;
     }
     if (unknownCommand ==1) {
          printf("Error:   Unkown options in command.\n");
          printUsageInfo = 1;
-         abortProgram = 1;
     }
-
+    
     if (printUsageInfo == 1)
         printUsage();
     if (abortProgram == 1)
         exit(-1);
 }
 
-void dataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int len,
+void dataArriveNotif(unsigned int assocID, unsigned int streamID, unsigned int len,
                      unsigned short streamSN,unsigned int TSN, unsigned int protoID,
                      unsigned int unordered, void* ulpDataPtr)
 {
     unsigned char chunk[MAXIMUM_PAYLOAD_LENGTH];
-    unsigned int length;
-    int index=0, result;
+    int length, index=0, result;
     unsigned short ssn;
     unsigned int the_tsn;
 
-    if (vverbose) {
+    if (vverbose) {  
       fprintf(stdout, "%-8x: Data arrived (%u bytes on stream %u, %s)\n",
                       assocID, len, streamID, (unordered==SCTP_ORDERED_DELIVERY)?"ordered":"unordered");
       fflush(stdout);
@@ -325,27 +239,28 @@ void dataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int
             if ((ulpData[index].maximumStreamID != -1)&&
                 (!(ulpData[index].ShutdownReceived))) {
                 result = SCTP_send(ulpData[index].assocID,
-                                   (unsigned short)min(streamID, (unsigned int)(ulpData[index].maximumStreamID)),
+                                   min(streamID, ulpData[index].maximumStreamID),
                                    chunk, length,
                                    protoID,
                                    SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive, unordered, SCTP_BUNDLING_DISABLED);
                 if (vverbose) {
                     fprintf(stdout, "%-8x: Data sent (%u bytes on stream %u, %s) Result: %d\n",
-                                    ulpData[index].assocID, len, min(streamID, (unsigned int)ulpData[index].maximumStreamID),
+                                    ulpData[index].assocID, len, min(streamID, ulpData[index].maximumStreamID),
                                     (unordered==SCTP_ORDERED_DELIVERY)?"ordered":"unordered", result);
                     fflush(stdout);
                 }
+
             }
         }
     } else {
         result = SCTP_send(assocID,
-                           (unsigned short)min(streamID, (unsigned int)(((struct ulp_data *) ulpDataPtr)->maximumStreamID)),
+                           min(streamID, ((struct ulp_data *) ulpDataPtr)->maximumStreamID),
                            chunk, length,
                            protoID,
                            SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive, unordered, SCTP_BUNDLING_DISABLED);
         if (vverbose) {
             fprintf(stdout, "%-8x: Data sent (%u bytes on stream %u, %s) Result: %d\n",
-                            ulpData[index].assocID, len, min(streamID, (unsigned int)(((struct ulp_data *) ulpDataPtr)->maximumStreamID)),
+                            ulpData[index].assocID, len, min(streamID, ((struct ulp_data *) ulpDataPtr)->maximumStreamID),
                             (unordered==SCTP_ORDERED_DELIVERY)?"ordered":"unordered", result);
             fflush(stdout);
         }
@@ -355,42 +270,53 @@ void dataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int
 void sendFailureNotif(unsigned int assocID,
                       unsigned char *unsent_data, unsigned int dataLength, unsigned int *context, void* dummy)
 {
-  if (verbose) {
+  if (verbose) {  
     fprintf(stdout, "%-8x: Send failure\n", assocID);
     fflush(stdout);
   }
 }
 
-void networkStatusChangeNotif(unsigned int assocID, short affectedPathID, unsigned short newState, void* ulpDataPtr)
+void networkStatusChangeNotif(unsigned int assocID, unsigned int affectedPathID, int newState, void* ulpDataPtr)
 {
     SCTP_AssociationStatus assocStatus;
     SCTP_PathStatus pathStatus;
-    unsigned short pathID;
+    unsigned int pathKey;
+    int pathIndex;
+    unsigned char status[40];
+
+    if (newState == SCTP_PATH_OK) strcpy(status, "ACTIVE STATE");
+    if (newState == SCTP_PATH_UNREACHABLE) strcpy(status,"INACTIVE");
+    if (newState == SCTP_PATH_ADDED) strcpy(status, "PATH ADDED");
+    if (newState == SCTP_PATH_REMOVED) strcpy(status, "PATH DELETED");
+    if (newState == SCTP_ASCONF_SUCCEEDED) strcpy(status, "ASCONF SUCCEEDED");
+    if (newState == SCTP_ASCONF_FAILED) strcpy(status, "ASCONF FAILED");
+    if (newState == SCTP_PATH_CONFIRMED) strcpy(status, "PATH CONFIRMED");
+    if (newState == SCTP_PATH_UNCONFIRMED) strcpy(status, "PATH UNCONFIRMED");    
 
     if (verbose) {
         SCTP_getPathStatus(assocID, affectedPathID, &pathStatus);
-        fprintf(stdout, "%-8x: Network status change: path %u (towards %s) is now %s\n",
+        fprintf(stdout, "%-8x: Network status change: path %u (towards %s) is now %s\n", 
                         assocID, affectedPathID,
                         pathStatus.destinationAddress,
-                        pathStateName(newState));
+                        status);
         fflush(stdout);
     }
-
+    
     /* if the primary path has become inactive */
-    if ((newState == SCTP_PATH_UNREACHABLE) &&
-        (affectedPathID == SCTP_getPrimary(assocID))) {
-
+    if ((newState == SCTP_PATH_UNREACHABLE) && (affectedPathID == SCTP_getPrimary(assocID))) {
+        
         /* select a new one */
         SCTP_getAssocStatus(assocID, &assocStatus);
-        for (pathID=0; pathID < assocStatus.numberOfAddresses; pathID++){
-            SCTP_getPathStatus(assocID, pathID, &pathStatus);
+        for (pathIndex=0; pathIndex < assocStatus.numberOfDestinationPaths; pathIndex++){
+            pathKey = assocStatus.destinationPathIDs[pathIndex];
+            SCTP_getPathStatus(assocID, pathKey, &pathStatus);
             if (pathStatus.state == SCTP_PATH_OK)
                 break;
         }
-
+        
         /* and use it */
-        if (pathID < assocStatus.numberOfAddresses) {
-            SCTP_setPrimary(assocID, pathID);
+        if (pathIndex < assocStatus.numberOfDestinationPaths) {
+            SCTP_setPrimary(assocID, pathKey);
         }
     }
 }
@@ -398,15 +324,14 @@ void networkStatusChangeNotif(unsigned int assocID, short affectedPathID, unsign
 void* communicationUpNotif(unsigned int assocID, int status,
                            unsigned int noOfPaths,
                            unsigned short noOfInStreams, unsigned short noOfOutStreams,
-                           int associationSupportsPRSCTP, void* dummy)
-{
+                           int associationSupportsPRSCTP, int adLayerIndLen, void* adLayerInd, void* dummy)
+{	
     unsigned int index, packetNumber;
     unsigned char chunk[MAXIMUM_PAYLOAD_LENGTH];
     SCTP_PathStatus pathStatus;
     unsigned short pathID;
-    unsigned short streamID = 0;
-
-    if (verbose) {
+     
+    if (verbose) {  
         fprintf(stdout, "%-8x: Communication up (%u paths:", assocID, noOfPaths);
         for (pathID=0; pathID < noOfPaths; pathID++){
             SCTP_getPathStatus(assocID, pathID, &pathStatus);
@@ -417,37 +342,44 @@ void* communicationUpNotif(unsigned int assocID, int status,
                 assocID, noOfInStreams, noOfOutStreams);
         fflush(stdout);
     }
-
+  
     /* look for a free ULP data */
     for (index=0; index < MAXIMUM_NUMBER_OF_ASSOCIATIONS; index++) {
         if (ulpData[index].maximumStreamID == -1)
             break;
     }
-
+   
     /* if found */
     if (index < MAXIMUM_NUMBER_OF_ASSOCIATIONS) {
         /* use it */
         ulpData[index].maximumStreamID    = noOfOutStreams - 1;
         ulpData[index].assocID            = assocID;
         ulpData[index].nrOfReceivedChunks = 0;
-        ulpData[index].nrOfReceivedBytes  = 0;
-        ulpData[index].ShutdownReceived   = 0;
+        ulpData[index].nrOfReceivedBytes  = 0; 
+        ulpData[index].ShutdownReceived   = 0;        
+        ulpData[index].highestStream      = noOfOutStreams;
+        ulpData[index].lastStream         = 0;
 
         /* send the initial packets */
         memset(chunk, 0, sizeof(chunk));
         for(packetNumber=1; packetNumber <= numberOfInitialPackets; packetNumber++) {
+            if(sendUnorderedAndOrdered)
+            {
+            }
             SCTP_send(assocID,
-                      streamID,
+                      ulpData[index].lastStream,
                       chunk, chunkLength,
                       SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
                       SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive,
-                      (sendUnordered)?SCTP_UNORDERED_DELIVERY:SCTP_ORDERED_DELIVERY,
+                      (sendUnordered)?SCTP_UNORDERED_DELIVERY:SCTP_ORDERED_DELIVERY, 
                       SCTP_BUNDLING_DISABLED);
-            if (rotateStreams) {
-                streamID = (streamID + 1) % noOfOutStreams;
+            if (rotateStreams){
+                ulpData[index].lastStream += 1;
+                ulpData[index].lastStream = ulpData[index].lastStream % ulpData[index].highestStream;
             }
+
         }
-        return &ulpData[index];
+        return &ulpData[index];       
     } else {
         /* abort assoc due to lack of resources */
         SCTP_abort(assocID);
@@ -455,8 +387,8 @@ void* communicationUpNotif(unsigned int assocID, int status,
     }
 }
 
-void communicationLostNotif(unsigned int assocID, unsigned short status, void* ulpDataPtr)
-{
+void communicationLostNotif(unsigned int assocID, int status, void* ulpDataPtr)
+{	
     unsigned char buffer[MAXIMUM_PAYLOAD_LENGTH];
     unsigned int bufferLength;
     unsigned short streamID, streamSN;
@@ -467,7 +399,7 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
         fprintf(stdout, "%-8x: Communication lost (status %u)\n", assocID, status);
         fflush(stdout);
     }
-
+  
     /* retrieve data */
     bufferLength = sizeof(buffer);
     while (SCTP_receiveUnsent(assocID, buffer, &bufferLength, &tsn,
@@ -476,25 +408,25 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
         /* after that, reset bufferLength */
         bufferLength = sizeof(buffer);
     }
-
+    
     bufferLength = sizeof(buffer);
     while (SCTP_receiveUnacked(assocID, buffer, &bufferLength, &tsn,
-                                   &streamID, &streamSN, &protoID) >= 0){
+			       &streamID, &streamSN, &protoID) >= 0){
         /* do something with the retrieved data */
         /* after that, reset bufferLength */
         bufferLength = sizeof(buffer);
     }
-
+                      
     /* free ULP data */
     ((struct ulp_data *) ulpDataPtr)->maximumStreamID = -1;
-
+    
     /* delete the association */
     SCTP_deleteAssociation(assocID);
 }
 
-void communicationErrorNotif(unsigned int assocID, unsigned short status, void* dummy)
+void communicationErrorNotif(unsigned int assocID, int status, void* dummy)
 {
-  if (verbose) {
+  if (verbose) {  
     fprintf(stdout, "%-8x: Communication error (status %u)\n", assocID, status);
     fflush(stdout);
   }
@@ -503,14 +435,14 @@ void communicationErrorNotif(unsigned int assocID, unsigned short status, void*
 void restartNotif(unsigned int assocID, void* ulpDataPtr)
 {
     SCTP_AssociationStatus assocStatus;
-
-    if (verbose) {
+    
+    if (verbose) {  
         fprintf(stdout, "%-8x: Restart\n", assocID);
         fflush(stdout);
     }
-
+    
     SCTP_getAssocStatus(assocID, &assocStatus);
-
+    
     /* update ULP data */
     ((struct ulp_data *) ulpDataPtr)->maximumStreamID = assocStatus.outStreams - 1;
     ((struct ulp_data *) ulpDataPtr)->assocID         = assocID;
@@ -519,7 +451,7 @@ void restartNotif(unsigned int assocID, void* ulpDataPtr)
 
 void shutdownCompleteNotif(unsigned int assocID, void* ulpDataPtr)
 {
-  if (verbose) {
+  if (verbose) {  
     fprintf(stdout, "%-8x: Shutdown complete\n", assocID);
     fflush(stdout);
   }
@@ -531,7 +463,7 @@ void shutdownCompleteNotif(unsigned int assocID, void* ulpDataPtr)
 
 void shutdownReceivedNotif(unsigned int assocID, void* ulpDataPtr)
 {
-    if (verbose) {
+    if (verbose) {  
         fprintf(stdout, "%-8x: Shutdown received\n", assocID);
         fflush(stdout);
     }
@@ -546,17 +478,19 @@ measurementTimerRunOffFunction(unsigned int timerID, void *parameter1, void *par
     int index;
     SCTP_AssociationStatus assocStatus;
     SCTP_PathStatus pathStatus;
-    unsigned short pathID;
+    unsigned int pathKey;
+    int pathIndex;
 
     for (index=0; index < MAXIMUM_NUMBER_OF_ASSOCIATIONS; index++) {
         if (ulpData[index].maximumStreamID >= 0){
             if (doAllMeasurements) {
                 SCTP_getAssocStatus(ulpData[index].assocID, &assocStatus);
-                for (pathID=0; pathID < assocStatus.numberOfAddresses; pathID++){
-                    SCTP_getPathStatus(ulpData[index].assocID, pathID, &pathStatus);
+                for (pathIndex=0; pathIndex < assocStatus.numberOfDestinationPaths; pathIndex++){
+                    pathKey = assocStatus.destinationPathIDs[pathIndex];
+                    SCTP_getPathStatus(ulpData[index].assocID, pathKey, &pathStatus);
                     fprintf(stdout, "Asoc:%-8x Path:%-2u Ch:%-8lu By:%-8lu rto:%-8u srtt:%-8u qu:%-6u osb:%-8u cwnd:%-8u ssthresh:%-8u\n",
                         ulpData[index].assocID,
-                        pathID,
+                        pathKey,
                         ulpData[index].nrOfReceivedChunks,
                         ulpData[index].nrOfReceivedBytes,
                         pathStatus.rto,
@@ -579,18 +513,18 @@ measurementTimerRunOffFunction(unsigned int timerID, void *parameter1, void *par
 
 int main(int argc, char **argv)
 {
-    int sctpInstance;
+    unsigned short sctpInstance;
     SCTP_ulpCallbacks echoUlp;
     SCTP_InstanceParameters instanceParameters;
     SCTP_LibraryParameters params;
     unsigned int index;
-
+    
     /* initialize ULP data */
     for (index=0; index < MAXIMUM_NUMBER_OF_ASSOCIATIONS; index++) {
         ulpData[index].maximumStreamID    = -1;
         ulpData[index].nrOfReceivedChunks = 0;
-        ulpData[index].nrOfReceivedBytes  = 0;
-        ulpData[index].ShutdownReceived   = 0;
+        ulpData[index].nrOfReceivedBytes  = 0; 
+        ulpData[index].ShutdownReceived   = 0;        
     }
 
     /* initialize the echo_ulp variable */
@@ -621,24 +555,25 @@ int main(int argc, char **argv)
 
     /* set the TOS field */
 
-    SCTP_getAssocDefaults((unsigned short)sctpInstance, &instanceParameters);
+    SCTP_getAssocDefaults(sctpInstance, &instanceParameters);
     instanceParameters.maxSendQueue = 0;
     instanceParameters.ipTos=tosByte;
-    SCTP_setAssocDefaults((unsigned short)sctpInstance, &instanceParameters);
-
+    SCTP_setAssocDefaults(sctpInstance, &instanceParameters);
+    
     if (startAssociation) {
-        SCTP_associate((unsigned short)sctpInstance, MAXIMUM_NUMBER_OF_OUT_STREAMS, destinationAddress, remotePort, &ulpData[0]);
+        SCTP_associate(sctpInstance, MAXIMUM_NUMBER_OF_OUT_STREAMS, destinationAddress, remotePort, &ulpData[0]);
     }
-
+    
     if (doMeasurements) {
         SCTP_startTimer(deltaT, &measurementTimerRunOffFunction, NULL, NULL);
     }
-
+    
     /* run the event handler forever */
     while (1) {
         SCTP_eventLoop();
     };
 
     /* this will never be reached */
-    return 0;
+    exit(0);
 }
+
