@@ -1,44 +1,35 @@
 /*
- * --------------------------------------------------------------------------
- *
- *           //=====   //===== ===//=== //===//  //       //   //===//
- *          //        //         //    //    // //       //   //    //
- *         //====//  //         //    //===//  //       //   //===<<
- *              //  //         //    //       //       //   //    //
- *       ======//  //=====    //    //       //=====  //   //===//
- *
- * -------------- An SCTP implementation according to RFC 4960 --------------
+ *  $Id: localcom.c,v 1.4 2004/01/13 16:13:14 ajung Exp $
  *
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
- * Copyright (C) 2001-2004 Andreas Jungmaier
- * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Acknowledgements:
- * Realized in co-operation between Siemens AG and the University of
- * Duisburg-Essen, Institute for Experimental Mathematics, Computer
- * Networking Technology group.
- * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany
- * (Förderkennzeichen 01AK045).
- * The authors alone are responsible for the contents.
+ * Realized in co-operation between Siemens AG
+ * and University of Essen, Institute of Computer Networking Technology.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * There are two mailinglists available at http://www.sctp.de which should be
+ * used for any discussion related to this implementation.
+ *
+ * Contact: discussion@sctp.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
+ *
+ * localcom.c  - main program that tests two instances on localhost
  *
- * Contact: sctp-discussion@sctp.de
- *          thomas.dreibholz@gmail.com
- *          tuexen@fh-muenster.de
- *          andreas.jungmaier@web.de
  */
 
 #include "sctp_wrapper.h"
@@ -55,9 +46,7 @@
 #define MAXIMUM_NUMBER_OF_OUT_STREAMS        17
 #define MAXIMUM_PAYLOAD_LENGTH             8192
 
-#ifndef min
 #define min(x,y)            (x)<(y)?(x):(y)
-#endif
 
 struct ulp_data {
     int maximumStreamID;
@@ -82,40 +71,12 @@ static int useDumpFile                        = 0;
 
 FILE* fptr;
 
-const char *
-pathStateName(unsigned int state)
-{
-    switch (state) {
-        case SCTP_PATH_OK:
-            return "REACHABLE";
-            break;
-        case SCTP_PATH_UNREACHABLE:
-            return "UNREACHABLE";
-            break;
-        case SCTP_PATH_ADDED:
-            return "ADDED";
-            break;
-        case SCTP_PATH_REMOVED:
-            return "REMOVED";
-            break;
-        case SCTP_PATH_CONFIRMED:
-            return "CONFIRMED";
-            break;
-        case SCTP_PATH_UNCONFIRMED:
-            return "UNCONFIRMED";
-            break;
-        default:
-            return "UNKNOWN";
-            break;
-    }
-}
-
 void printUsage(void)
 {
     printf("usage:   local_communication [options] \n");
     printf("options:\n");
-    printf("-l length           number of bytes of the payload when generating traffic (default 512)\n");
-    printf("-n number           number of packets initially send out (default 1)\n");
+    printf("-l length           number of bytes of the payload when generating traffic (default 512)\n");   
+    printf("-n number           number of packets initially send out (default 1)\n");   
     printf("-u                  use unordered delivery\n");
     printf("-e number           end after that many chunks\n");
 }
@@ -154,24 +115,24 @@ void getArgs(int argc, char **argv)
 void checkArgs(void)
 {
     int printUsageInfo;
-
+    
     printUsageInfo = 0;
-
+    
     if (unknownCommand ==1) {
          printf("Error:   Unkown options in command.\n");
          printUsageInfo = 1;
     }
-
+    
     if (printUsageInfo == 1)
         printUsage();
 }
 
-void serverDataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int len,
+void serverDataArriveNotif(unsigned int assocID, unsigned int streamID, unsigned int len,
                            unsigned short streamSN,unsigned int TSN, unsigned int protoID,
                            unsigned int unordered, void* ulpDataPtr)
 {
     unsigned char chunk[MAXIMUM_PAYLOAD_LENGTH];
-    unsigned int length;
+    int length, result;
     unsigned short ssn;
     unsigned int the_tsn;
 
@@ -182,32 +143,33 @@ void serverDataArriveNotif(unsigned int assocID, unsigned short streamID, unsign
     /* update counter */
     ((struct ulp_data *) ulpDataPtr)->nrOfReceivedChunks += 1;
     ((struct ulp_data *) ulpDataPtr)->nrOfReceivedBytes  += length;
-
+    
     /* and send it */
-    SCTP_send(assocID,
-              (unsigned short)min(streamID, ((struct ulp_data *) ulpDataPtr)->maximumStreamID),
-              chunk, length,
-              protoID,
-              SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, SCTP_INFINITE_LIFETIME, unordered, SCTP_BUNDLING_DISABLED);
-}
+    result = SCTP_send(assocID,
+                       min(streamID, ((struct ulp_data *) ulpDataPtr)->maximumStreamID),
+                       chunk, length,
+                       protoID,
+//                       SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, SCTP_INFINITE_LIFETIME, unordered, SCTP_BUNDLING_DISABLED);
+                       SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, SCTP_NO_RETRANSMISSION, unordered, SCTP_BUNDLING_DISABLED);}
 
-void clientDataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int len,
+void clientDataArriveNotif(unsigned int assocID, unsigned int streamID, unsigned int len,
                            unsigned short streamSN,unsigned int TSN, unsigned int protoID,
                            unsigned int unordered, void* ulpDataPtr)
 {
     unsigned char chunk[MAXIMUM_PAYLOAD_LENGTH];
-    unsigned int length;
+    int length,  result;
     unsigned short ssn;
     unsigned int the_tsn;
 
     chunkCount++;
     length = sizeof(chunk);
     SCTP_receive(assocID, streamID, chunk, &length, &ssn, &the_tsn, SCTP_MSG_DEFAULT);
-    SCTP_send(assocID,
-              streamID,
-              chunk, length,
-              protoID,
-              SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, SCTP_INFINITE_LIFETIME, unordered, SCTP_BUNDLING_DISABLED);
+    result = SCTP_send(assocID,
+                       streamID,
+                       chunk, length,
+                       protoID,
+                       SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, SCTP_NO_RETRANSMISSION, unordered, SCTP_BUNDLING_DISABLED);
+//                       SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, SCTP_INFINITE_LIFETIME, unordered, SCTP_BUNDLING_DISABLED);
 
     if (chunkCount > ende)     SCTP_shutdown(assocID);
 }
@@ -219,34 +181,44 @@ void sendFailureNotif(unsigned int assocID,
     fflush(fptr);
 }
 
-void networkStatusChangeNotif(unsigned int assocID, short affectedPathID, unsigned short newState, void* ulpDataPtr)
+void networkStatusChangeNotif(unsigned int assocID, unsigned int affectedPathID, int newState, void* ulpDataPtr)
 {
     SCTP_AssociationStatus assocStatus;
     SCTP_PathStatus pathStatus;
-    unsigned short pathID;
+    unsigned int pathKey;
+    int pathIndex;
+    unsigned char status[40];
 
+    if (newState == SCTP_PATH_OK) strcpy(status, "ACTIVE STATE");
+    if (newState == SCTP_PATH_UNREACHABLE) strcpy(status,"INACTIVE");
+    if (newState == SCTP_PATH_ADDED) strcpy(status, "PATH ADDED");
+    if (newState == SCTP_PATH_REMOVED) strcpy(status, "PATH DELETED");
+    if (newState == SCTP_ASCONF_SUCCEEDED) strcpy(status, "ASCONF SUCCEEDED");
+    if (newState == SCTP_ASCONF_FAILED) strcpy(status, "ASCONF FAILED");
+    if (newState == SCTP_PATH_CONFIRMED) strcpy(status, "PATH CONFIRMED");
+    if (newState == SCTP_PATH_UNCONFIRMED) strcpy(status, "PATH UNCONFIRMED");
+    
     SCTP_getPathStatus(assocID, affectedPathID, &pathStatus);
     fprintf(fptr, "%-8x: Network status change: path %u (towards %s) is now %s\n",
                     assocID, affectedPathID,
                     pathStatus.destinationAddress,
-                    pathStateName(newState));
+                    status);
     fflush(fptr);
-
+    
     /* if the primary path has become inactive */
-    if ((newState == SCTP_PATH_UNREACHABLE) &&
-        (affectedPathID == SCTP_getPrimary(assocID))) {
-
+    if ((newState == SCTP_PATH_UNREACHABLE) && (affectedPathID == SCTP_getPrimary(assocID))) {
         /* select a new one */
         SCTP_getAssocStatus(assocID, &assocStatus);
-        for (pathID=0; pathID < assocStatus.numberOfAddresses; pathID++){
-            SCTP_getPathStatus(assocID, pathID, &pathStatus);
+        for (pathIndex=0; pathIndex < assocStatus.numberOfDestinationPaths; pathIndex++){
+            pathKey = assocStatus.destinationPathIDs[pathIndex];
+            SCTP_getPathStatus(assocID, pathKey, &pathStatus);
             if (pathStatus.state == SCTP_PATH_OK)
                 break;
         }
-
+        
         /* and use it */
-        if (pathID < assocStatus.numberOfAddresses) {
-            SCTP_setPrimary(assocID, pathID);
+        if (pathIndex < assocStatus.numberOfDestinationPaths) {
+            SCTP_setPrimary(assocID, pathKey);
         }
     }
 }
@@ -254,11 +226,11 @@ void networkStatusChangeNotif(unsigned int assocID, short affectedPathID, unsign
 void* clientCommunicationUpNotif(unsigned int assocID, int status,
                            unsigned int noOfPaths,
                            unsigned short noOfInStreams, unsigned short noOfOutStreams,
-                           int associationSupportsPRSCTP, void* dummy)
-{
+                           int associationSupportsPRSCTP, int adLayerIndLen, void* adLayerInd, void* dummy)
+{	
     unsigned int  packetNumber;
     unsigned char chunk[MAXIMUM_PAYLOAD_LENGTH];
-
+       
     /* send the initial packets */
     memset(chunk, 0xFF, sizeof(chunk));
     for(packetNumber=1; packetNumber <= numberOfInitialPackets; packetNumber++) {
@@ -267,31 +239,31 @@ void* clientCommunicationUpNotif(unsigned int assocID, int status,
                   chunk, chunkLength,
                   SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
                   SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, SCTP_INFINITE_LIFETIME,
-                  (sendUnordered)?SCTP_UNORDERED_DELIVERY:SCTP_ORDERED_DELIVERY,
+                  (sendUnordered)?SCTP_UNORDERED_DELIVERY:SCTP_ORDERED_DELIVERY, 
                   SCTP_BUNDLING_DISABLED);
     }
-    return NULL;
+    return NULL;       
 }
 
 void* serverCommunicationUpNotif(unsigned int assocID, int status,
                                  unsigned int noOfPaths,
                                  unsigned short noOfInStreams, unsigned short noOfOutStreams,
-                                 int associationSupportsPRSCTP, void* dummy)
-{
+                                 int associationSupportsPRSCTP, int adLayerIndLen, void* adLayerInd, void* dummy)
+{	
     int index;
-
+       
     /* look for a free ULP data */
     for (index=0; index < MAXIMUM_NUMBER_OF_ASSOCIATIONS; index++) {
         if (ulpData[index].maximumStreamID == -1)
             break;
     }
-
+   
     /* if found */
     if (index < MAXIMUM_NUMBER_OF_ASSOCIATIONS) {
         /* use it */
         ulpData[index].maximumStreamID = noOfOutStreams - 1;
         ulpData[index].assocID = assocID;
-        return &ulpData[index];
+        return &ulpData[index];       
     } else {
         /* abort assoc due to lack of resources */
         SCTP_abort(assocID);
@@ -299,8 +271,8 @@ void* serverCommunicationUpNotif(unsigned int assocID, int status,
     }
 }
 
-void communicationLostNotif(unsigned int assocID, unsigned short status, void* ulpDataPtr)
-{
+void communicationLostNotif(unsigned int assocID, int status, void* ulpDataPtr)
+{	
     unsigned char buffer[MAXIMUM_PAYLOAD_LENGTH];
     unsigned int bufferLength;
     unsigned short streamID, streamSN;
@@ -309,7 +281,7 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
 
     fprintf(fptr, "%-8x: Communication lost (status %u)\n", assocID, status);
     fflush(fptr);
-
+  
     /* retrieve data */
     bufferLength = sizeof(buffer);
     while (SCTP_receiveUnsent(assocID, buffer, &bufferLength, &tsn,
@@ -318,7 +290,7 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
         /* after that, reset bufferLength */
         bufferLength = sizeof(buffer);
     }
-
+    
     bufferLength = sizeof(buffer);
     while (SCTP_receiveUnacked(assocID, buffer, &bufferLength, &tsn,
 			       &streamID, &streamSN, &protoID) >= 0){
@@ -326,7 +298,7 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
         /* after that, reset bufferLength */
         bufferLength = sizeof(buffer);
     }
-
+                      
     /* free ULP data */
     if (ulpDataPtr) {
         ((struct ulp_data *) ulpDataPtr)->maximumStreamID = -1;
@@ -336,7 +308,7 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
     exit(0);
 }
 
-void communicationErrorNotif(unsigned int assocID, unsigned short status, void* dummy)
+void communicationErrorNotif(unsigned int assocID, int status, void* dummy)
 {
     fprintf(fptr, "%-8x: Communication error (status %u)\n", assocID, status);
     fflush(fptr);
@@ -345,10 +317,10 @@ void communicationErrorNotif(unsigned int assocID, unsigned short status, void*
 void restartNotif(unsigned int assocID, void* ulpDataPtr)
 {
     SCTP_AssociationStatus assocStatus;
-
+    
     fprintf(fptr, "%-8x: Restart\n", assocID);
     fflush(fptr);
-
+    
     /* update ULP data */
     if (ulpDataPtr) {
         SCTP_getAssocStatus(assocID, &assocStatus);
@@ -376,20 +348,24 @@ measurementTimerRunOffFunction(unsigned int timerID, void *parameter1, void *par
     int index;
     SCTP_AssociationStatus assocStatus;
     SCTP_PathStatus pathStatus;
+    unsigned int pathKey;
 
     for (index=0; index < MAXIMUM_NUMBER_OF_ASSOCIATIONS; index++) {
         if (ulpData[index].maximumStreamID >= 0){
             SCTP_getAssocStatus(ulpData[index].assocID, &assocStatus);
-            SCTP_getPathStatus(ulpData[index].assocID, 0, &pathStatus);
-            fprintf(fptr, "%-8x %-8lu %-8lu %-8u %-8u %-8u %-8u %-8u\n",
+            pathKey = assocStatus.destinationPathIDs[0];
+            SCTP_getPathStatus(ulpData[index].assocID, pathKey, &pathStatus);
+            fprintf(fptr, "%-7x %-7lu %-7lu %-7u %-7u o:%-7u obpa:%-7u c:%-7u s:%-7u pb%-7u\n",
                     ulpData[index].assocID,
                     ulpData[index].nrOfReceivedChunks,
                     ulpData[index].nrOfReceivedBytes,
                     assocStatus.currentReceiverWindowSize,
                     assocStatus.noOfChunksInSendQueue,
                     assocStatus.outstandingBytes,
+                    pathStatus.outstandingBytesPerAddress,
                     pathStatus.cwnd,
-                    pathStatus.ssthresh);
+                    pathStatus.ssthresh,
+                    pathStatus.partialBytesAcked);
             fflush(fptr);
             ulpData[index].nrOfReceivedChunks = 0;
             ulpData[index].nrOfReceivedBytes  = 0;
@@ -402,17 +378,17 @@ measurementTimerRunOffFunction(unsigned int timerID, void *parameter1, void *par
 
 int main(int argc, char **argv)
 {
-    int sctpClientInstance;
+    unsigned short sctpServerInstance, sctpClientInstance;
     SCTP_ulpCallbacks echoServerUlp, echoClientUlp;
     SCTP_LibraryParameters params;
-    unsigned int index, version;
+    unsigned int index;
 
 
     /* initialize ULP data */
     for (index=0; index < MAXIMUM_NUMBER_OF_ASSOCIATIONS; index++) {
         ulpData[index].maximumStreamID    = -1;
         ulpData[index].nrOfReceivedChunks = 0;
-        ulpData[index].nrOfReceivedBytes  = 0;
+        ulpData[index].nrOfReceivedBytes  = 0;        
     }
 
     /* initialize the echo_ulp variable */
@@ -440,44 +416,41 @@ int main(int argc, char **argv)
     getArgs(argc, argv);
     checkArgs();
 
-    version = sctp_getLibraryVersion();
-    fprintf(fptr, "We are using sctplib version (%u) maj=%u, min=%u\n",
-        version, version & 0xFFFF0000, version & 0xFFFF);
-    fflush(fptr);
-
     SCTP_initLibrary();
     SCTP_getLibraryParameters(&params);
     params.sendOotbAborts = 0;
     params.checksumAlgorithm = SCTP_CHECKSUM_ALGORITHM_CRC32C;
     SCTP_setLibraryParameters(&params);
 
-
+    
     noOfLocalAddresses = 1;
-    strcpy((char *)localAddressList[0], "127.0.0.1");
-
-    SCTP_registerInstance(ECHO_PORT,
-                          MAXIMUM_NUMBER_OF_IN_STREAMS, MAXIMUM_NUMBER_OF_OUT_STREAMS,
-                          noOfLocalAddresses, localAddressList,
-                          echoServerUlp);
+    strcpy(localAddressList[0], "127.0.0.1");
+    
+    sctpServerInstance = SCTP_registerInstance(ECHO_PORT,
+                                               MAXIMUM_NUMBER_OF_IN_STREAMS, MAXIMUM_NUMBER_OF_OUT_STREAMS,
+                                               noOfLocalAddresses, localAddressList,
+                                               echoServerUlp);
 
     sctpClientInstance = SCTP_registerInstance(CLIENT_PORT,
                                                MAXIMUM_NUMBER_OF_IN_STREAMS, MAXIMUM_NUMBER_OF_OUT_STREAMS,
                                                noOfLocalAddresses, localAddressList,
                                                echoClientUlp);
 
-    strcpy((char *)destinationAddress, "127.0.0.1");
+    strcpy(destinationAddress, "127.0.0.1");    
 
 
     SCTP_associate(sctpClientInstance, 1, destinationAddress, ECHO_PORT, NULL);
 
     SCTP_startTimer(deltaT, &measurementTimerRunOffFunction, NULL, NULL);
 
-
+    
     /* run the event handler forever */
     while (1) {
         SCTP_eventLoop();
     };
 
     /* this will never be reached */
-    return 0;
+    exit(0);
 }
+
+
