@@ -1,44 +1,35 @@
 /*
- * --------------------------------------------------------------------------
- *
- *           //=====   //===== ===//=== //===//  //       //   //===//
- *          //        //         //    //    // //       //   //    //
- *         //====//  //         //    //===//  //       //   //===<<
- *              //  //         //    //       //       //   //    //
- *       ======//  //=====    //    //       //=====  //   //===//
- *
- * -------------- An SCTP implementation according to RFC 4960 --------------
+ *  $Id: main.c,v 1.1.1.1 2002/11/20 14:51:17 ajung Exp $
  *
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
- * Copyright (C) 2001-2004 Andreas Jungmaier
- * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Acknowledgements:
- * Realized in co-operation between Siemens AG and the University of
- * Duisburg-Essen, Institute for Experimental Mathematics, Computer
- * Networking Technology group.
- * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany
- * (Förderkennzeichen 01AK045).
- * The authors alone are responsible for the contents.
+ * Realized in co-operation between Siemens AG
+ * and University of Essen, Institute of Computer Networking Technology.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * There are two mailinglists available at http://www.sctp.de which should be
+ * used for any discussion related to this implementation.
+ *
+ * Contact: discussion@sctp.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
+ *
+ * main.c  - main program module
  *
- * Contact: sctp-discussion@sctp.de
- *          thomas.dreibholz@gmail.com
- *          tuexen@fh-muenster.de
- *          andreas.jungmaier@web.de
  */
 
 #ifdef HAVE_CONFIG_H
@@ -52,15 +43,10 @@
 #include <string.h>
 #include <syslog.h>
 
-#ifdef HAVE_SYS_POLL_H
-    #include <sys/poll.h>
-#else
-    #define POLLIN     0x001
-    #define POLLPRI    0x002
-    #define POLLOUT    0x004
-    #define POLLERR    0x008
-#endif
-
+#define POLLIN     0x001
+#define POLLPRI    0x002
+#define POLLOUT    0x004
+#define POLLERR    0x008
 
 #include "sctp.h"
 #include "mini-ulp.h"
@@ -83,12 +69,12 @@ unsigned short localNumberOfInStreams = 15; /* Our default number of streams.
                                                imports this in order to limit number of
                                                in-streams a-side requests. */
 static gboolean streamRobin = FALSE; /* By default all data are transmitted over one stream */
-gboolean use_unordered = FALSE;
+gboolean unordered = FALSE;
 
 
 /* this one is global, as we want to know, whether to enable ipv6 calls, even if it may be compiled in */
 gboolean support_ipv6;
-int sctp_instance,sctp_instance2,sctp_instance3;
+gushort sctp_instance,sctp_instance2,sctp_instance3;
 unsigned short ulpPort = 0;
 
 
@@ -375,7 +361,7 @@ void getArgs(int argc, char **argv)
         useLocalPort = FALSE;
 
     if (vflg) {
-        use_unordered = TRUE;
+        unordered = TRUE;
         printf("some chunks are transmitted unordered\n");
     }
 
@@ -388,7 +374,7 @@ void getArgs(int argc, char **argv)
         while ((tokstr = strtok(tokens_str, ",")) != NULL) {
             tokens_str = NULL;
             printf("sourceaddresses separated: %s\n", tokstr);
-            strncpy((char *)sourceAddrs[noOfsourceAddrs], tokstr, 16);
+            strncpy(sourceAddrs[noOfsourceAddrs], tokstr, 16);
             noOfsourceAddrs++;
         }
     }
@@ -400,7 +386,7 @@ void getArgs(int argc, char **argv)
         while ((tokstr = strtok(tokens_str, ",")) != NULL) {
             tokens_str = NULL;
             printf("IPv6 sourceaddresses separated: %s\n", tokstr);
-            strncpy((char *)sourceAddrs[noOfsourceAddrs], tokstr, SCTP_MAX_IP_LEN);
+            strncpy(sourceAddrs[noOfsourceAddrs], tokstr, SCTP_MAX_IP_LEN);
             noOfsourceAddrs++;
         }
     }
@@ -411,13 +397,13 @@ void getArgs(int argc, char **argv)
 
     for (i = 0; i < dflg; i++) {
         printf("Destinationaddress to Host%d: %s\n", i, destins[i]);
-        strncpy((char *)destinAddrs[noOfdestinAddrs], destins[i], 16);
+        strncpy(destinAddrs[noOfdestinAddrs], destins[i], 16);
         noOfdestinAddrs++;
     }
 #ifdef HAVE_IPV6
     for (i = 0; i < dv6flg; i++) {
         printf("Destinationaddress to Host%d: %s\n", i, v6_destins[i]);
-        strncpy((char *)destinAddrs[noOfdestinAddrs], v6_destins[i], SCTP_MAX_IP_LEN);
+        strncpy(destinAddrs[noOfdestinAddrs], v6_destins[i], SCTP_MAX_IP_LEN);
         noOfdestinAddrs++;
     }
 
@@ -457,8 +443,6 @@ int main(int argc, char **argv)
     SCTP_ulpCallbacks my_ulp;
     int i;
 
-
-
     my_ulp.dataArriveNotif = &ulp_dataArriveNotif;
     my_ulp.networkStatusChangeNotif = &ulp_networkStatusChangeNotif;
     my_ulp.sendFailureNotif = &ulp_sendFailureNotif;
