@@ -1,44 +1,37 @@
 /*
- * --------------------------------------------------------------------------
- *
- *           //=====   //===== ===//=== //===//  //       //   //===//
- *          //        //         //    //    // //       //   //    //
- *         //====//  //         //    //===//  //       //   //===<<
- *              //  //         //    //       //       //   //    //
- *       ======//  //=====    //    //       //=====  //   //===//
- *
- * -------------- An SCTP implementation according to RFC 4960 --------------
+ *  $Id: mini-ulp.c,v 1.2 2003/04/08 09:38:32 ajung Exp $
  *
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
- * Copyright (C) 2001-2004 Andreas Jungmaier
- * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Acknowledgements:
- * Realized in co-operation between Siemens AG and the University of
- * Duisburg-Essen, Institute for Experimental Mathematics, Computer
- * Networking Technology group.
- * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany
- * (Förderkennzeichen 01AK045).
- * The authors alone are responsible for the contents.
+ * Realized in co-operation between Siemens AG
+ * and University of Essen, Institute of Computer Networking Technology.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * Contact: sctp-discussion@sctp.de
- *          thomas.dreibholz@gmail.com
- *          tuexen@fh-muenster.de
- *          andreas.jungmaier@web.de
+ * There are two mailinglists available at http://www.sctp.de which should be
+ * used for any discussion related to this implementation.
+ *
+ * Contact: discussion@sctp.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
+ *
+ * Purpose: Implements a few simple callback-functions for an "ULP"
+ * --------------------------------------------------------------------------
+ * !!!!!!!!!!!!!!!!   Watch out. Code here is frequently changing !!!!!!
+ * --------------------------------------------------------------------------
  */
 
 #include <stdio.h>
@@ -72,7 +65,7 @@ unsigned int payload = 0;
 /* global parameters, valid for all associations */
 extern unsigned int localNumberOfInStreams;
 
-extern gboolean use_unordered;
+extern gboolean unordered;
 
 static int dosend = 0;
 gboolean heartbeat = FALSE;
@@ -117,51 +110,51 @@ void timer_expired(unsigned int tID, void *associationIDvoid, void *unused);
 */
 void printString(char *log_name, unsigned char *byte_string, short nbytes)
 {
-    int i, j, linesOut;
-    char buff1[64];
-    char buff2[64];
-    char *ptr1, *ptr2, *dptrlast, *dptr;
-    char *hexes = "0123456789ABCDEF";
+	int i, j, linesOut;
+	char buff1[64];
+	char buff2[64];
+	char *ptr1, *ptr2, *dptrlast, *dptr;
+	char *hexes = "0123456789ABCDEF";
 
     fprintf(stdout, "Bytestring: %s.................................\n", log_name);
 
-    ptr1 = buff1;
-    ptr2 = buff2;
-    dptrlast = dptr = (char *)byte_string;
-    for (i = 0, linesOut = 0; i < nbytes; i++) {
-        *ptr1++ = hexes[0x0f & ((*dptr) >> 4)];
-        *ptr1++ = hexes[0x0f & (*dptr)];
-        *ptr1++ = ' ';
-        if ((*dptr >= 040) && (*dptr <= 0176))
-            *ptr2++ = *dptr;
-        else
-            *ptr2++ = '.';
-        dptr++;
-        if (((i + 1) % 16) == 0) {
-            *ptr1 = 0;
-            *ptr2 = 0;
-            fprintf(stdout, "%s %s\n", buff1, buff2);
-            linesOut++;
-            ptr1 = buff1;
-            ptr2 = buff2;
-            dptrlast = dptr;
-        }
-    }
-    if ((linesOut * 16) < nbytes) {
-        char spaces[64];
-        int dist, sp;
+	ptr1 = buff1;
+	ptr2 = buff2;
+	dptrlast = dptr = (char *)byte_string;
+	for (i = 0, linesOut = 0; i < nbytes; i++) {
+		*ptr1++ = hexes[0x0f & ((*dptr) >> 4)];
+		*ptr1++ = hexes[0x0f & (*dptr)];
+		*ptr1++ = ' ';
+		if ((*dptr >= 040) && (*dptr <= 0176))
+			*ptr2++ = *dptr;
+		else
+			*ptr2++ = '.';
+		dptr++;
+		if (((i + 1) % 16) == 0) {
+			*ptr1 = 0;
+			*ptr2 = 0;
+			fprintf(stdout, "%s %s\n", buff1, buff2);
+			linesOut++;
+			ptr1 = buff1;
+			ptr2 = buff2;
+			dptrlast = dptr;
+		}
+	}
+	if ((linesOut * 16) < nbytes) {
+		char spaces[64];
+		int dist, sp;
 
-        j = (linesOut * 16);
-        dist = ((16 - (i - j)) * 3) + 2;
-        *ptr1 = 0;
-        *ptr2 = 0;
-        for (sp = 0; sp < dist; sp++) {
-            spaces[sp] = ' ';
-        }
-        spaces[sp] = 0;
-        fprintf(stdout, "%s %s%s\n", buff1, spaces, buff2);
-    }
-    fflush(stdout);
+		j = (linesOut * 16);
+		dist = ((16 - (i - j)) * 3) + 2;
+		*ptr1 = 0;
+		*ptr2 = 0;
+		for (sp = 0; sp < dist; sp++) {
+			spaces[sp] = ' ';
+		}
+		spaces[sp] = 0;
+		fprintf(stdout, "%s %s%s\n", buff1, spaces, buff2);
+	}
+	fflush(stdout);
 }
 
 
@@ -224,11 +217,11 @@ void nextPath(void)
 int doPingPong(void)
 {
     int i;
-    /* use ascii bulk ping mode. */
-    strncpy((char *)pingBuffer, "ping", 4);
-    for (i = 4; i < pingBufSize; i++) {
-            pingBuffer[i] = 'A' + (i % 26);
-    }
+	/* use ascii bulk ping mode. */
+    strncpy(pingBuffer, "ping", 4);
+	for (i = 4; i < pingBufSize; i++) {
+			pingBuffer[i] = 'A' + (i % 26);
+	}
     gettimeofday(&pingstart_time, NULL);
     return sctp_send(assocID, pingStream, (unsigned char *) pingBuffer,
                 pingBufSize, payload, -1, 0, 0, 0, 0);
@@ -240,9 +233,9 @@ int handlePong(void)
     int secs, msecs, timediff;
     struct timeval result;
 
-    pingPongCount--;
-    if (pingPongCount <= 0) {
-        /* done */
+	pingPongCount--;
+	if (pingPongCount <= 0) {
+		/* done */
         printf("#######################################################################\n");
 
         gettimeofday(&pongstop_time, NULL);
@@ -256,55 +249,53 @@ int handlePong(void)
 
         printf("#######################################################################\n");
         fflush(stdout);
-        return -1 ;
-    }
+		return -1 ;
+	}
     return sctp_send(assocID, pingStream, (unsigned char *) pingBuffer,
          pingBufSize, payload, -1, 0, 0, 0, 0);
 }
 
 
-void ulp_stdin_cb(int fd, short int revents, short int* gotEvents, void* dummy)
+void ulp_stdin_cb(int fd, short int revents, short int* gotEvents, void * dummy)
 {
     int lenread,i;
     SCTP_AssociationStatus status;
 
-    char readBuffer[256];
-    fgets(readBuffer, 256, stdin);
-    lenread = strlen(readBuffer);
+	char readBuffer[256];
+	fgets(readBuffer, 256, stdin);
+	lenread = strlen(readBuffer);
 
     while(lenread > 0 && (readBuffer[(lenread - 1)] == '\n' || readBuffer[(lenread - 1)] == '\r')) {
         readBuffer[(lenread - 1)] = 0;
         lenread --;
     }
 
-    if (lenread == 0) {
-        printf(">");
-        fflush(stdout);
-        return;
-    }
-    if (strcmp(readBuffer, "help") == 0) {
-        printf("Available commands are:\n");
-        printf(" quit - exit the program\n");
-        /*
-        printf(" setassoc:WWW.XXX.YYY.ZZZ - set association id for assoc with this destination\n");
-        */
-        printf(" setpay:payload - set the payload type\n");
-        printf(" setdefstrm:num - set the default stream to\n");
-        printf(" ping:size:stream:times - play ping pong\n");
-        printf(" stat - print queue counts\n");
-        printf(" bulk:size:stream:number - send a bulk of messages\n");
-        printf(" assoc - associate with the set destination\n");
-        printf(" abort - send an abort to the peer\n");
-        printf(" nextpath - switch to next path\n");
-        printf(" term - terminate the set destination association (graceful shutdown )\n");
-        printf(" heart:on/off - Turn HB on or off  to the destination\n");
-        printf(" heartdelay:time - Add number of seconds + RTO to hb interval\n");
-        printf(" getrtt - print RTT on primary path\n");
-        printf(" dohb:N - send a HB on path N\n");
-        printf(" sendloop:N - send test script loopback request of N size\n");
-        printf(" sendloopend:N - send test script loopback request of N size and terminate\n");
-        printf(" timeoutval:msecs - set bulk timer to send one message every msecs\n");
-        printf(" some-other-string - send this to a peer if a peer is set\n");
+	if (lenread == 0) {
+		printf(">");
+		fflush(stdout);
+		return;
+	}
+	if (strcmp(readBuffer, "help") == 0) {
+		printf("Available commands are:\n");
+		printf(" quit - exit the program\n");
+/*        printf(" setassoc:WWW.XXX.YYY.ZZZ - set association id for assoc with this destination\n"); */
+		printf(" setpay:payload - set the payload type\n");
+		printf(" setdefstrm:num - set the default stream to\n");
+		printf(" ping:size:stream:times - play ping pong\n");
+		printf(" stat - print queue counts\n");
+		printf(" bulk:size:stream:number - send a bulk of messages\n");
+		printf(" assoc - associate with the set destination\n");
+		printf(" abort - send an abort to the peer\n");
+		printf(" nextpath - switch to next path\n");
+		printf(" term - terminate the set destination association (graceful shutdown )\n");
+		printf(" heart:on/off - Turn HB on or off  to the destination\n");
+		printf(" heartdelay:time - Add number of seconds + RTO to hb interval\n");
+		printf(" getrtt - print RTT on primary path\n");
+		printf(" dohb:N - send a HB on path N\n");
+		printf(" sendloop:N - send test script loopback request of N size\n");
+		printf(" sendloopend:N - send test script loopback request of N size and terminate\n");
+    	printf(" timeoutval:msecs - set bulk timer to send one message every msecs\n");
+		printf(" some-other-string - send this to a peer if a peer is set\n");
     } else if (strcmp(readBuffer, "quit") == 0) {
         /* add some cleanups */
         exit(0);
@@ -312,183 +303,178 @@ void ulp_stdin_cb(int fd, short int revents, short int* gotEvents, void* dummy)
         nextPath();
     } else if (strcmp(readBuffer, "abort") == 0) {
         int result;
-        result = sctp_abort(assocID);
+        result = sctp_abort(assocID, 0, NULL);
         timerID = 0;
-        printf("Sent an abort -> result %d\n", result);
+		printf("Sent an abort -> result %d\n", result);
     } else if (strncmp(readBuffer, "sendloop:", 9) == 0) {
-        int x, ret = -1;
-        x = (int) strtol(&readBuffer[9], NULL, 0);
-        if (x == 0) {
-            printf("N was 0? defaulting to 64\n");
-            x = 64;
-        }
-        /*
-        ret = sendLoopRequest(m, x);
-        */
-        printf("Sent loop returned %d\n", ret);
+		int x, ret = -1;
+		x = (int) strtol(&readBuffer[9], NULL, 0);
+		if (x == 0) {
+			printf("N was 0? defaulting to 64\n");
+			x = 64;
+		}
+		/* ret = sendLoopRequest(m, x); */
+		printf("Sent loop returned %d\n", ret);
     } else if (strncmp(readBuffer, "setpay:", 7) == 0) {
-        payload = strtol(&readBuffer[7], NULL, 0);
-        printf("payloadtype set to %d\n", payload);
+		payload = strtol(&readBuffer[7], NULL, 0);
+		printf("payloadtype set to %d\n", payload);
     } else if (strncmp(readBuffer, "setassoc:", 9) == 0) {
-/*      this could be used when we have several assocs    */
-/*         = strtol(&readBuffer[9], NULL, 0);  */
-        printf("assoc ID is %u\n", assocID);
+/*      this could be used when we have several assocs		 */
+/*         = strtol(&readBuffer[9], NULL, 0); */
+		printf("assoc ID is %u\n", assocID);
     } else if (strncmp(readBuffer, "timeoutval:", 11) == 0) {
-        timeoutval = strtol(&readBuffer[11], NULL, 0);
-        printf("Timeoutval set to %d msecs\n", timeoutval);
-     } else if (strncmp(readBuffer, "heart:", 6) == 0) {
-        if (strncmp(&readBuffer[6], "off", 3) == 0) {
+		timeoutval = strtol(&readBuffer[11], NULL, 0);
+		printf("Timeoutval set to %d msecs\n", timeoutval);
+ 	} else if (strncmp(readBuffer, "heart:", 6) == 0) {
+		if (strncmp(&readBuffer[6], "off", 3) == 0) {
             for (i = 0; i < noOfPaths; i++)
                 sctp_changeHeartBeat(assocID, i, FALSE, 0);
-        } else {
+		} else {
             for (i = 0; i < noOfPaths; i++)
                 sctp_changeHeartBeat(assocID, i, TRUE, timeoutval);
-        }
-    } else if (strcmp(readBuffer, "getrtt") == 0) {
-        /* printf("RTT of TO is %d\n", sctpGETRTTREPORT(m, &to_ip)); */
+		}
+	} else if (strcmp(readBuffer, "getrtt") == 0) {
+		/* printf("RTT of TO is %d\n", sctpGETRTTREPORT(m, &to_ip)); */
     } else if (strncmp(readBuffer, "dohb:",5) == 0) {
         int path;
-        path = strtol(&readBuffer[5], NULL, 0);
+		path = strtol(&readBuffer[5], NULL, 0);
         path =  sctp_requestHeartbeat(assocID, (short)path);
-        printf("HB Request %s\n", (path==0)?"successfully sent":"failed");
+		printf("HB Request %s\n", (path==0)?"successfully sent":"failed");
     } else if (strncmp(readBuffer, "heartdelay:", 11) == 0) {
-        int newdelay;
-        newdelay = strtol(&readBuffer[11], NULL, 0);
+	    int newdelay;
+		newdelay = strtol(&readBuffer[11], NULL, 0);
         for (i = 0; i < noOfPaths; i++)
             sctp_changeHeartBeat(assocID, i, TRUE, newdelay);
         timeoutval =  newdelay;
-        printf("HB.Intervall set to %d msecs\n", newdelay);
-     } else if (strcmp("term", readBuffer) == 0) {
+		printf("HB.Intervall set to %d msecs\n", newdelay);
+ 	} else if (strcmp("term", readBuffer) == 0) {
         sctp_shutdown(assocID);
         timerID = 0;
-     } else if (strcmp("stat", readBuffer) == 0) {
-        i = sctp_getAssocStatus(assocID,&status);
-        if (i == 0) {
-            printf(" %u chunks in SendQueue, %u chunks in RTX queue\n",
-                status.noOfChunksInSendQueue, status.noOfChunksInRetransmissionQueue);
-        }
-    } else if (strncmp("setdefstrm:", readBuffer, 11) == 0) {
-        sID1 = strtol(&readBuffer[11], NULL, 0);
-    } else if (strcmp("assoc", readBuffer) == 0) {
-        printf("Sorry - command not yet implemented\n");
-        /*    sctpASSOCIATE(m, &to_ip, 0); */
-    } else if (strncmp("ping:", readBuffer, 5) == 0) {
-        char *end, *nxt;
-        int skip;
-        skip = 0;
-        if (pingPongCount) {
-            printf("Sorry ping-pong already in progress\n");
-            printf(">");
-            fflush(stdout);
-            return;
-        }
-        pingBufSize = strtol(&readBuffer[5], &end, 0);
-        if (end != NULL) {
-            if (*end != ':') {
-                skip = 1;
-            }
-        } else {
-            skip = 1;
-        }
+ 	} else if (strcmp("stat", readBuffer) == 0) {
+	    i = sctp_getAssocStatus(assocID,&status);
+	    if (i == 0) {
+    		printf(" %u chunks in SendQueue, %u chunks in RTX queue\n",
+    		    status.noOfChunksInSendQueue, status.noOfChunksInRetransmissionQueue);
+	    }
+	} else if (strncmp("setdefstrm:", readBuffer, 11) == 0) {
+		sID1 = strtol(&readBuffer[11], NULL, 0);
+	} else if (strcmp("assoc", readBuffer) == 0) {
+		printf("Sorry - command not yet implemented\n");
+        /*  sctpASSOCIATE(m, &to_ip, 0); */
+	} else if (strncmp("ping:", readBuffer, 5) == 0) {
+		char *end, *nxt;
+		int skip;
+		skip = 0;
+		if (pingPongCount) {
+			printf("Sorry ping-pong already in progress\n");
+			printf(">");
+			fflush(stdout);
+			return;
+		}
+		pingBufSize = strtol(&readBuffer[5], &end, 0);
+		if (end != NULL) {
+			if (*end != ':') {
+				skip = 1;
+			}
+		} else {
+			skip = 1;
+		}
 
-        if (skip) {
-            printf("mal-formed request at size\n");
-            printf(">");
-            fflush(stdout);
-            return;
-        }
-        nxt = end;
-        nxt++;
-        pingStream = strtol(nxt, &end, 0);
-        if (end != NULL) {
-            if (*end != ':') {
-                skip = 1;
-            }
-        } else {
-            skip = 1;
-        }
-        if (skip) {
-            printf("mal-formed request stream\n");
-            printf(">");
-            fflush(stdout);
-            return;
-        }
-        nxt = end;
-        nxt++;
-        pingPongCount = strtol(nxt, NULL, 0);
-        if (pingPongCount == 0) {
-            printf("mal-formed request at times\n");
-            printf(">");
-            fflush(stdout);
-            return;
-        }
-        /* prepare ping buffer */
+		if (skip) {
+			printf("mal-formed request at size\n");
+			printf(">");
+			fflush(stdout);
+			return;
+		}
+		nxt = end;
+		nxt++;
+		pingStream = strtol(nxt, &end, 0);
+		if (end != NULL) {
+			if (*end != ':') {
+				skip = 1;
+			}
+		} else {
+			skip = 1;
+		}
+		if (skip) {
+			printf("mal-formed request stream\n");
+			printf(">");
+			fflush(stdout);
+			return;
+		}
+		nxt = end;
+		nxt++;
+		pingPongCount = strtol(nxt, NULL, 0);
+		if (pingPongCount == 0) {
+			printf("mal-formed request at times\n");
+			printf(">");
+			fflush(stdout);
+			return;
+		}
+		/* prepare ping buffer */
         doPingPong();
-    } else if (strncmp("bulk:", readBuffer, 5) == 0) {
-        /*
-        int ret;
-        */
-        char *end, *nxt;
-        int skip;
-        skip = 0;
-        dataLength = strtol(&readBuffer[5], &end, 0);
-        if (end != NULL) {
-            if (*end != ':') {
-                skip = 1;
-            }
-        } else {
-            skip = 1;
-        }
+	} else if (strncmp("bulk:", readBuffer, 5) == 0) {
+		char *end, *nxt;
+		int skip;
+		skip = 0;
+		dataLength = strtol(&readBuffer[5], &end, 0);
+		if (end != NULL) {
+			if (*end != ':') {
+				skip = 1;
+			}
+		} else {
+			skip = 1;
+		}
 
-        if (skip) {
-            printf("mal-formed request at size\n");
-            printf(">");
-            fflush(stdout);
-            return;
-        }
-        nxt = end;
-        nxt++;
-        sID1 = strtol(nxt, &end, 0);
-        if (end != NULL) {
-            if (*end != ':') {
-                skip = 1;
-            }
-        } else {
-            skip = 1;
-        }
-        if (skip) {
-            printf("mal-formed request stream\n");
-            printf(">");
-            fflush(stdout);
-            return;
-        }
-        nxt = end;
-        nxt++;
-        sendEvents = strtol(nxt, NULL, 0);
-        if (sendEvents == 0) {
-            printf("mal-formed request at times\n");
-            printf(">");
-            fflush(stdout);
+		if (skip) {
+			printf("mal-formed request at size\n");
+			printf(">");
+			fflush(stdout);
+			return;
+		}
+		nxt = end;
+		nxt++;
+		sID1 = strtol(nxt, &end, 0);
+		if (end != NULL) {
+			if (*end != ':') {
+				skip = 1;
+			}
+		} else {
+			skip = 1;
+		}
+		if (skip) {
+			printf("mal-formed request stream\n");
+			printf(">");
+			fflush(stdout);
+			return;
+		}
+		nxt = end;
+		nxt++;
+		sendEvents = strtol(nxt, NULL, 0);
+		if (sendEvents == 0) {
+			printf("mal-formed request at times\n");
+			printf(">");
+			fflush(stdout);
             sendEvents = 0;
-            return;
-        }
+			return;
+		}
         timerID = sctp_startTimer(timeoutval/1000, (timeoutval%1000)*1000, &timer_expired, NULL, NULL);
 
-    } else if (strcmp("inqueue", readBuffer) == 0) {
-/*        printf("Outbound queue count to dest = %d\n", sctpHOWMANYINQUEUE(m, &to_ip)); */
-/*        printf("Inbound queue count = %d\n", sctpHOWMANYINBOUND(m)); */
-    } else {
+	} else if (strcmp("inqueue", readBuffer) == 0) {
+/*		printf("Outbound queue count to dest = %d\n", sctpHOWMANYINQUEUE(m, &to_ip));  */
+/*		printf("Inbound queue count = %d\n", sctpHOWMANYINBOUND(m)); */
+	} else {
         if (timerID != 0) {
-            printf("bulk or ping might be in progress, not sending !!!\n");
-        } else {
-            int xsxx;
+		    printf("bulk or ping might be in progress, not sending !!!\n");
+		} else {
+			int xsxx;
             xsxx = sctp_send(assocID, sID1, (unsigned char *)readBuffer,
                    lenread, payload, -1, 0, 0, 0, 0);
-            printf("Returned %d from the send (1==association error, 0==success, -1==could not send)\n", xsxx);
-        }
-    }
-    printf(">");
-    fflush(stdout);
+			printf("Returned %d from the send (1==association error, 0==success, -1==could not send)\n", xsxx);
+		}
+	}
+	printf(">");
+	fflush(stdout);
 
 }
 
@@ -510,9 +496,7 @@ void timer_expired(unsigned int tID, void *associationIDvoid, void *unused)
     int i;
     unsigned char j;
 
-/*
-    AssociationStatus *state = NULL;
-*/
+//     AssociationStatus *state = NULL;
 
 /*  fprintf(stderr, "#######################################################################\n");
     fprintf(stderr, "ULP timer expired: sending next chunk\n");
@@ -554,9 +538,9 @@ void timer_expired(unsigned int tID, void *associationIDvoid, void *unused)
             /*                send chunk with length = datalength  */
 
         for (i = 0; i < SEND_EVENTS_WHEN_TIMER_EXPIRES; i++) {
-           /*gettimeofday((struct timeval *) &chunk[0], NULL); */
+           /* gettimeofday((struct timeval *) &chunk[0], NULL); */
            result = sctp_send(assocID, sID1, (unsigned char *) chunk,
-                           dataLength, payload, -1, 0, 0, use_unordered, 0);
+                           dataLength, payload, -1, 0, 0, unordered, 0);
         }
 
         if (result == -1) {
@@ -576,7 +560,7 @@ void timer_expired(unsigned int tID, void *associationIDvoid, void *unused)
               2. streamID
 */
 
-void ulp_dataArriveNotif(unsigned int assoc_id, unsigned short stream_id, unsigned int len,
+void ulp_dataArriveNotif(unsigned int assoc_id, unsigned int stream_id, unsigned int len,
                          unsigned short streamSN,unsigned int TSN, unsigned int protoID,
                          unsigned int unordered, void* dummy)
 {
@@ -585,9 +569,7 @@ void ulp_dataArriveNotif(unsigned int assoc_id, unsigned short stream_id, unsign
     unsigned int theTSN;
     unsigned short SID, SSN;
     gboolean ascii_chunk;
-/*
-    struct timeval *t;
-*/
+/*    struct timeval *t; */
 
     fprintf(stderr, "\n#######################################################################\n");
     fprintf(stderr, "ulp_dataArriveNotif : Association ID : %u, Stream ID : %u, Length : %u, TSN: %u, ProtoID : %u\n",
@@ -598,13 +580,13 @@ void ulp_dataArriveNotif(unsigned int assoc_id, unsigned short stream_id, unsign
     sctp_receive(assoc_id, stream_id, chunk, (unsigned int *) &length, &SSN, &theTSN, SCTP_MSG_DEFAULT);
     chunks_received++;
 
-/*
-    t = (struct timeval *) chunk;
-    fprintf(stderr, "%lu.%06lu \n", t->tv_sec, t->tv_usec);
-*/
+/*     t = (struct timeval *) chunk; */
+
+/*    fprintf(stderr, "%lu.%06lu \n", t->tv_sec, t->tv_usec); */
+
     if (pingPongCount > 0) {
-        if (strncmp((char *)chunk, "pong", 4) == 0) {
-            i = handlePong();
+        if (strncmp(chunk, "pong", 4) == 0) {
+			i = handlePong();
             if (i==-1) {
                 printf("Data not sent, SHUTDOWN-state reached, terminating Ping-Pong !\n");
                 pingPongCount = 0;
@@ -614,12 +596,12 @@ void ulp_dataArriveNotif(unsigned int assoc_id, unsigned short stream_id, unsign
 
     }
 
-    if(strncmp((char *)chunk, "ping", 4) == 0) {
+    if(strncmp(chunk, "ping", 4) == 0) {
         /* it is a ping-pong message, send it back after changing
-         * the first 4 bytes to pong
-         */
-        strncpy((char *)chunk, "pong", 4);
-        SID =  stream_id;
+		 * the first 4 bytes to pong
+		 */
+		strncpy(chunk, "pong", 4);
+        SID = (unsigned short) stream_id;
         i = sctp_send(assoc_id, SID, (unsigned char *) chunk,
             len, protoID, -1, 0, 0, 0, 0);
         if (i==-1) {
@@ -644,8 +626,8 @@ void ulp_dataArriveNotif(unsigned int assoc_id, unsigned short stream_id, unsign
     } else {
         printString("ULP chunk received:", chunk, length);
     }
-    printf(">");
-    fflush(stdout);
+	printf(">");
+	fflush(stdout);
 }
 
 
@@ -653,19 +635,19 @@ void ulp_dataArriveNotif(unsigned int assoc_id, unsigned short stream_id, unsign
 /* indicates a change of network status (chapter 9.2.C).
    params: 1.  associationID
            2.  destinationAddresses
-             3.  newState
+	         3.  newState
 */
-void ulp_networkStatusChangeNotif(unsigned int assoc_id, short dest_add, unsigned short new_state, void* dummy)
+void ulp_networkStatusChangeNotif(unsigned int assoc_id, unsigned int dest_add, int new_state, void* dummy)
 {
     fprintf(stderr, "\n#######################################################################\n");
-    fprintf(stderr, "ulp_networkStatusChangeNotif: associationID = %u, destination address = %d \n",
+    fprintf(stderr, "ulp_networkStatusChangeNotif: associationID = %u, destination address = %u \n",
         assoc_id, dest_add);
     fprintf(stderr, "ulp_networkStatusChangeNotif: state = %s, noOfPaths=%u\n",
         ((new_state == 0) ? "ACTIVE" : "INACTIVE"), noOfPaths);
     fprintf(stderr, "#######################################################################\n");
 
-    printf(">");
-    fflush(stdout);
+	printf(">");
+	fflush(stdout);
     /* if (noOfPaths > 0 && new_state) nextPath(); */
 }
 
@@ -689,7 +671,7 @@ ulp_sendFailureNotif(unsigned int assoc_id,
    params: 1.  associationID
            2.  status, type of event
 */
-void ulp_communicationLostNotif(unsigned int assoc_id, unsigned short status, void* dummy)
+void ulp_communicationLostNotif(unsigned int assoc_id, int status, void* dummy)
 {
     int i, timediff, secs, msecs;
     struct timeval result;
@@ -697,7 +679,7 @@ void ulp_communicationLostNotif(unsigned int assoc_id, unsigned short status, vo
 
     fprintf(stderr, "\n#######################################################################\n");
     fprintf(stderr,
-            "ulp_communicationLostNotif : Association ID : %d, State: %d\n", assoc_id, status);
+            "ulp_communicationLostNotif : Association ID : %u, State: %d\n", assoc_id, status);
     fprintf(stderr, "  Information  : Total number of Chunks received : %u\n", chunks_received);
     i = sctp_deleteAssociation(assoc_id);
     fprintf(stderr, "   Deleted Association with new ULP Primitive, result : %d\n", i);
@@ -716,11 +698,9 @@ void ulp_communicationLostNotif(unsigned int assoc_id, unsigned short status, vo
     if (timerID)
         sctp_stopTimer(timerID);
     timerID = 0;
-    printf(">");
-    fflush(stdout);
-/*
-    exit(0);
-*/
+	printf(">");
+	fflush(stdout);
+/*    exit(0); */
 }
 
 void ulp_ShutdownCompleteNotif(unsigned int assoc_id, void* dummy)
@@ -746,8 +726,8 @@ void ulp_ShutdownCompleteNotif(unsigned int assoc_id, void* dummy)
     if (timerID)
         sctp_stopTimer(timerID);
     timerID = 0;
-    printf(">");
-    fflush(stdout);
+	printf(">");
+	fflush(stdout);
 }
 
 
@@ -758,7 +738,7 @@ void ulp_ShutdownCompleteNotif(unsigned int assoc_id, void* dummy)
 */
 void* ulp_communicationUpNotif(unsigned int assoc_id, int status,
                          unsigned int noOfDestinations, unsigned short noOfInStreams, unsigned short noOfOutStreams,
-                        int associationSupportsPRSCTP, void* dummy)
+                        int associationSupportsPRSCTP, int adLayerIndLen, void* adLayerInd, void* dummy)
 {
     int i;
 
@@ -788,7 +768,9 @@ void* ulp_communicationUpNotif(unsigned int assoc_id, int status,
         for (i = 0; i < noOfPaths; i++)
             sctp_changeHeartBeat(assocID, i, TRUE, timeInt);
     /* by default, HB  is with adaptive intervall enabled */
-    printf(">");
-    fflush(stdout);
+	printf(">");
+	fflush(stdout);
     return NULL;
 }
+
+
