@@ -1,27 +1,32 @@
 /*
+ *  $Id: mini-ulp.h,v 1.1.1.1 2002/11/20 14:51:17 ajung Exp $
  *
- * SCTP implementation according to RFC 4960.
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
  *
  * Realized in co-operation between Siemens AG
  * and University of Essen, Institute of Computer Networking Technology.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * There are two mailinglists available at http://www.sctp.de which should be
+ * used for any discussion related to this implementation.
  *
  * Contact: discussion@sctp.de
- *          tuexen@fh-muenster.de
- *          ajung@iem.uni-due.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
  *
  * Purpose: Implements a few simple callback-functions for an "ULP"
  */
@@ -33,7 +38,7 @@
 #include <config.h>
 #endif
 #include <glib.h>
-#include "sctp.h"
+#include <sctp.h>
 
 #ifdef SOLARIS
 #define timersub(tvp, uvp, vvp)                                     \
@@ -60,6 +65,7 @@ void mulp_streamRoundRobin(void);
 void
 ulp_stdin_cb(int fd, short int revents, short int* gotEvents, void * dummy);
 
+
 void
 ulp_socket_error(gint sin_fd,
                  unsigned char *buffer,
@@ -70,7 +76,7 @@ ulp_socket_error(gint sin_fd,
       params: 1. associationID
               2. streamID
 */
-void ulp_dataArriveNotif(unsigned int assoc_id, unsigned short stream_id, unsigned int len,
+void ulp_dataArriveNotif(unsigned int assoc_id, unsigned int stream_id, unsigned int len,
                         unsigned short streamSN,unsigned int TSN, unsigned int protoID,
                         unsigned int unordered, void* dummy);
 
@@ -79,7 +85,7 @@ void ulp_dataArriveNotif(unsigned int assoc_id, unsigned short stream_id, unsign
            2.  destinationAddresses
 	       3.  newState
 */
-void ulp_networkStatusChangeNotif(unsigned int assoc_id, short dest_add, unsigned short new_state, void* dummy);
+void ulp_networkStatusChangeNotif(unsigned int assoc_id, unsigned int dest_add, int new_state, void* dummy);
 
 /* indicates a sned failure (chapter 10.2.B).
    params: 1.  associationID
@@ -95,7 +101,7 @@ void ulp_sendFailureNotif(unsigned int assoc_id,
    params: 1.  associationID
            2.  status, type of event
 */
-void ulp_communicationLostNotif(unsigned int assoc_id, unsigned short status, void* dummy);
+void ulp_communicationLostNotif(unsigned int assoc_id, int status, void* dummy);
 
 /* indicates that a association is established (chapter 10.2.D).
    params: 1.  associationID
@@ -104,7 +110,7 @@ void ulp_communicationLostNotif(unsigned int assoc_id, unsigned short status, vo
 void* ulp_communicationUpNotif(unsigned int assoc_id, int status,
                               unsigned int noOfDestinations,
                               unsigned short noOfInStreams, unsigned short noOfOutStreams,
-                              int associationSupportsPRSCTP, void* dummy);
+                              int associationSupportsPRSCTP, int adLayerIndLen, void* adLayerInd, void* dummy);
 
 /* indicates that association has been gracefully shutdown (chapter 10.2.H).
    params: 1.  associationID
