@@ -26,13 +26,17 @@
  * used for any discussion related to this implementation.
  *
  * Contact: discussion@sctp.de
- *          tuexen@fh-muenster.de
+ *          Michael.Tuexen@icn.siemens.de
  *          ajung@exp-math.uni-essen.de
  *
  * monitor.c  - main program module
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "sctp.h"
 
 #include <unistd.h>
@@ -59,7 +63,7 @@ static unsigned char localAddressList[MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES][SCTP_MA
 static unsigned char destinationAddress[SCTP_MAX_IP_LEN];
 static unsigned short noOfLocalAddresses = 0;
 static unsigned int associationID;
-static short sctpInstance;
+static unsigned short sctpInstance;
 
 static unsigned short remotePort    = DISCARD_PORT;
 static unsigned short localPort     = 1000;
@@ -96,34 +100,6 @@ typedef struct SCTP_Monitor_ToolStatus
 
 SCTP_MonitorToolStatus statusUpdate;
 
-char *
-pathStateName(unsigned int state)
-{
-    switch (state) {
-        case SCTP_PATH_OK:
-            return "REACHABLE";
-            break;
-        case SCTP_PATH_UNREACHABLE:
-            return "UNREACHABLE";
-            break;
-        case SCTP_PATH_ADDED:
-            return "ADDED";
-            break;
-        case SCTP_PATH_REMOVED:
-            return "REMOVED";
-            break;
-        case SCTP_PATH_CONFIRMED:
-            return "CONFIRMED";
-            break;
-        case SCTP_PATH_UNCONFIRMED:
-            return "UNCONFIRMED";
-            break;
-        default:
-            return "UNKNOWN";
-            break;
-    }
-}
-
 void printUsage(void)
 {
     printf("usage: monitor [options] -s source_addr_1 -d destination_addr ...\n");
@@ -150,7 +126,7 @@ void getArgs(int argc, char **argv)
         switch (c) {
         case 'd':
             if (strlen(optarg) < SCTP_MAX_IP_LEN) {
-                strcpy((char *)destinationAddress, optarg);
+                strcpy(destinationAddress, optarg);
 		client = 1;
             }
             break;
@@ -163,7 +139,7 @@ void getArgs(int argc, char **argv)
         case 's':
             if ((noOfLocalAddresses < MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES) &&
                 (strlen(optarg) < SCTP_MAX_IP_LEN  )) {
-                strcpy((char *)localAddressList[noOfLocalAddresses], optarg);
+                strcpy(localAddressList[noOfLocalAddresses], optarg);
                 noOfLocalAddresses++;
             }
             break;
@@ -324,47 +300,45 @@ void initializecurses()
 void ncurses_display_AssocStatus(unsigned int assocID)
 
 {
-  SCTP_AssociationStatus assocStatus;
-  int i; 
-  char tstr[200];
+    SCTP_AssociationStatus assocStatus;
+    int i; 
+    char tstr[200];
 
-  wclear(assocWin);
-  wrefresh(assocWin);
+    wclear(assocWin);
+    wrefresh(assocWin);
 
-  sctp_getAssocStatus(assocID,&assocStatus);
+    sctp_getAssocStatus(assocID,&assocStatus);
 
-  sprintf(assocInfo," Association ID : %d, Local port : %d, Remote port : %d\n",
-	  assocID,assocStatus.sourcePort,assocStatus.destPort);
+    sprintf(assocInfo," Association ID : %d, Local port : %d, Remote port : %d\n",
+        assocID,assocStatus.sourcePort,assocStatus.destPort);
 
-  waddstr(assocWin,assocInfo);
-  wrefresh(assocWin);
+    waddstr(assocWin,assocInfo);
+    wrefresh(assocWin);
 
-  sprintf(assocInfo," Local address(es) is(are) :");
+    sprintf(assocInfo," Local address(es) is(are) :");
 
-  for (i=0; i <  noOfLocalAddresses; i++)
-     {      
-       sprintf(tstr," %s,", localAddressList[i]);
-       strcat(assocInfo,tstr);
-     }
-  waddstr(assocWin,assocInfo);
-  wrefresh(assocWin);
+    for (i=0; i <  noOfLocalAddresses; i++) {      
+        sprintf(tstr," %s,", localAddressList[i]);
+        strcat(assocInfo,tstr);
+    }
+    waddstr(assocWin,assocInfo);
+    wrefresh(assocWin);
   
-  sprintf(assocInfo,"\n No. of Destination address : %d,\t\t Primary destination address : %s\n Number of incoming Streams : %d,\t\t Number of outgoing streams : %d\n Primary Address Index: %d,\t\t\t Current Receiver Window Size : %d\n Outstanding bytes : %-8d,\t\t\t No. of chunks in send queue : %d\n No. of chunks in retransmission queue : %d,\t No. of chunks in reception queue : %d\n Initial Round Trip Timeout : %d (msecs),\t Minimum Round Trip Timeout : %d (msecs)\n Maximum Round Trip Timeout : %d(msecs),\t Cookie Lifetime : %d (msecs)\n Max retransmission per association : %d,\t Max retransmission per path : %d\n Max Initial Retransmission : %d,\t\t Local Receiver Window : %d\n Delay for delay acknowledge : %d (msecs),\t IP Type Of Service : %x\n Current Incoming stream ID : %d,\t\t Current Outgoing stream ID : %d ",
-	  assocStatus.numberOfAddresses, assocStatus.primaryDestinationAddress, 
-	  statusUpdate.noOfInStreams, statusUpdate.noOfOutStreams, assocStatus.primaryAddressIndex,
-	  assocStatus.currentReceiverWindowSize, assocStatus.outstandingBytes, 
-	  assocStatus.noOfChunksInSendQueue, assocStatus.noOfChunksInRetransmissionQueue, 
-	  assocStatus.noOfChunksInReceptionQueue, assocStatus.rtoInitial, assocStatus.rtoMin, 
-	  assocStatus.rtoMax, assocStatus.validCookieLife, assocStatus.assocMaxRetransmits, 
-	  assocStatus.pathMaxRetransmits, assocStatus.maxInitRetransmits, assocStatus.myRwnd, 
-	  assocStatus.delay, assocStatus.ipTos, 
-	  statusUpdate.InstreamIDList[currInstreamID], 
-	  statusUpdate.OutstreamIDList[currOutstreamID] );
+    sprintf(assocInfo,"\n No. of Destination address : %d,\t\t Primary destination address : %s\n Number of incoming Streams : %d,\t\t Number of outgoing streams : %d\n Primary Address Index: %d,\t\t\t Current Receiver Window Size : %d\n Outstanding bytes : %-8d,\t\t\t No. of chunks in send queue : %d\n No. of chunks in retransmission queue : %d,\t No. of chunks in reception queue : %d\n Initial Round Trip Timeout : %d (msecs),\t Minimum Round Trip Timeout : %d (msecs)\n Maximum Round Trip Timeout : %d(msecs),\t Cookie Lifetime : %d (msecs)\n Max retransmission per association : %d,\t Max retransmission per path : %d\n Max Initial Retransmission : %d,\t\t Local Receiver Window : %d\n Delay for delay acknowledge : %d (msecs),\t IP Type Of Service : %x\n Current Incoming stream ID : %d,\t\t Current Outgoing stream ID : %d ",
+        assocStatus.numberOfDestinationPaths, assocStatus.primaryDestinationAddress, 
+        statusUpdate.noOfInStreams, statusUpdate.noOfOutStreams, assocStatus.primaryAddressKey,
+        assocStatus.currentReceiverWindowSize, assocStatus.outstandingBytes, 
+        assocStatus.noOfChunksInSendQueue, assocStatus.noOfChunksInRetransmissionQueue, 
+        assocStatus.noOfChunksInReceptionQueue, assocStatus.rtoInitial, assocStatus.rtoMin, 
+        assocStatus.rtoMax, assocStatus.validCookieLife, assocStatus.assocMaxRetransmits, 
+        assocStatus.pathMaxRetransmits, assocStatus.maxInitRetransmits, assocStatus.myRwnd, 
+        assocStatus.delay, assocStatus.ipTos, 
+        statusUpdate.InstreamIDList[currInstreamID], 
+        statusUpdate.OutstreamIDList[currOutstreamID] );
   
-  waddstr(assocWin,assocInfo);
-  wrefresh(assocWin);
-  wrefresh(textWin);
-
+    waddstr(assocWin,assocInfo);
+    wrefresh(assocWin);
+    wrefresh(textWin);
 }
 
 
@@ -389,61 +363,53 @@ void ncurses_display_PathStatus(unsigned int assocID)
     wrefresh(pathWin);
 
     sctp_getAssocStatus(assocID, &assocStatus);
-    for (pathID=0; pathID < assocStatus.numberOfAddresses; pathID++)
-      {
-	sctp_getPathStatus(assocID, pathID, &pathStatus);
-	strcpy((char *)destaddr[pathID], (const char *)pathStatus.destinationAddress);
-	State[pathID] = pathStatus.state;
-	Rto[pathID] = pathStatus.rto;
-	HB_Interval[pathID] = pathStatus.heartbeatIntervall;
-	SRTT[pathID] = pathStatus.srtt;
-	RTTVar[pathID] = pathStatus.rttvar;
-	SlowStThres[pathID] = pathStatus.ssthresh;
-	PartBytesAck[pathID] = pathStatus.partialBytesAcked;
-	CongestWin[pathID] = pathStatus.cwnd;
-	CongestWin2[pathID] = pathStatus.cwnd2;
-	MTU[pathID] = pathStatus.mtu;
-	OutBytesPerAddr[pathID] = pathStatus.outstandingBytesPerAddress;
-	IPTos[pathID] = pathStatus.ipTos;
-	
-	if (displayPathDetails == 0)
-	  {
-	    /* Check if the current path is the primary path */ 					       
-	    if (pathID == sctp_getPrimary(assocID))
-	      {
-		sprintf(pathInfo," Primary Path ID : %d, state of path : %s\n",
-			pathID, pathStateName(State[pathID])); 
-		waddstr(pathWin,pathInfo);
-		wrefresh(pathWin);
-	      }  
-	    else 
-	      {
-		sprintf(pathInfo," Path ID : %d, state of path : %s\n",
-			pathID, pathStateName(State[pathID])); 
-		waddstr(pathWin,pathInfo);
-		wrefresh(pathWin);
-	      }
-	  }
-	
-	else if (displayPathDetails == 1)
-	  { 
-	    mvwaddstr(pathWin,0,0,"[Hit Backspace to return to main menu]\n");
-	    wrefresh(pathWin);
+    for (pathID=0; pathID < assocStatus.numberOfDestinationPaths; pathID++) {
+        sctp_getPathStatus(assocID, pathID, &pathStatus);
+        strcpy(destaddr[pathID], pathStatus.destinationAddress);
+        State[pathID] = pathStatus.state;
+        Rto[pathID] = pathStatus.rto;
+        HB_Interval[pathID] = pathStatus.heartbeatIntervall;
+        SRTT[pathID] = pathStatus.srtt;
+        RTTVar[pathID] = pathStatus.rttvar;
+        SlowStThres[pathID] = pathStatus.ssthresh;
+        PartBytesAck[pathID] = pathStatus.partialBytesAcked;
+        CongestWin[pathID] = pathStatus.cwnd;
+        CongestWin2[pathID] = pathStatus.cwnd2;
+        MTU[pathID] = pathStatus.mtu;
+        OutBytesPerAddr[pathID] = pathStatus.outstandingBytesPerAddress;
+        IPTos[pathID] = pathStatus.ipTos;
 
-	    if (pathID == chosenPath)
-	      {
-		sprintf(pathInfo,"\n Path ID : %d, State of path : %s\t\t Destination address : %s\n Heartbeat Interval : %-8u\t\t Retransmisson time(msecs) : %-8d\n Smooth Round Trip time(msecs) : %-8d\t Round Trip time Variations(msecs) : %-8d\n Slow start threshold : %-8d\t\t Congestion Window Size : %-8d\n Outstanding Bytes per Address : %-8d\t Congestion Window Size 2 : %-8d\n Partial bytes acknowledge : %-8d\t\t IP type of service : %x\n MTU : %d bytes\n\n", 
-			pathID, pathStateName(State[pathID]),
-			destaddr[pathID],HB_Interval[pathID],Rto[pathID],
-			SRTT[pathID],RTTVar[pathID],SlowStThres[pathID],CongestWin[pathID],
-			OutBytesPerAddr[pathID], CongestWin2[pathID],PartBytesAck[pathID],
-			IPTos[pathID],MTU[pathID]);
+        if (displayPathDetails == 0)  {
+            /* Check if the current path is the primary path */
+            if (pathID == sctp_getPrimary(assocID)) {
+                sprintf(pathInfo," Primary Path ID : %d, state of path : %s\n",
+                    pathID,(State[pathID]==SCTP_PATH_OK)?"ACTIVE":"INACTIVE"); 
+                waddstr(pathWin,pathInfo);
+                wrefresh(pathWin);
+            } else {
+                sprintf(pathInfo," Path ID : %d, state of path : %s\n",
+                    pathID,(State[pathID]==SCTP_PATH_OK)?"ACTIVE":"INACTIVE"); 
+                waddstr(pathWin,pathInfo);
+                wrefresh(pathWin);
+            }
+        } else if (displayPathDetails == 1) {
+            mvwaddstr(pathWin,0,0,"[Hit Backspace to return to main menu]\n");
 
-		waddstr(pathWin,pathInfo);
-		wrefresh(pathWin);
-	      }
-	  }
-      }
+            wrefresh(pathWin);
+
+            if (pathID == chosenPath) {
+                sprintf(pathInfo,"\n Path ID : %d, State of path : %s\t\t Destination address : %s\n Heartbeat Interval : %-8u\t\t Retransmisson time(msecs) : %-8d\n Smooth Round Trip time(msecs) : %-8d\t Round Trip time Variations(msecs) : %-8d\n Slow start threshold : %-8d\t\t Congestion Window Size : %-8d\n Outstanding Bytes per Address : %-8d\t Congestion Window Size 2 : %-8d\n Partial bytes acknowledge : %-8d\t\t IP type of service : %x\n MTU : %d bytes\n\n", 
+                    pathID,(State[pathID]==SCTP_PATH_OK)?"ACTIVE":"INACTIVE",
+                    destaddr[pathID],HB_Interval[pathID],Rto[pathID],
+                    SRTT[pathID],RTTVar[pathID],SlowStThres[pathID],CongestWin[pathID],
+                    OutBytesPerAddr[pathID], CongestWin2[pathID],PartBytesAck[pathID],
+                    IPTos[pathID],MTU[pathID]);
+
+                waddstr(pathWin,pathInfo);
+                wrefresh(pathWin);
+            }
+        }
+    }
     wrefresh(textWin);
     displayPathDetails = 0;
 }
@@ -462,7 +428,7 @@ void periodicRefreshFunction(unsigned int timerID, void *parameter1, void *param
 
 void* communicationUpNotif(unsigned int assocID, int status, unsigned int noOfDestinations,
                            unsigned short noOfInStreams, unsigned short noOfOutStreams,
-                           int associationSupportsPRSCTP, void* dummy)
+                           int associationSupportsPRSCTP, int adLayerIndLen, void* adLayerInd, void* dummy)
 {	
     
   /* SCTP_AssociationStatus assocStatus; */
@@ -523,13 +489,12 @@ void* communicationUpNotif(unsigned int assocID, int status, unsigned int noOfDe
   return NULL;
 }
 
-void dataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int len,
+void dataArriveNotif(unsigned int assocID, unsigned int streamID, unsigned int len,
                      unsigned short streamSN,unsigned int TSN, unsigned int protoID,
                      unsigned int unordered, void* ulpDataPtr)
 {
     char chunk[SCTP_MAXIMUM_DATA_LENGTH + 1];
-    unsigned int length;
-    int i; 
+    int length, i; 
     unsigned int InstreamID_position = 0;
     unsigned int tsn;
     unsigned short ssn;
@@ -542,7 +507,7 @@ void dataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int
 
     /* read it */
     length = SCTP_MAXIMUM_DATA_LENGTH;
-    sctp_receive(assocID, streamID, (unsigned char *)chunk, &length, &ssn, &tsn, SCTP_MSG_DEFAULT);
+    sctp_receive(assocID, streamID, chunk, &length, &ssn, &tsn, SCTP_MSG_DEFAULT);
     chunk[length]=0;  
     
     /* and display it */
@@ -586,7 +551,7 @@ void dataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int
 }
 
 
-void networkStatusChangeNotif(unsigned int assocID, short destAddrIndex, unsigned short newState, void* ulpDataPtr)
+void networkStatusChangeNotif(unsigned int assocID, unsigned int destAddrIndex, int newState, void* ulpDataPtr)
 
 {
     SCTP_AssociationStatus assocStatus;
@@ -595,7 +560,7 @@ void networkStatusChangeNotif(unsigned int assocID, short destAddrIndex, unsigne
 
     sctp_getPathStatus(assocID, destAddrIndex , &pathStatus);
     sprintf(statusInfo, " Association ID =%-8x: Network status change: path %u (towards %s) is now %s\n", 
-	    assocID, destAddrIndex, pathStatus.destinationAddress, pathStateName(newState));
+	    assocID, destAddrIndex, pathStatus.destinationAddress,((newState == SCTP_PATH_OK) ? "ACTIVE" : "INACTIVE"));
     waddstr(statusWin,statusInfo);
     wrefresh(statusWin);
     
@@ -604,7 +569,7 @@ void networkStatusChangeNotif(unsigned int assocID, short destAddrIndex, unsigne
         (destAddrIndex == sctp_getPrimary(assocID))) {
         
         sctp_getAssocStatus(assocID, &assocStatus);
-        for (pathID=0; pathID < assocStatus.numberOfAddresses; pathID++)
+        for (pathID=0; pathID < assocStatus.numberOfDestinationPaths; pathID++)
 	  {
             sctp_getPathStatus(assocID, pathID, &pathStatus);
             if (pathStatus.state == SCTP_PATH_OK)
@@ -614,7 +579,7 @@ void networkStatusChangeNotif(unsigned int assocID, short destAddrIndex, unsigne
 	  }
         
         /* and use it */
-        if (pathID < assocStatus.numberOfAddresses) 
+        if (pathID < assocStatus.numberOfDestinationPaths) 
 	  {
 	    sctp_setPrimary(assocID, pathID);
 	  }
@@ -626,7 +591,7 @@ void networkStatusChangeNotif(unsigned int assocID, short destAddrIndex, unsigne
     ncurses_display_PathStatus(assocID);
 }
 
-void communicationLostNotif(unsigned int assocID, unsigned short status, void* ulpDataPtr)
+void communicationLostNotif(unsigned int assocID, int status, void* ulpDataPtr)
 {	
     unsigned char buffer[SCTP_MAXIMUM_DATA_LENGTH];
     unsigned int bufferLength;
@@ -666,7 +631,7 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
     exit(0);
 }
 
-void communicationErrorNotif(unsigned int assocID, unsigned short status, void* dummy)
+void communicationErrorNotif(unsigned int assocID, int status, void* dummy)
 {
     sprintf(statusInfo, " Association ID =%-8x: Communication error (status %u)\n", assocID, status);
     waddstr(statusWin,statusInfo);
@@ -750,11 +715,12 @@ void stdinCallback(int fd, short int revents, short int* gotEvents, void* dummy)
 	if (outstreamID == statusUpdate.noOfOutStreams)
 	  outstreamID = 0;
 
-    sctp_send(associationID, outstreamID, (unsigned char *)buffer, strlen(buffer),
+    sctp_send(associationID, outstreamID, buffer, strlen(buffer),
                   SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
-                  SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, 
-                  SCTP_INFINITE_LIFETIME, SCTP_ORDERED_DELIVERY, 
-		  SCTP_BUNDLING_DISABLED);
+                  SCTP_USE_PRIMARY, 
+                  SCTP_INFINITE_LIFETIME,
+                  SCTP_NO_CONTEXT, SCTP_ORDERED_DELIVERY,
+        		  SCTP_BUNDLING_DISABLED);
 
 	/* Update the Outstream ID list in Monitor tool structure */
 	if (statusUpdate.OutstreamIDList[outstreamID] == -1)
@@ -794,7 +760,7 @@ void stdinCallback(int fd, short int revents, short int* gotEvents, void* dummy)
 	chosenPath = key - 48;
 	displayPathDetails = 1;
 	sctp_getAssocStatus(associationID,&assocStatus);
-	if (chosenPath < assocStatus.numberOfAddresses)
+	if (chosenPath < assocStatus.numberOfDestinationPaths)
 	  ncurses_display_PathStatus(associationID);
 	break;
 
