@@ -1,56 +1,47 @@
 /*
- * --------------------------------------------------------------------------
- *
- *           //=====   //===== ===//=== //===//  //       //   //===//
- *          //        //         //    //    // //       //   //    //
- *         //====//  //         //    //===//  //       //   //===<<
- *              //  //         //    //       //       //   //    //
- *       ======//  //=====    //    //       //=====  //   //===//
- *
- * -------------- An SCTP implementation according to RFC 4960 --------------
+ *  $Id: sctp_wrapper.c,v 1.3 2003/04/08 09:38:32 ajung Exp $
  *
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
- * Copyright (C) 2001-2004 Andreas Jungmaier
- * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Acknowledgements:
- * Realized in co-operation between Siemens AG and the University of
- * Duisburg-Essen, Institute for Experimental Mathematics, Computer
- * Networking Technology group.
- * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany
- * (Förderkennzeichen 01AK045).
- * The authors alone are responsible for the contents.
+ * Realized in co-operation between Siemens AG
+ * and University of Essen, Institute of Computer Networking Technology.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * There are two mailinglists available at http://www.sctp.de which should be
+ * used for any discussion related to this implementation.
+ *
+ * Contact: discussion@sctp.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
+ *
+ * wrapper for SCTPLIB functions. Outputs appropriate error messages
  *
- * Contact: sctp-discussion@sctp.de
- *          thomas.dreibholz@gmail.com
- *          tuexen@fh-muenster.de
- *          andreas.jungmaier@web.de
  */
 
+
 #include "sctp_wrapper.h"
+#include <stdio.h>
+#include <stdlib.h>
 
 #define POLLIN     0x001
 #define POLLPRI    0x002
 #define POLLOUT    0x004
 #define POLLERR    0x008
 
-#include <stdio.h>
-#include <stdlib.h>
-
 int 
 SCTP_setLibraryParameters(SCTP_LibraryParameters *params)
 {
@@ -112,7 +103,7 @@ SCTP_initLibrary(void)
         exit(result);
     }
     return result;
-}
+};
 
 int
 SCTP_registerInstance(unsigned short port,
@@ -141,9 +132,9 @@ SCTP_registerInstance(unsigned short port,
         exit(result);
     }
     return result;
-}
+};
 
-int SCTP_unregisterInstance(unsigned short instance_name)
+int SCTP_unregisterInstance(int instance_name)
 {
     int result;
     
@@ -152,7 +143,7 @@ int SCTP_unregisterInstance(unsigned short instance_name)
         fflush(stderr);        
     }
     return result;
-}
+};
 
 int
 SCTP_deleteAssociation(unsigned int associationID)
@@ -172,22 +163,22 @@ SCTP_deleteAssociation(unsigned int associationID)
         fflush(stderr);
     }
     return result;
-}
+};
 
 int 
 SCTP_send(unsigned int associationID, unsigned short streamID,
-          unsigned char *buffer, unsigned int length, unsigned int protocolId, short path_id,
+          unsigned char *buffer, unsigned int length, unsigned int protocolId, unsigned int path_id,
           void*  context, 
           unsigned int lifetime,
           int unorderedDelivery, 
           int dontBundle)     
 {
     int result;
-    
+
     if ((result = sctp_send(associationID, streamID,
                            buffer, length, protocolId, path_id,
-                           context, 
                            lifetime,
+                           context,
                            unorderedDelivery, 
                            dontBundle))!= SCTP_SUCCESS) {
         if (result == SCTP_QUEUE_EXCEEDED) {
@@ -213,11 +204,11 @@ SCTP_send(unsigned int associationID, unsigned short streamID,
         fflush(stderr);
     }
     return result;
-}
+};
 
 int
 SCTP_receive(unsigned int associationID, unsigned short streamID, unsigned char *buffer, 
-			unsigned int *length, unsigned short* streamSN, unsigned int * tsn, unsigned int flags)
+            unsigned int *length, unsigned short* streamSN, unsigned int * tsn, unsigned int flags)
 {
     int result;
 
@@ -240,7 +231,7 @@ SCTP_receive(unsigned int associationID, unsigned short streamID, unsigned char
         fflush(stderr);
     }
     return result;
-}
+};
 
 int
 SCTP_shutdown(unsigned int associationID)
@@ -257,14 +248,14 @@ SCTP_shutdown(unsigned int associationID)
         fflush(stderr);
     }
     return result;
-}
+};
 
 int 
 SCTP_abort(unsigned int associationID)
 {
     int result;
     
-    if ((result = sctp_abort(associationID)) != SCTP_SUCCESS) {
+    if ((result = sctp_abort(associationID, 0, NULL)) != SCTP_SUCCESS) {
         if (result == SCTP_LIBRARY_NOT_INITIALIZED) {
             fprintf(stderr, "sctp_abort: library not initialized.\n");
         } else
@@ -274,7 +265,7 @@ SCTP_abort(unsigned int associationID)
         fflush(stderr);
     }
     return result;
-}
+};
 
 int SCTP_getAssocStatus(unsigned int associationID, SCTP_AssociationStatus* status)
 {
@@ -295,7 +286,7 @@ int SCTP_getAssocStatus(unsigned int associationID, SCTP_AssociationStatus* stat
         fflush(stderr);
     }
     return result;
-}
+};
 
 int
 SCTP_receiveUnacked(unsigned int associationID, unsigned char *buffer, unsigned int *length, unsigned int* tsn,
@@ -324,7 +315,7 @@ SCTP_receiveUnacked(unsigned int associationID, unsigned char *buffer, unsigned
         fflush(stderr);
     }
     return result;
-}
+};
 
 int
 SCTP_receiveUnsent(unsigned int associationID, unsigned char *buffer, unsigned int *length, unsigned int* tsn,
@@ -352,14 +343,14 @@ SCTP_receiveUnsent(unsigned int associationID, unsigned char *buffer, unsigned i
         fflush(stderr);
     }
     return result;
-}
+};
 
-short
-SCTP_setPrimary(unsigned int associationID, short path_id)
+int
+SCTP_setPrimary(unsigned int associationID, unsigned int pathKey)
 {
     int result;
     
-    if ((result = sctp_setPrimary(associationID, path_id)) != 0) {
+    if ((result = sctp_setPrimary(associationID, pathKey)) != 0) {
         if (result == SCTP_LIBRARY_NOT_INITIALIZED) {
             fprintf(stderr, "sctp_setPrimary: library not initialized.\n");
         } else
@@ -383,10 +374,10 @@ SCTP_setPrimary(unsigned int associationID, short path_id)
     return result;
 }
 
-short
+unsigned int
 SCTP_getPrimary(unsigned int associationID)
 {
-    int result;
+    unsigned int result;
     
     if ((result = sctp_getPrimary(associationID)) < 0) {
         fprintf(stderr, "sctp_setPrimary: error value (%i) returned.\n", result);
@@ -409,29 +400,29 @@ SCTP_eventLoop(void)
         fflush(stderr);
     }
     return result;
-}
+};
 
 int 
-SCTP_getPathStatus(unsigned int associationID, short path_id, SCTP_PathStatus* status)
+SCTP_getPathStatus(unsigned int associationID, unsigned int path_id, SCTP_PathStatus* status)
 {
     int result;
     
     if ((result = sctp_getPathStatus(associationID, path_id, status)) != 0) {
         if (result == SCTP_LIBRARY_NOT_INITIALIZED) {
-            fprintf(stderr, "sctp_getP: library not initialized.\n");
+            fprintf(stderr, "sctp_getPathStatus: library not initialized.\n");
         } else
         if (result == SCTP_ASSOC_NOT_FOUND) {
-            fprintf(stderr, ": association not found.\n");
+            fprintf(stderr, "sctp_getPathStatus: association not found.\n");
         } else
         if (result == SCTP_PARAMETER_PROBLEM) {
-            fprintf(stderr, ": Path Id invalid.\n");
+            fprintf(stderr, "sctp_getPathStatus: Path Id invalid.\n");
         } else {
             fprintf(stderr, "sctp_getPathStatus: error value (%i) returned.\n", result);
             fflush(stderr);
         }
     }
     return result;
-}
+};
 
 unsigned int
 SCTP_associate(unsigned short SCTP_InstanceName,
@@ -451,57 +442,19 @@ SCTP_associate(unsigned short SCTP_InstanceName,
         fflush(stderr);        
     }
     return result;
-}
+};
 
-
-int SCTP_changeHeartBeat(unsigned int associationID,
-                         short path_id, int heartbeatON, unsigned int timeInterval)
-{
-    int result;
-    
-    if ((result = sctp_changeHeartBeat(associationID, path_id, heartbeatON, timeInterval)) < 0) {
-        fprintf(stderr, "sctp_changeHeartBeat: an error occured.\n");
-        fflush(stderr);        
-    }
-    return result;        
-}
-
-int
-SCTP_registerStdinCallback(sctp_StdinCallback sdf, char* buffer, int length)
-{
-    int result;
-    
-    if ((result = sctp_registerStdinCallback(sdf, buffer, length)) < 0) {
-        fprintf(stderr, "sctp_registerStdinCallback: error value (%i) returned.\n", result);
-        fflush(stderr);
-    }
-    return result;
-}
-
-int SCTP_unregisterStdinCallback()
-{
-    int result;
-    
-    if ((result = sctp_unregisterStdinCallback()) < 0) {
-        fprintf(stderr, "sctp_unregisterStdinCallback: error value (%i) returned.\n", result);
-        fflush(stderr);
-    }
-    return result;
-}
-
-#ifndef WIN32
 int
 SCTP_registerUserCallback(int fd, sctp_userCallback sdf, void* userData)
 {
     int result;
     
-    if ((result = sctp_registerUserCallback(fd, sdf, userData, POLLIN|POLLPRI)) < 0) {
+    if ((result = sctp_registerUserCallback(fd, sdf, userData, POLLIN|POLLPRI)) == 0) {
         fprintf(stderr, "sctp_registerUserCallback: error value (%i) returned.\n", result);
         fflush(stderr);
     }
     return result;
-}
-#endif
+};
 
 int 
 SCTP_getAssocDefaults(unsigned short SCTP_InstanceName, SCTP_InstanceParameters* params)
@@ -513,7 +466,7 @@ SCTP_getAssocDefaults(unsigned short SCTP_InstanceName, SCTP_InstanceParameters*
         fflush(stderr);
     }
     return result;
-}
+};
 
 int 
 SCTP_setAssocDefaults(unsigned short SCTP_InstanceName, SCTP_InstanceParameters* params)
@@ -525,12 +478,12 @@ SCTP_setAssocDefaults(unsigned short SCTP_InstanceName, SCTP_InstanceParameters*
         fflush(stderr);
     }
     return result;
-}
+};
 
 unsigned int
 SCTP_startTimer(unsigned int milliseconds, sctp_timerCallback timer_cb,
                 void *param1, void *param2)
 {    
      return sctp_startTimer(milliseconds/1000, (milliseconds%1000)*1000, timer_cb, param1, param2);
-}
+};
 
