@@ -1,47 +1,43 @@
 /*
- * --------------------------------------------------------------------------
- *
- *           //=====   //===== ===//=== //===//  //       //   //===//
- *          //        //         //    //    // //       //   //    //
- *         //====//  //         //    //===//  //       //   //===<<
- *              //  //         //    //       //       //   //    //
- *       ======//  //=====    //    //       //=====  //   //===//
- *
- * -------------- An SCTP implementation according to RFC 4960 --------------
+ *  $Id: sctp_wrapper.h,v 1.1.1.1 2002/11/20 14:51:17 ajung Exp $
  *
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
- * Copyright (C) 2001-2004 Andreas Jungmaier
- * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Acknowledgements:
- * Realized in co-operation between Siemens AG and the University of
- * Duisburg-Essen, Institute for Experimental Mathematics, Computer
- * Networking Technology group.
- * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany
- * (Förderkennzeichen 01AK045).
- * The authors alone are responsible for the contents.
+ * Realized in co-operation between Siemens AG
+ * and University of Essen, Institute of Computer Networking Technology.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * There are two mailinglists available at http://www.sctp.de which should be
+ * used for any discussion related to this implementation.
+ *
+ * Contact: discussion@sctp.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
+ *
+ * wrapper for SCTPLIB functions. Outputs appropriate error messages
  *
- * Contact: sctp-discussion@sctp.de
- *          thomas.dreibholz@gmail.com
- *          tuexen@fh-muenster.de
- *          andreas.jungmaier@web.de
  */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+
+#include <sctp.h>
 
-#include "../sctp/sctp.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -66,21 +62,21 @@ SCTP_registerInstance(unsigned short port,
                       SCTP_ulpCallbacks ULPcallbackFunctions);
 
 int
-SCTP_unregisterInstance(unsigned short instance_name);
+SCTP_unregisterInstance(int instance_name);
 
 int
 SCTP_deleteAssociation(unsigned int associationID);
 
 int 
 SCTP_send(unsigned int associationID, unsigned short streamID,
-          unsigned char *buffer, unsigned int length, unsigned int protocolId, short path_id,
+          unsigned char *buffer, unsigned int length, unsigned int protocolId, unsigned int path_id,
           void*  context, 
           unsigned int lifetime,
           int unorderedDelivery, 
           int dontBundle);
 int
 SCTP_receive(unsigned int associationID, unsigned short streamID, unsigned char *buffer,
-			unsigned int *length, unsigned short* streamSN, unsigned int * tsn, unsigned int flags);
+             unsigned int *length, unsigned short* streamSN, unsigned int * tsn, unsigned int flags);
 
 int
 SCTP_shutdown(unsigned int associationID);
@@ -88,10 +84,10 @@ SCTP_shutdown(unsigned int associationID);
 int
 SCTP_abort(unsigned int associationID);
 
-short 
-SCTP_setPrimary(unsigned int associationID, short path_id);
+int 
+SCTP_setPrimary(unsigned int associationID, unsigned int pathKey);
 
-short
+unsigned int
 SCTP_getPrimary(unsigned int associationID);
 
 int
@@ -105,7 +101,7 @@ int
 SCTP_eventLoop(void);
 
 int 
-SCTP_getPathStatus(unsigned int associationID, short path_id, SCTP_PathStatus* status);
+SCTP_getPathStatus(unsigned int associationID, unsigned int path_id, SCTP_PathStatus* status);
 
 int
 SCTP_getAssocStatus(unsigned int associationID, SCTP_AssociationStatus* status);
@@ -113,23 +109,11 @@ SCTP_getAssocStatus(unsigned int associationID, SCTP_AssociationStatus* status);
 unsigned int
 SCTP_associate(unsigned short SCTP_InstanceName,
                unsigned short noOfOutStreams,
-               unsigned char destinationAddress[],
+               unsigned char  destinationAddress[],
                unsigned short destinationPort,
                void* ulp_data);
-
-
-int
-SCTP_registerStdinCallback(sctp_StdinCallback sdf, char* buffer, int length);
-
-int SCTP_unregisterStdinCallback();
-
-int SCTP_changeHeartBeat(unsigned int associationID,
-                         short path_id, int heartbeatON, unsigned int timeIntervall);
-
-#ifndef WIN32
 int
 SCTP_registerUserCallback(int fd, sctp_userCallback sdf, void* userData);
-#endif
 
 int
 SCTP_getAssocDefaults(unsigned short SCTP_InstanceName, SCTP_InstanceParameters* params);
