@@ -1,5 +1,5 @@
 /*
- *  $Id: adaptation.h 532 2004-11-17 21:11:53Z tuexen $
+ *  $Id: adaptation.h,v 1.5 2004/01/13 16:13:14 ajung Exp $
  *
  * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
@@ -30,7 +30,7 @@
  * used for any discussion related to this implementation.
  *
  * Contact: discussion@sctp.de
- *          tuexen@fh-muenster.de
+ *          Michael.Tuexen@icn.siemens.de
  *          ajung@exp-math.uni-essen.de
  *
  * Purpose: This headerfile defines the interface to the adaption-module.
@@ -55,11 +55,11 @@
 #include "distribution.h"
 
 
-unsigned int adl_random(void);
-
 boolean adl_equal_address(union sockunion *one, union sockunion *two);
 
 
+unsigned int adl_random(void);
+
 /**
  *  converts address-string (hex for ipv6, dotted decimal for ipv4
  *  to a sockunion structure
@@ -70,8 +70,6 @@ int adl_str2sockunion(guchar * str, union sockunion *su);
 int adl_sockunion2str(union sockunion *su, guchar * buf, size_t len);
 
 
-
-
 /*------------------------------------------------------------------------------------------------------------------*/
 /*------------------------------------------------------------------------------------------------------------------*/
 
@@ -99,7 +97,7 @@ gint adl_get_sctpv6_socket(void);
  * @param	dest_len size of the address
  * @return returns number of bytes actually sent, or error
  */
-int adl_send_message(int sfd, void *buf, int len, union sockunion *dest, unsigned char tos);
+int adl_send_message(int sfd, void *buf, int len, union sockunion *dest, unsigned char tos, gboolean dontFragment);
 
 
 /**
@@ -159,10 +157,6 @@ int adl_unregisterUdpCallback(int udp_sfd);
 int adl_sendUdpData(int sfd, unsigned char* buf, int length,
                      unsigned char destination[], unsigned short dest_port);
 
-
-int adl_registerStdinCallback(sctp_StdinCallback sdf, char* buffer, int length);
-int adl_unregisterStdinCallback();
-
 int adl_registerUserCallback(int fd, sctp_userCallback sdf, void* userData, short int eventMask);
 
 int adl_unregisterUserCallback(int fd);
@@ -199,7 +193,17 @@ gboolean adl_gatherLocalAddresses(union sockunion **localAddresses,
      int sctp_fd,
      gboolean with_ipv6,
      int *max_mtu,
+     int *min_mtu,
      const AddressScopingFlags  flags);
 
 
+#ifdef DO_PMTUD
+int adl_open_icmp_socket(void);
+void adl_icmp_cb(int sfd, unsigned char *buffer, int buflen, union sockunion *src, int src_len);
+#ifdef HAVE_IPV6
+int adl_open_icmpv6_socket(void);
+void adl_icmpv6_cb(int sfd, unsigned char *buffer, int buflen, union sockunion *src, int src_len);
+#endif /* HAVE IP_V6 */
+#endif /* DO_PMTUD */
+
 #endif
