@@ -1,5 +1,5 @@
 /*
- *  $Id: errorhandler.c 535 2004-11-17 23:04:09Z tuexen $
+ *  $Id: errorhandler.c,v 1.1.1.1 2002/11/20 14:51:16 ajung Exp $
  *
  * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
@@ -12,6 +12,7 @@
  * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -30,7 +31,7 @@
  * used for any discussion related to this implementation.
  *
  * Contact: discussion@sctp.de
- *          tuexen@fh-muenster.de
+ *          Michael.Tuexen@icn.siemens.de
  *          ajung@exp-math.uni-essen.de
  *
  * Purpose: This module decodes error chunks and handles them accordingly.
@@ -42,7 +43,7 @@
 #include "globals.h"            /* for chunk struct definition */
 #include "chunkHandler.h"
 #include "bundling.h"
-#include "SCTP-control.h"
+#include "sctp-control.h"
 
 
 void eh_init_errorhandler(void)
@@ -72,7 +73,7 @@ void eh_delete(void *instancePtr)
  *  accordingly....
  *  @return  error code, 0 for success, less than one for error
  */
-int eh_recv_chunk(SCTP_simple_chunk * errchunk)
+int eh_recv_chunk(Association* asok, SCTP_simple_chunk * errchunk)
 {
     SCTP_error_chunk *chunk;
     SCTP_vlparam_header *header;
@@ -98,7 +99,7 @@ int eh_recv_chunk(SCTP_simple_chunk * errchunk)
         break;
     case ECC_STALE_COOKIE_ERROR:
         event_logi(EXTERNAL_EVENT, "Stale Cookie Error, Len %u ", cause_len);
-        sctlr_staleCookie((SCTP_simple_chunk *) errchunk);
+        scr_staleCookie(asok, (SCTP_simple_chunk *) errchunk);
         result = 0;
         break;
     case ECC_OUT_OF_RESOURCE_ERROR:
@@ -118,7 +119,7 @@ int eh_recv_chunk(SCTP_simple_chunk * errchunk)
         header = (SCTP_vlparam_header*)data;
         if (ntohs(header->param_type) == VLPARAM_PRSCTP) {
             /* set peer does not understand PRSCTP - do not use it in this ASSOC ! */
-            event_log(EXTERNAL_EVENT, "Unrecognized Parameter: PR_SCTP ");
+            event_log(EXTERNAL_EVENT, "Unrecognized Parameter: PRSCTP ");
         }
         break;
     case ECC_NO_USER_DATA:
@@ -137,24 +138,15 @@ int eh_recv_chunk(SCTP_simple_chunk * errchunk)
  * function to trigger sending of error chunk, after receiving an invalid stream id
  * @return error value, 0 on success, -1 on error
  */
-int eh_make_invalid_streamid_error(unsigned short streamid)
+int eh_send_invalid_streamid(Association* asok, unsigned short streamid)
 {
-    ChunkID errorCID;
-    SCTP_InvalidStreamIdError error_info;
-    
-    /* build chunk */
-    errorCID = ch_makeErrorChunk();
+    return -1;
+}
 
-    error_info.stream_id = htons(streamid);
-    error_info.reserved = htons(0);
 
-    /* add parameters */
-    ch_enterErrorCauseData(errorCID, ECC_INVALID_STREAM_ID, 4, (unsigned char*)&error_info);
-
-    bu_put_Ctrl_Chunk(ch_chunkString(errorCID),NULL);
-    ch_deleteChunk(errorCID);
-    
-    return 0;
+int eh_send_invalid_chunk(Association* asok)
+{
+    return -1;
 }
 
 
@@ -162,26 +154,24 @@ int eh_make_invalid_streamid_error(unsigned short streamid)
  * function to put an error_chunk with type UNKNOWN PARAMETER
  * @return error value, 0 on success, -1 on error
  */
-int eh_send_unrecognized_chunktype(unsigned char* faulty_chunk, unsigned short length)
+int eh_send_unrecognized_chunktype(Association* asok, unsigned char* faulty_chunk, unsigned short length)
 {
     ChunkID errorCID;
 
     /* build chunk */
     errorCID = ch_makeErrorChunk();
     /* add parameters */
-    ch_enterErrorCauseData(errorCID, ECC_UNRECOGNIZED_CHUNKTYPE, length, (unsigned char*)faulty_chunk);
+    ch_enterErrorCauseData(errorCID, ECC_UNRECOGNIZED_CHUNKTYPE, length, faulty_chunk);
 
-    bu_put_Ctrl_Chunk(ch_chunkString(errorCID),NULL);
-    ch_deleteChunk(errorCID);
-    
-    return bu_sendAllChunks(NULL);
+    bu_put_Ctrl_Chunk(asok, ch_chunkString(errorCID), 0);
+    return bu_sendAllChunks(asok, 0);
 }
 
 /**
  * function to trigger sending of error chunk, after mandatory parameter(s) was(were) missing
  * @return error value, 0 on success, -1 on error
  */
-int eh_make_missing_mandatory_param(unsigned int number, unsigned short *param_types)
+int eh_send_missing_mandatory_param(unsigned int number, unsigned short *param_types)
 {
     return -1;
 }
@@ -196,26 +186,3 @@ int eh_send_unresolvable_address(unsigned int number, unsigned char *addresses)
 {
     return -1;
 }
-
-
-/**
- * function to add an error chunk, after empty data chunk was received
- * @return error value, 0 on success, -1 on error
- */
-int eh_make_empty_data_chunk_error(unsigned int tsn)
-{
-    ChunkID errorCID;
-
-    /* build chunk */
-    errorCID = ch_makeErrorChunk();
-
-    /* add parameters */
-    ch_enterErrorCauseData(errorCID, ECC_NO_USER_DATA, sizeof(unsigned int), (unsigned char*)&tsn);
-
-    bu_put_Ctrl_Chunk(ch_chunkString(errorCID),NULL);
-    ch_deleteChunk(errorCID);
-
-    return 0;
-}
-
-
