@@ -1,62 +1,63 @@
 /*
- * --------------------------------------------------------------------------
- *
- *           //=====   //===== ===//=== //===//  //       //   //===//
- *          //        //         //    //    // //       //   //    //
- *         //====//  //         //    //===//  //       //   //===<<
- *              //  //         //    //       //       //   //    //
- *       ======//  //=====    //    //       //=====  //   //===//
- *
- * -------------- An SCTP implementation according to RFC 4960 --------------
+ *  $Id: globals.c,v 1.2 2004/01/13 16:13:14 ajung Exp $
  *
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
- * Copyright (C) 2001-2004 Andreas Jungmaier
- * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Acknowledgements:
- * Realized in co-operation between Siemens AG and the University of
- * Duisburg-Essen, Institute for Experimental Mathematics, Computer
- * Networking Technology group.
- * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany
- * (Förderkennzeichen 01AK045).
+ * Realized in co-operation between Siemens AG
+ * and University of Essen, Institute of Computer Networking Technology.
+ *
+ * Acknowledgement
+ * This work was partially funded by the Bundesministerium fr Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * Contact: sctp-discussion@sctp.de
- *          thomas.dreibholz@gmail.com
- *          tuexen@fh-muenster.de
- *          andreas.jungmaier@web.de
+ * There are two mailinglists available at http://www.sctp.de which should
+ * be used for any discussion related to this implementation.
+ *
+ * Contact: discussion@sctp.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
+ *
+ * Purpose: This header-file defines types that must be known most
+ *          modules of the SCTP-project.
  */
 
 #include "globals.h"
 #include "adaptation.h"
+
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
-#include <time.h>
-#include <sys/types.h>
-        
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 
 #ifdef WIN32
-#include <process.h>
+    #include <process.h>
 #endif
 
+#ifdef HAVE_UNISTD_H
+    #include <unistd.h>
+#endif
+
+
 
 boolean globalTrace;
 boolean fileTrace = FALSE;
@@ -95,7 +96,6 @@ int sort_prChunk(pr_stream_data* one, pr_stream_data* two)
     return 0;
 }
 
-
 /**
    read_tracelevels reads from a file the tracelevels for errors and events for each module.
    Modules that are not listed in the file will not be traced. if the file does not exist or
@@ -128,41 +128,32 @@ void read_tracelevels()
         globalTrace = TRUE;
 
         for (i = 0; i < 50; i++) {
-            if(fscanf(fptr, "%s %d %d", tracedModules[i], &errorTraceLevel[i], &eventTraceLevel[i]) >= 1) {
-               if (strcmp(tracedModules[i], "LOGFILE") == 0) {
-                   /*
-                   printf("Logging all errors and events to file ./tmp%d.log\n", (int)getpid());
-                   */
-                   fileTrace = TRUE;
-                   sprintf(filename, "./tmp%d.log",(int)getpid());
-                   logfile = fopen(filename, "w+");
-                   return;
-               }
+            fscanf(fptr, "%s %d %d", tracedModules[i], &errorTraceLevel[i], &eventTraceLevel[i]);
+            if (strcmp(tracedModules[i], "LOGFILE") == 0) {
+                fileTrace = TRUE;
+                sprintf(filename, "./tmp%d.log",(int)getpid());
+                logfile = fopen(filename, "w+");
+                return;
             }
             if (ferror(fptr))
-                abort();
+                exit(-1);
             if (feof(fptr))
                 break;
             globalTrace = FALSE;
         }
         noOftracedModules = i;
         if (i<= 1) globalTrace = TRUE;
-        /*
-        printf("  globalTrace = %s \n",  (globalTrace==TRUE)?"TRUE":"FALSE");
-        */
     } else {
         globalTrace = TRUE; /* ??? */
     }
-    /*
-    printf("global = %d, #of modules = %d\n", (int) globalTrace, noOftracedModules);
-    for (i = 0; i < noOftracedModules; i++)
-      printf("%20s %2d %2d\n", tracedModules[i], errorTraceLevel[i], eventTraceLevel[i]);
-    */
+    /* printf("global = %d, #of modules = %d\n", (int) globalTrace, noOftracedModules);
+       for (i = 0; i < noOftracedModules; i++)
+       printf("%20s %2d %2d\n", tracedModules[i], errorTraceLevel[i], eventTraceLevel[i]);  */
 }
 
 
 
-boolean traceModule(const char *moduleName, int *moduleIndex)
+boolean traceModule(char *moduleName, int *moduleIndex)
 {
     int i;
     boolean found;
@@ -183,7 +174,7 @@ boolean traceModule(const char *moduleName, int *moduleIndex)
 
 
 
-int debug_vwrite(FILE * fd, const char *format, va_list ap)
+int debug_vwrite(FILE * fd, char *format, va_list ap)
 {
     struct timeval tv;
     struct tm *the_time;
@@ -201,7 +192,7 @@ int debug_vwrite(FILE * fd, const char *format, va_list ap)
 }
 
 
-void debug_print(FILE * fd, const char *f, ...)
+void debug_print(FILE * fd, char *f, ...)
 {
     va_list va;
     va_start(va, f);
@@ -216,7 +207,7 @@ void debug_print(FILE * fd, const char *f, ...)
 void perr_exit(const char *infostring)
 {
     perror(infostring);
-    abort();
+    exit(-1);
 }
 
 
@@ -225,6 +216,7 @@ void print_time(short level)
     struct timeval now;
 
     adl_gettime(&now);
+
     event_logii(level, "Time now: %ld sec, %ld usec \n", now.tv_sec, now.tv_usec);
 }
 
@@ -238,7 +230,7 @@ void print_time(short level)
    @param anyno            optional pointer to unsigned int, which is printed along with log_info.
                             The conversion specification must be contained in log_info.
 */
-void event_log1(short event_log_level, const char *module_name, const char *log_info, ...)
+void event_log1(short event_log_level, char *module_name, char *log_info, ...)
 {
     int mi;
     struct timeval tv;
@@ -265,17 +257,20 @@ void event_log1(short event_log_level, const char *module_name, const char *log_
             fprintf(logfile, "%02d:%02d:%02d.%03d - ",
                     the_time->tm_hour, the_time->tm_min, the_time->tm_sec, (int) (tv.tv_usec / 1000));
             vfprintf(logfile, log_info, va);
-            fprintf(logfile, "\n");
+            va_end(va);
+            fflush(logfile);
+            fprintf(logfile, " \n");
             fflush(logfile);
         } else {
             fprintf(stdout, "%02d:%02d:%02d.%03d - ",
                     the_time->tm_hour, the_time->tm_min, the_time->tm_sec, (int) (tv.tv_usec / 1000));
             vfprintf(stdout, log_info, va);
-            fprintf(stdout, "\n");
+            va_end(va);
+            fflush(logfile);
+            fprintf(stdout, " \n");
             fflush(stdout);
         }
     }
-    va_end(va);
     return;
 }
 
@@ -292,7 +287,7 @@ void event_log1(short event_log_level, const char *module_name, const char *log_
                            The conversion specification must be contained in log_info.
 
 */
-void error_log1(short error_log_level, const char *module_name, int line_no, const char *log_info, ...)
+void error_log1(short error_log_level, char *module_name, int line_no, char *log_info, ...)
 {
     int mi;
     va_list va;
@@ -327,7 +322,9 @@ void error_log1(short error_log_level, const char *module_name, int line_no, con
        fflush(stderr);
     }
     if (error_log_level == ERROR_FATAL) {
-        abort();
+	 fprintf(stderr, "Fatal Error condition ocurred, library terminates....SORRY !!!! \n");
+         fflush(stderr);
+         exit(EXIT_FAILURE);
     }
 }
 
@@ -339,12 +336,54 @@ void error_log1(short error_log_level, const char *module_name, int line_no, con
    @param errnumber        the errno from systemlibrary.
    @param log_info         the info that is printed with the modulename and error text.
 */
-void error_log_sys1(short error_log_level, const char *module_name, int line_no, short errnumber)
+void error_log_sys1(short error_log_level, char *module_name, int line_no, short errnumber)
 {
     error_log1(error_log_level, module_name, line_no, strerror(errnumber));
 }
 
 
+
+/* this function is taken from the reference implementation written 
+   by Randy Stewart. I did not want to reinvent the wheel :-) */
+unsigned short
+in_check(unsigned char *buf, int sz)
+{
+  unsigned short *psh;
+  unsigned long sum;
+  int i;
+  
+  sum = 0;
+  psh = (unsigned short *)buf;
+
+  if(sz & 0x1){
+    /* odd byte boundary pad up 1 byte */
+    /* this cheats, but it should be ok :-> */
+    buf[sz] = 0;
+    sz++;
+  }
+  sz /= 2;
+  for(i=0;i<sz;i++){
+    sum += *psh++;
+  }
+  sum = (sum >> 16) + (sum & 0xffff);
+  sum += (sum >> 16);
+  return((unsigned short) (~sum & 0x0000ffff));
+}
+
+/**
+ * functions correctly handle wraparound
+ */
+int sBefore(unsigned short seq1, unsigned short seq2)
+{
+    return (int)((short) (seq1 - seq2) < 0);
+}
+
+int sAfter(unsigned short seq1, unsigned short seq2)
+{
+    return (int)((short)(seq2 - seq1)) < 0;
+}
+
+
 /**
  * functions correctly handle wraparound
  */
@@ -358,17 +397,6 @@ int after(unsigned int seq1, unsigned int seq2)
     return (int) (seq2 - seq1) < 0;
 }
 
-int sAfter(unsigned short seq1, unsigned short seq2)
-{
-    return (int)((short)(seq2 - seq1)) < 0;
-}
-
-int sBefore(unsigned short seq1, unsigned short seq2)
-{
-    return (int)((short) (seq1 - seq2) < 0);
-}
-
-
 
 /**
  *  is s1 <= s2 <= s3 ?
