@@ -1,5 +1,5 @@
 /*
- *  $Id: globals.h 686 2005-08-04 10:49:29Z dreibh $
+ *  $Id: globals.h,v 1.17 2004/01/19 16:08:30 rohde Exp $
  *
  * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
@@ -30,7 +30,7 @@
  * used for any discussion related to this implementation.
  *
  * Contact: discussion@sctp.de
- *          tuexen@fh-muenster.de
+ *          Michael.Tuexen@icn.siemens.de
  *          ajung@exp-math.uni-essen.de
  *
  * Purpose: This header-file defines types that must be known most
@@ -38,6 +38,8 @@
  *          for Solaris 8 and FreeBSD 4.2
  */
 
+#define __EXTENSIONS__
+
 #ifndef GLOBALS_H
 #define GLOBALS_H
 
@@ -52,6 +54,7 @@
 #include <stdlib.h>
 
 
+
 #ifdef  STDC_HEADERS
  #ifdef  HAVE_SYS_TIME_H
   #include <sys/time.h>
@@ -64,11 +67,6 @@
  #endif
 #endif
 
-#ifdef WIN32
-#include <winsock2.h>
-#include <time.h>
-#endif
-
 #ifdef FreeBSD
 #include <netinet/in_systm.h>
 #include <sys/types.h>
@@ -79,22 +77,27 @@
 #include <stdarg.h>
 #endif
 
+#include <netinet/in.h>
+
 
 /* turn on  Posix 1g for compatible cmsg structure */
-/* #ifdef USE_RFC2292BIS
-     #define _XPG4_2
-  #endif
-*/
-
-#ifndef WIN32
-#include <sys/socket.h>
-#include <netinet/in.h>
+#ifdef USE_RFC2292BIS
+   #define _XPG4_2
 #endif
+ 
+#include <sys/socket.h>
+  
+/* turn off  Posix 1g for compatible cmsg structure */
+#ifdef USE_RFC2292BIS
+  #undef _XPG4_2
+#endif
+  
 
 #include "messages.h"
 
+
 /* timer granularity in millliseconds..... */
-#define GRANULARITY		1
+#define GRANULARITY		10
 
 /* Define a protocol id to be used in the IP Header..... */
 #ifndef IPPROTO_SCTP
@@ -102,16 +105,19 @@
 #endif
 
 /** this parameter specifies the maximum number of addresses that an endpoint may have */
-#define MAX_NUM_ADDRESSES      32
+#define MAX_NUM_ADDRESSES      20
 
 
 #define SECRET_KEYSIZE  4096
 #define KEY_INIT     0
-#ifndef KEY_READ
 #define KEY_READ     1
-#endif
+
 #define MAX_DEST 	16
 
+/* Do Path MTUDiscovery */
+#define DO_PMTUD                1
+#define PATH_MTU_COMMON_DIFFERENCE 8
+
 
 /* Definition of levels for the logging of events */
 #define ByteString_log_    0    /* set to != 0 if byte string logging should be done */
@@ -124,7 +130,7 @@
 #define EXTERNAL_EVENT_X   1    /* for unexpected external events from ULP, peer or Timers */
 
 
-#define Current_event_log_ 0    /* Defines the level up to which the events are printed.
+#define Current_event_log_ 0   /* Defines the level up to which the events are printed.
                                    VVERBOSE (6) means all events are printed.
                                    This parameter could also come from a command line option */
 
@@ -135,7 +141,7 @@
                                    for instance abort of an association. */
 #define ERROR_FATAL        1    /* recovery from error was not possible, the program exits. */
 
-#define Current_error_log_ 4    /* Defines the level up to which the errors are printed.
+#define Current_error_log_ 1      /* Defines the level up to which the errors are printed.
                                    ERROR_WARNING (4) means all events are printed.
                                    This parameter could also come from a command line option */
 
@@ -148,7 +154,10 @@ typedef unsigned int TimerID;
 #define   TIMER_TYPE_SACK       2
 #define   TIMER_TYPE_CWND       4
 #define   TIMER_TYPE_HEARTBEAT  5
-#define   TIMER_TYPE_USER       6
+#define   TIMER_TYPE_ASCONF     6
+#define   TIMER_TYPE_USER       7
+#define   TIMER_TYPE_PMTUD      8
+#define   TIMER_TYPE_PMTUD_INCREASE 9
 
 typedef struct chunk_data_struct
 {
@@ -162,34 +171,57 @@ typedef struct chunk_data_struct
     unsigned int num_of_transmissions;
     /* time after which chunk should not be retransmitted */
     struct timeval expiry_time;
-    gboolean dontBundle;
-    /* lst destination used to send chunk to */
-    unsigned int last_destination;
-    int initial_destination;
-    /* this is set to true, whenever chunk is sent/received on unreliable stream */
-    gboolean isUnreliable;
+    boolean dontBundle;
     gboolean hasBeenAcked;
     gboolean hasBeenDropped;
     gboolean hasBeenFastRetransmitted;
-    gboolean hasBeenRequeued;
+    gboolean counts_as_outstanding;
+    /* last destination used to send chunk to */
+    unsigned int lastDestinationKey;
+    unsigned int initialDestinationKey;
     gpointer context;
 } chunk_data;
 
+/**
+ * these are also in <sctp.h>
+ * if you chage them here, be sure to also change them there
+ */
+#ifndef SCTP_MAXIMUM_DATA_LENGTH
+    #define SCTP_MAXIMUM_DATA_LENGTH     1404
+#endif
+
+#ifndef SCTP_MAXIMUM_MESSAGE_LENGTH
+    #define SCTP_MAXIMUM_MESSAGE_LENGTH  32000
+#endif
+
+
+#define SCTP_MAX_NUM_CHUNKS        ((SCTP_MAXIMUM_MESSAGE_LENGTH/SCTP_MAXIMUM_DATA_LENGTH+1))
+
+typedef struct extended_message_vector
+{
+    /* can only take a maximum number of chunks */
+    chunk_data*   cdat[SCTP_MAX_NUM_CHUNKS];
+    unsigned int  numberOfChunks;
+    unsigned int  totalLength;
+}
+message_vector;
+
 #ifndef max
-#define max(x,y)            ((x)>(y))?(x):(y)
+    #define max(x,y)            ((x)>(y))?(x):(y)
 #endif
+
 #ifndef min
-#define min(x,y)            ((x)<(y))?(x):(y)
+    #define min(x,y)            ((x)<(y))?(x):(y)
 #endif
 
+
+
 #define event_log(x,y)        if (Current_event_log_ >= x) event_log1((x), __FILE__, (y))
 #define event_logi(x,y,z)	  if (Current_event_log_ >= x) event_log1((x), __FILE__, (y), (z))
 #define event_logii(x,y,z,i)	  if (Current_event_log_ >= x) event_log1((x), __FILE__, (y), (z), (i))
 #define event_logiii(x,y,z,i,j)	  if (Current_event_log_ >= x) event_log1((x), __FILE__, (y), (z), (i), (j))
 #define event_logiiii(x,y,z,i,j,k)	  if (Current_event_log_ >= x) event_log1((x), __FILE__, (y), (z), (i), (j),(k))
 #define event_logiiiii(x,y,z,i,j,k,l)	  if (Current_event_log_ >= x) event_log1((x), __FILE__, (y), (z), (i), (j),(k),(l))
-#define event_logiiiiiiii(x,y,z,i,j,k,l,m,n,o)	  if (Current_event_log_ >= x) event_log1((x), __FILE__, (y), (z), (i), (j),(k),(l),(m),(n),(o))
-
 
 #define error_log(x,y)        if (Current_error_log_ >= x) error_log1((x), __FILE__, __LINE__, (y))
 #define error_logi(x,y,z)        if (Current_error_log_ >= x) error_log1((x), __FILE__, __LINE__, (y),(z))
@@ -207,7 +239,7 @@ typedef struct chunk_data_struct
    module1.c errorTraceLevel eventTraceLevel
    module2.c errorTraceLevel eventTraceLevel
    ....
-
+   
    The file must be terminated by a null line.
 */
 void read_tracelevels(void);
@@ -216,7 +248,7 @@ void read_tracelevels(void);
 void debug_print(FILE * fd, char *f, ...);
 
 /**
- * function to output the result of the adl_gettime-call, i.e. the time now
+ * function to output the result of the gettimeofday-call, i.e. the time now
  */
 void print_time(short level);
 
@@ -256,7 +288,7 @@ void error_log1(short error_log_level, char *module_name, int line_no, char *log
    @param error_log_level : ERROR_MINOR ERROR_MAJOR ERROR_FATAL
    @param module_name :     the name of the module that received the event.
    @param line_no :         the line number within above module.
-   @param errnumber :       the errno from systemlibrary.
+   @param errnumber :       the errno from systemlibrary.  
    @param log_info :        the info that is printed with the modulename and error text.
    @author     Hlzlwimmer
 */
@@ -268,8 +300,13 @@ void error_log_sys1(short error_log_level, char *module_name, int line_no, short
  */
 int before(unsigned int seq1, unsigned int seq2);
 int after(unsigned int seq1, unsigned int seq2);
-int sAfter(unsigned short seq1, unsigned short seq2);
+
+/**
+ * helper functions that correctly handle the 16bit wrapround
+ */
 int sBefore(unsigned short seq1, unsigned short seq2);
+int sAfter(unsigned short seq1, unsigned short seq2);
+
 
 /**
  *  is s1 <= s2 <= s3 ?
@@ -285,15 +322,16 @@ int between(unsigned int seq1, unsigned int seq2, unsigned int seq3);
  */
 unsigned short in_check(unsigned char *buf, int sz);
 
-
-int sort_prChunk(pr_stream_data* one, pr_stream_data* two);
-
 /*
  * function that correctly sorts TSN values, minding the
  * wrapround
  */
 int sort_tsn(chunk_data * one, chunk_data * two);
 
+
+int sort_prChunk(pr_stream_data* one, pr_stream_data* two);
+
+
 void free_list_element(gpointer list_element, gpointer user_data);
 
 
@@ -321,7 +359,6 @@ union sockunion
 #define sockunion_family(X)  (X)->sa.sa_family
 
 #define SUPPORT_ADDRESS_TYPE_IPV4        0x00000001
-
 #define SUPPORT_ADDRESS_TYPE_IPV6        0x00000002
 #define SUPPORT_ADDRESS_TYPE_DNS         0x00000004
 
@@ -344,8 +381,5 @@ typedef enum  {
 #define DEFAULT_MTU_CEILING     1500
 
 
-#ifndef CHECK
-#define CHECK(cond) if(!(cond)) { fprintf(stderr, "INTERNAL ERROR in %s, line %u: condition %s is not satisfied!\n", __FILE__, __LINE__, #cond); abort(); }
-#endif
 
 #endif
