@@ -1,23 +1,13 @@
 /*
+ *  $Id: md5.h,v 1.2 2004/01/13 16:13:14 ajung Exp $
  *
- * MD5 hash calculation
- * Copyright (C) 1991-1992 RSA Data Security, Inc.
- *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * MD5.H - header file for MD5C.C
  */
 
-/******************************************************************
+/*****************************************************************
+ Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991.
+ All rights reserved.
+
  License to copy and use this software is granted provided that it
  is identified as the "RSA Data Security, Inc. MD5 Message-Digest
  Algorithm" in all material mentioning or referencing this software
@@ -32,10 +22,12 @@
  the merchantability of this software or the suitability of this
  software for any particular purpose. It is provided "as is"
  without express or implied warranty of any kind.
-
  These notices must be retained in any copies of any part of this
  documentation and/or software.
-*******************************************************************/
+******************************************************************/
+
+/* GLOBAL.H - RSAREF types and constants
+ */
 
 /* PROTOTYPES should be set to one if and only if the compiler supports
   function argument prototyping.
@@ -44,16 +36,15 @@
  */
 
 #include <string.h>
-#include <glib.h>
 
 /* POINTER defines a generic pointer type */
 typedef unsigned char *POINTER;
 
 /* UINT2 defines a two byte word */
-typedef guint16 UINT2;
+typedef unsigned short int UINT2;
 
 /* UINT4 defines a four byte word */
-typedef guint32 UINT4;
+typedef unsigned long int UINT4;
 
 /* PROTO_LIST is defined depending on how PROTOTYPES is defined above.
 If using PROTOTYPES, then PROTO_LIST returns the list, otherwise it
