@@ -1,5 +1,5 @@
 /*
- *  $Id: messages.h 654 2005-07-19 09:35:16Z dreibh $
+ *  $Id: messages.h,v 1.8 2004/01/14 15:52:54 rohde Exp $
  *
  * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
@@ -30,12 +30,12 @@
  * used for any discussion related to this implementation.
  *
  * Contact: discussion@sctp.de
- *          tuexen@fh-muenster.de
+ *          Michael.Tuexen@icn.siemens.de
  *          ajung@exp-math.uni-essen.de
  *
  *
  * Purpose: defines all message structures used in SCTP
- *
+ * 
  */
 
 #ifndef MESSAGES_H
@@ -61,18 +61,6 @@
 
 /**************************** SCTP common message definitions *********************************/
 
-#ifdef SCTP_OVER_UDP
-#define SCTP_OVER_UDP_UDPPORT 9899
-#warning Using SCTP over UDP!
-typedef struct UDP_HEADER {
-   gushort src_port;
-   gushort dest_port;
-   gushort length;
-   gushort checksum;
-} udp_header;
-#endif
-
-
 #define MAX_MTU_SIZE              1500
 #define IP_HEADERLENGTH             20
 
@@ -223,6 +211,7 @@ SCTP_data_chunk;
 #define VLPARAM_SUCCESS_REPORT          0xC005
 #define VLPARAM_ADAPTATION_LAYER_IND    0xC006
 
+
 /* Header of variable length parameters */
 typedef struct SCTP_VLPARAM_HEADER
 {
@@ -351,6 +340,8 @@ duplicate;
 
 /*--------------------------- heartbeat chunk defs --------------------------------*/
 
+// #define MAX_HEARTBEAT_PADDING  (MAX_SCTP_PDU - sizeof(SCTP_chunk_header) - sizeof(SCTP_vlparam_header) - 64)
+
 /* our heartbeat chunk structure */
 typedef struct SCTP_HEARTBEAT
 {
@@ -364,7 +355,7 @@ typedef struct SCTP_HEARTBEAT
     guint32 hmac[5];
 #endif
 }
-SCTP_heartbeat;
+SCTP_heartbeat;                                                             
 
 
 /*--------------------------- simple chunk --------------------------------------------------*/
@@ -374,8 +365,8 @@ SCTP_heartbeat;
    CHUNK_SHUTDOWN_ACK
    CHUNK_COOKIE_ACK
    ? CHUNK_COOKIE_ECHO ?
-
-   simple chunk can also be used for transfering chunks to/from bundling, since bundling
+   
+   simple chunk can also be used for transfering chunks to/from bundling, since bundling 
    looks only on the chunk header.
 */
 #define MAX_SIMPLE_CHUNKDATA_LENGTH   (MAX_SCTP_PDU - sizeof(SCTP_chunk_header))
@@ -400,7 +391,6 @@ typedef struct SCTP_FORWARD_TSN_CHUNK
     guchar              variableParams[MAX_SCTP_PDU];
 }
 SCTP_forward_tsn_chunk;
-
 /*--------------------------- parameter definitions ------------------------------------------*/
 
 /**
@@ -485,25 +475,25 @@ typedef struct SCTP_ERROR_CAUSE
 
 /*--------------------------- Error causes definitions ---------------------------------------*/
 /* Error cause codes */
-#define ECC_INVALID_STREAM_ID                   1
-#define ECC_MISSING_MANDATORY_PARAM             2
-#define ECC_STALE_COOKIE_ERROR                  3
-#define ECC_OUT_OF_RESOURCE_ERROR               4
-#define ECC_UNRESOLVABLE_ADDRESS                5
-#define ECC_UNRECOGNIZED_CHUNKTYPE              6
-#define ECC_INVALID_MANDATORY_PARAM             7
-#define ECC_UNRECOGNIZED_PARAMS                 8
-#define ECC_NO_USER_DATA                        9
+#define ECC_INVALID_STREAM_ID           1
+#define ECC_MISSING_MANDATORY_PARAM     2
+#define ECC_STALE_COOKIE_ERROR          3
+#define ECC_OUT_OF_RESOURCE_ERROR       4
+#define ECC_UNRESOLVABLE_ADDRESS        5
+#define ECC_UNRECOGNIZED_CHUNKTYPE      6
+#define ECC_INVALID_MANDATORY_PARAM     7
+#define ECC_UNRECOGNIZED_PARAMS         8
+#define ECC_NO_USER_DATA                9
 #define ECC_COOKIE_RECEIVED_DURING_SHUTDWN      10
 #define ECC_RESTART_WITH_NEW_ADDRESSES          11
-
 #define ECC_USER_INITIATED_ABORT                12
 #define ECC_PROTOCOL_VIOLATION                  13
 
-
-#define ECC_DELETE_LAST_IP_FAILED       0xC
-#define ECC_OP_REFUSED_NO_RESOURCES     0xD
-#define ECC_DELETE_SOURCE_ADDRESS       0xE
+#define ECC_ASCONF_DELETE_LAST_IP_FAILED       0x000B
+#define ECC_ASCONF_OP_REFUSED_NO_RESOURCES     0x000C
+#define ECC_ASCONF_DELETE_SOURCE_ADDRESS       0x000D
+#define ECC_ASCONF_ILLEGAL_ASCONF_ACK          0x000E
+#define ECC_ILLEGAL_REQUEST                     0x000F
 
 
 /*--------------------------- Error causes param defs ----------------------------------------*/
@@ -518,6 +508,7 @@ SCTP_staleCookieError;
 
 typedef struct SCTP_INVALID_STREAMID_ERROR
 {
+    SCTP_vlparam_header vlparam_header;
     guint16 stream_id;
     guint16 reserved;
 }
@@ -545,44 +536,37 @@ typedef struct SCTP_MISSING_PARAMS_ERROR
 }
 SCTP_MissingParams;
 
+typedef struct SCTP_GENERIC_PARAMETER
+{
+    SCTP_vlparam_header vlparam_header;
+    guchar the_params[MAX_SCTP_PDU];
+}
+SCTP_GenericParameter;
+
 /*--------------------------- ASCONF Chunk and Parameter Types ---------------------------------*/
 
-typedef struct SCTP_ASCONF_FIXED
-{
-    guint32     serial_number;
-    guint16     reserved16;
-    guint8      reserved8;
-    guint8      address_type;
-    guint32     sctp_address[4];
-    guchar      variableParams[MAX_SCTP_PDU];
-}
-SCTP_asconf_fixed;
-
-typedef struct SCTP_ASCONF_ACK
-{
-    guint32     serial_number;
-    guchar      variableParams[MAX_SCTP_PDU];
-}
-SCTP_asconf_ack;
-
-
 typedef struct SCTP_ASCONF
 {
-    SCTP_chunk_header  chunk_header;
-    SCTP_asconf_fixed  asc_fixed;
-    guchar             variableParams[MAX_INIT_OPTIONS_LENGTH];
+    /* type 0xC1 */
+    SCTP_chunk_header   chunk_header;
+    guint32             serial_number;
+    SCTP_vlparam_header addressParameterHeader;
+    /* this starts with either IPv4 address of IPv6 address */
+    guchar              variableParams[MAX_SCTP_PDU];
+    /* only then come the ASCONF parameters */
 }
 SCTP_asconf;
 
 typedef struct SCTP_ASCONF_ACK_CHUNK
 {
-    SCTP_chunk_header  chunk_header;
-    SCTP_asconf_ack    asc_ack;
-    guchar             variableParams[MAX_INIT_OPTIONS_LENGTH];
+    /* type 0x80 */
+    SCTP_chunk_header   chunk_header;
+    guint32             serial_number;
+    /* ASCONF parameter responses */
+    guchar              variableParams[MAX_INIT_OPTIONS_LENGTH];
 }
 SCTP_asconf_ack_chunk;
 
-
 /*--------------------------- and some useful (?) macros ----------------------------------------*/
 
 #define CHUNKP_LENGTH(chunk)		(ntohs((chunk)->chunk_length))
