@@ -1,5 +1,5 @@
 /*
- *  $Id: rbundling.c 535 2004-11-17 23:04:09Z tuexen $
+ *  $Id: rbundling.c,v 1.5 2004/01/13 16:13:14 ajung Exp $
  *
  * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
@@ -30,7 +30,7 @@
  * used for any discussion related to this implementation.
  *
  * Contact: discussion@sctp.de
- *          tuexen@fh-muenster.de
+ *          Michael.Tuexen@icn.siemens.de
  *          ajung@exp-math.uni-essen.de
  *
  * Purpose: This module treats the received chunks, gets the SCTP data from SCTP message-distribution,
@@ -44,19 +44,24 @@
 #include "recvctrl.h"
 #include "reltransfer.h"
 #include "errorhandler.h"
+#include "sctp_asconf.h"
 #include "streamengine.h"
-#include "distribution.h"
 
 #include <stdio.h>
 
 #define TOTAL_SIZE(buf)		((buf)->ctrl_position+(buf)->sack_position+(buf)->data_position- 2*sizeof(SCTP_common_header))
 
 
-
+/**
+ * this function scans through a received SCTP packet and sets bits in an unsigned int
+ * according to the chunk types (from 0 to 31) that have appeared in the packet.
+ * for all chunk types greater than 30, the 31st bit (i.e. highest bit) is set
+ * @param pdu  pointer to the first byte of the first chunk
+ * @param len  len of all chunks
+ */
 unsigned int rbu_scanPDU(guchar * pdu, guint len)
 {
     gushort processed_len = 0;
-    gushort chunk_len = 0;
     unsigned int result = 0;
     guchar *current_position;
     guint pad_bytes;
@@ -65,33 +70,27 @@ unsigned int rbu_scanPDU(guchar * pdu, guint len)
     current_position = pdu; /* points to the first chunk in this pdu */
 
     while (processed_len < len) {
-        
+
         event_logii(VERBOSE, "rbu_scanPDU : len==%u, processed_len == %u", len, processed_len);
 
         chunk = (SCTP_simple_chunk *) current_position;
-        chunk_len = CHUNKP_LENGTH((SCTP_chunk_header *) chunk);
-        
-        if (chunk_len < 4 || chunk_len + processed_len > len) return result;
-        
         if (chunk->chunk_header.chunk_id <= 30) {
             result = result | (1 << chunk->chunk_header.chunk_id);
             event_logii(VERBOSE, "rbu_scanPDU : Chunk type==%u, result == %x", chunk->chunk_header.chunk_id, result);
         } else {
             result = result | (1 << 31);
             event_logii(VERBOSE, "rbu_scanPDU : Chunk type==%u setting bit 31 --> result == %x", chunk->chunk_header.chunk_id, result);
-        }    
-        processed_len += chunk_len;
+        }
+        processed_len += CHUNKP_LENGTH((SCTP_chunk_header *) chunk);
         pad_bytes = ((processed_len % 4) == 0) ? 0 : (4 - processed_len % 4);
         processed_len += pad_bytes;
-        chunk_len = (CHUNKP_LENGTH((SCTP_chunk_header *) chunk) + pad_bytes * sizeof(unsigned char));
-        
-        if (chunk_len < 4 || chunk_len + processed_len > len) return result;
-        current_position += chunk_len;
-        
+        current_position +=
+                (CHUNKP_LENGTH((SCTP_chunk_header *) chunk) + pad_bytes * sizeof(unsigned char));
     }
     return result;
 }
 
+
 gboolean rbu_datagramContains(gushort chunk_type, unsigned int chunkArray)
 {
     unsigned int val = 0;
@@ -106,8 +105,7 @@ gboolean rbu_datagramContains(gushort chunk_type, unsigned int chunkArray)
     if ((val & chunkArray) != 0) return TRUE;
     else return FALSE;
 
-}    
-        
+}
 
 guchar* rbu_scanInitChunkForParameter(guchar * chunk, gushort paramType)
 {
@@ -133,9 +131,7 @@ guchar* rbu_scanInitChunkForParameter(guchar * chunk, gushort paramType)
                     "rbu_scanInitChunkForParameter : len==%u, processed_len == %u", len, processed_len);
         vlp = (SCTP_vlparam_header*) current_position;
         parameterLength = ntohs(vlp->param_length);
-        
-        if (parameterLength < 4 || parameterLength + processed_len > len) return NULL;
-        
+
         if (ntohs(vlp->param_type) == paramType) {
             return current_position;
         }
@@ -161,7 +157,7 @@ guchar* rbu_scanInitChunkForParameter(guchar * chunk, gushort paramType)
  */
 guchar* rbu_findChunk(guchar * datagram, guint len, gushort chunk_type)
 {
-    gushort processed_len = 0, chunk_len = 0;
+    gushort processed_len = 0;
     guchar *current_position;
     guint pad_bytes;
     SCTP_simple_chunk *chunk;
@@ -174,17 +170,14 @@ guchar* rbu_findChunk(guchar * datagram, guint len, gushort chunk_type)
 
         chunk = (SCTP_simple_chunk *) current_position;
         if (chunk->chunk_header.chunk_id == chunk_type)
+        {
             return current_position;
-        else {
-            chunk_len = CHUNKP_LENGTH((SCTP_chunk_header *) chunk);
-            if (chunk_len < 4 || chunk_len + processed_len > len) return NULL;
-            
+        } else {
             processed_len += CHUNKP_LENGTH((SCTP_chunk_header *) chunk);
             pad_bytes = ((processed_len % 4) == 0) ? 0 : (4 - processed_len % 4);
             processed_len += pad_bytes;
-            chunk_len = (CHUNKP_LENGTH((SCTP_chunk_header *) chunk) + pad_bytes * sizeof(unsigned char));
-            if (chunk_len < 4 || chunk_len + processed_len > len) return NULL;
-            current_position += chunk_len;
+            current_position +=
+                (CHUNKP_LENGTH((SCTP_chunk_header *) chunk) + pad_bytes * sizeof(unsigned char));
         }
     }
     return NULL;
@@ -204,7 +197,7 @@ guchar* rbu_findChunk(guchar * datagram, guint len, gushort chunk_type)
  */
 gint rbu_findAddress(guchar * chunk, guint n, union sockunion* foundAddress, int supportedAddressTypes)
 {
-    gushort processed_len;
+    gushort processed_len = 0;
     guint len = 0, parameterLength = 0;
     guchar *current_position;
     guint pad_bytes;
@@ -222,16 +215,14 @@ gint rbu_findAddress(guchar * chunk, guint n, union sockunion* foundAddress, int
     }
     len = ntohs(initChunk->chunk_header.chunk_length);
     current_position = initChunk->variableParams;
-    processed_len = (sizeof(SCTP_chunk_header)+sizeof(SCTP_init_fixed));
+    processed_len += (sizeof(SCTP_chunk_header)+sizeof(SCTP_init_fixed));
 
     while (processed_len < len) {
         event_logii(INTERNAL_EVENT_0,
                     "rbu_findAddress : len==%u, processed_len == %u", len, processed_len);
         vlp = (SCTP_vlparam_header*) current_position;
         parameterLength = ntohs(vlp->param_length);
-        
-        if (parameterLength < 4 || parameterLength + processed_len > len) return -1;
-        
+
         if (ntohs(vlp->param_type) == VLPARAM_IPV4_ADDRESS &&
             supportedAddressTypes & SUPPORT_ADDRESS_TYPE_IPV4) {
             /* discard invalid addresses */
@@ -286,7 +277,7 @@ gint rbu_findAddress(guchar * chunk, guint n, union sockunion* foundAddress, int
  */
 gboolean rbu_scanDatagramForError(guchar * datagram, guint len, gushort error_cause)
 {
-    gushort processed_len = 0, param_length = 0, chunk_length = 0;
+    gushort processed_len = 0;
     gushort err_len = 0;
 
     guchar *current_position;
@@ -296,47 +287,35 @@ gboolean rbu_scanDatagramForError(guchar * datagram, guint len, gushort error_ca
 
 
     current_position = datagram; /* points to the first chunk in this pdu */
-    while (processed_len < len) {
-
-        event_logii(VERBOSE,
-                    "rbu_scanDatagramForError : len==%u, processed_len == %u", len, processed_len);
+    while (processed_len < len)
+    {
+        event_logii(VERBOSE, "rbu_scanDatagramForError : len==%u, processed_len == %u",
+                    len, processed_len);
 
         chunk = (SCTP_simple_chunk *) current_position;
-        chunk_length = CHUNKP_LENGTH((SCTP_chunk_header *) chunk);
-        if (chunk_length < 4 || chunk_length + processed_len > len) return FALSE;
-        
-        if (chunk->chunk_header.chunk_id == CHUNK_ERROR) {
-            
-            if (chunk_length < 4 || chunk_length + processed_len > len) return FALSE;
-            
+        if  (chunk->chunk_header.chunk_id == CHUNK_ERROR)
+        {
             event_log(INTERNAL_EVENT_0, "rbu_scanDatagramForError : Error Chunk Found");
-            /* now search for error parameter that fits */
-            while (err_len < chunk_length - sizeof(SCTP_chunk_header))  {
+            while (err_len < (ntohs(chunk->chunk_header.chunk_length)-sizeof(SCTP_chunk_header)) ) {
                 err_chunk = (SCTP_staleCookieError *) &(chunk->simple_chunk_data[err_len]);
-                if (ntohs(err_chunk->vlparam_header.param_type) == error_cause) {
-                    event_logi(VERBOSE,
-                               "rbu_scanDatagramForError : Error Cause %u found -> Returning TRUE",
+                if (ntohs(err_chunk->vlparam_header.param_type) == error_cause)
+                {
+                    event_logi(VERBOSE, "rbu_scanDatagramForError : Error Cause %u found -> Returning TRUE",
                                error_cause);
                     return TRUE;
                 }
-                param_length = ntohs(err_chunk->vlparam_header.param_length);
-                if (param_length < 4 || param_length + err_len > len) return FALSE;
-                
-                err_len += param_length;
+                err_len += ntohs(err_chunk->vlparam_header.param_length);
                 while ((err_len % 4) != 0)
                     err_len++;
             }
-        }
-        
-        processed_len += chunk_length;
+        } 
+        processed_len += CHUNKP_LENGTH((SCTP_chunk_header *) chunk);
         pad_bytes = ((processed_len % 4) == 0) ? 0 : (4 - processed_len % 4);
         processed_len += pad_bytes;
-        chunk_length = (CHUNKP_LENGTH((SCTP_chunk_header *) chunk) + pad_bytes * sizeof(unsigned char));
-        if (chunk_length < 4 || chunk_length + processed_len > len) return FALSE;
-        current_position += chunk_length;
+        current_position +=
+              (CHUNKP_LENGTH((SCTP_chunk_header *) chunk) + pad_bytes * sizeof(unsigned char));
     }
-    event_logi(VERBOSE,
-               "rbu_scanDatagramForError : Error Cause %u NOT found -> Returning FALSE",
+    event_logi(VERBOSE, "rbu_scanDatagramForError : Error Cause %u NOT found -> Returning FALSE",
                error_cause);
     return FALSE;
 }
@@ -356,7 +335,12 @@ gboolean rbu_scanDatagramForError(guchar * datagram, guint len, gushort error_ca
  * @param  datagram     pointer to first chunk of the newly received data
  * @param  len          length of payload (i.e. len of the concatenation of chunks)
  */
-gint rbu_rcvDatagram(guint address_index, guchar * datagram, guint len)
+gint rbu_rcvDatagram(SCTP_instance *sI,
+                     Association* asok,
+                     gint32 addressIndex,
+                     guint32 addressKey,
+                     guchar * datagram,
+                     guint32 len)
 {
     /* sctp common header header has been verified */
     /* tag (if association is established) and CRC is okay */
@@ -378,23 +362,21 @@ gint rbu_rcvDatagram(guint address_index, guchar * datagram, guint len)
     gboolean send_it = FALSE;
     int result;
 
-    bu_lock_sender();
+    bu_lock_sender(asok);
 
     current_position = datagram; /* points to the first chunk in this pdu */
-    event_log(INTERNAL_EVENT_0, "Entered rbu_rcvDatagram()...... ");
+    event_logi(INTERNAL_EVENT_0, "Entered rbu_rcvDatagram(addressKey=%u)...... ", addressKey);
     /* CHECKME : beim Empfangen leerer Chunks tritt im Bundling eine Endlosschleife auf ??? */
     while (processed_len < len) {
 
         chunk = (SCTP_simple_chunk *) current_position;
         chunk_len = CHUNKP_LENGTH((SCTP_chunk_header *) chunk);
         event_logiiii(INTERNAL_EVENT_0,
-                     "rbu_rcvDatagram(address=%u) : len==%u, processed_len = %u, chunk_len=%u",
-                     address_index, len, processed_len, chunk_len);
-        if ((processed_len+chunk_len) > len || chunk_len < 4) {
-            error_logiii(ERROR_MINOR, "Faulty chunklen=%u, total len=%u,processed_len=%u --> dropping packet  !",
-                                    chunk_len,len,processed_len);
-            /* if the association has already been removed, we cannot unlock it anymore */
-            bu_unlock_sender(&address_index);
+                     "rbu_rcvDatagram(address index=%u) : len==%u, processed_len = %u, chunk_len=%u",
+                     addressIndex, len, processed_len, chunk_len);
+        if (chunk_len > len) {
+            error_logii(ERROR_MINOR, "Faulty chunklen=%u, total len=%u --> dropping rest of data !",
+                                    chunk_len,len);
             return 1;
         }
         /*
@@ -404,80 +386,78 @@ gint rbu_rcvDatagram(guint address_index, guchar * datagram, guint len)
          */
         switch (chunk->chunk_header.chunk_id) {
         case CHUNK_DATA:
-            event_log(INTERNAL_EVENT_0, "*******************  Bundling received DATA chunk");
-            rxc_data_chunk_rx((SCTP_data_chunk*) chunk, address_index);
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxx  Bundling received DATA chunk xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            rxc_data_chunk_rx(asok, (SCTP_data_chunk*) chunk, addressKey);
             data_chunk_received = TRUE;
             break;
         case CHUNK_INIT:
-            event_log(INTERNAL_EVENT_0, "*******************  Bundling received INIT chunk");
-            association_state = sctlr_init((SCTP_init *) chunk);
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received INIT chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            association_state = scr_init(sI, asok, (SCTP_init *) chunk);
             break;
         case CHUNK_INIT_ACK:
-            event_log(INTERNAL_EVENT_0, "*******************  Bundling received INIT ACK chunk");
-            association_state = sctlr_initAck((SCTP_init *) chunk);
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received INIT ACK chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            association_state = scr_initAck(sI, asok, (SCTP_init *) chunk);
             break;
         case CHUNK_SACK:
-            event_log(INTERNAL_EVENT_0, "*******************  Bundling received SACK chunk");
-            rtx_process_sack(address_index, chunk, len);
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received SACK chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            rtx_process_sack(asok, addressKey, chunk, len);
             break;
         case CHUNK_HBREQ:
-            event_log(INTERNAL_EVENT_0, "*******************  Bundling received HB_REQ chunk");
-            pm_heartbeat((SCTP_heartbeat *) chunk, address_index);
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received HB_REQ chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            pm_heartbeat(asok, (SCTP_heartbeat *) chunk, addressKey);
             break;
         case CHUNK_HBACK:
-            event_log(INTERNAL_EVENT_0, "*******************  Bundling received HB_ACK chunk");
-            pm_heartbeatAck((SCTP_heartbeat *) chunk);
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received HB_ACK chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            pm_heartbeatAck(asok, (SCTP_heartbeat *) chunk);
             break;
         case CHUNK_ABORT:
-            event_log(INTERNAL_EVENT_0, "*******************  Bundling received ABORT chunk");
-            association_state = sctlr_abort();
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received ABORT chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            association_state = scr_abort(asok);
             break;
         case CHUNK_SHUTDOWN:
-            event_log(INTERNAL_EVENT_0, "*******************  Bundling received SHUTDOWN chunk");
-            association_state = sctlr_shutdown((SCTP_simple_chunk *) chunk);
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received SHUTDOWN chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            association_state = scr_shutdown(asok, (SCTP_simple_chunk *) chunk);
             break;
         case CHUNK_SHUTDOWN_ACK:
-            event_log(INTERNAL_EVENT_0, "*******************  Bundling received SHUTDOWN ACK chunk");
-            association_state = sctlr_shutdownAck();
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received SHUTDOWN ACK chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            association_state = scr_shutdownAck(asok);
             break;
         case CHUNK_ERROR:
-            event_log(INTERNAL_EVENT_0, "Error Chunk");
-            eh_recv_chunk(chunk);
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Error Chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            eh_recv_chunk(asok, chunk);
             break;
         case CHUNK_COOKIE_ECHO:
-            event_log(INTERNAL_EVENT_0, "*******************  Bundling received COOKIE ECHO chunk");
-            sctlr_cookie_echo((SCTP_cookie_echo *) chunk);
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received COOKIE ECHO chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            asok = scr_cookie_echo(sI, asok, (SCTP_cookie_echo *) chunk);
             break;
         case CHUNK_COOKIE_ACK:
-            event_log(INTERNAL_EVENT_0, "*******************  Bundling received COOKIE ACK chunk");
-            sctlr_cookieAck((SCTP_simple_chunk *) chunk);
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received COOKIE ACK chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            scr_cookieAck(asok, (SCTP_simple_chunk *) chunk);
             break;
-     /* case CHUNK_ECNE:
+        /* case CHUNK_ECNE:
         case CHUNK_CWR:
             event_logi(INTERNAL_EVENT_0,
-                       "Chunktype %u not Supported Yet !!!!!!!!", chunk->chunk_header.chunk_id);
+                       "xxxxxxxxxxxxxxxxxx  Chunktype %u not Supported Yet !!!!!!!! xxxxxxxxxxxxxxxxxx", chunk->chunk_header.chunk_id);
             break;*/
         case CHUNK_SHUTDOWN_COMPLETE:
-            event_log(INTERNAL_EVENT_0, "*******************  Bundling received SHUTDOWN_COMPLETE chunk");
-            association_state = sctlr_shutdownComplete();
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received SHUTDOWN_COMPLETE chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            association_state = scr_shutdownComplete(asok);
             break;
         case CHUNK_FORWARD_TSN:
-            if (mdi_supportsPRSCTP() == TRUE) {
-                event_log(INTERNAL_EVENT_0, "*******************  Bundling received FORWARD_TSN chunk");
-                rxc_process_forward_tsn((SCTP_simple_chunk *) chunk);
+            if (mdi_supportsPRSCTP(asok) == TRUE) {
+                event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received FORWARD_TSN chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+                rxc_process_forward_tsn(asok, (SCTP_simple_chunk *) chunk);
                 break;
             } else
                 continue;
-    /*    case CHUNK_ASCONF: */
-            /* check that ASCONF chunks are standalone chunks, not bundled with any other
-               chunks. Else ignore the ASCONF chunk (but not the others) */
-/*            event_log(INTERNAL_EVENT_0, "Bundling received ASCONF chunk");
-            asc_recv_asconf_chunk((SCTP_simple_chunk *) chunk);
+        case CHUNK_ASCONF: 
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received ASCONF chunk xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            asc_recv_asconf_chunk(asok, (SCTP_simple_chunk *) chunk, addressKey);
             break;
         case CHUNK_ASCONF_ACK:
-            event_log(INTERNAL_EVENT_0, "Bundling received ASCONF_ACK chunk");
-            asc_recv_asconf_ack((SCTP_simple_chunk *) chunk);
-            break; */
+            event_log(INTERNAL_EVENT_0, "xxxxxxxxxxxxxxxxxxxxxxxxxxxx  Bundling received ASCONF_ACK chunk  xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");
+            asc_recv_asconf_ack(asok, (SCTP_simple_chunk *) chunk, addressKey);
+            break; 
         default:
         /* 00 - Stop processing this SCTP packet and discard it, do not process
                 any further chunks within it.
@@ -491,16 +471,16 @@ gint rbu_rcvDatagram(guint address_index, guchar * datagram, guint len)
             if ((chunk->chunk_header.chunk_id & 0xC0) == 0x0) {            /* 00 */
                 processed_len = len;
                 event_logi(EXTERNAL_EVENT_X, "00: Unknown chunktype %u in rbundling.c", chunk->chunk_header.chunk_id);
-            } else if ((chunk->chunk_header.chunk_id & 0xC0) == 0x40) {    /* 01 */
+            } else if ((chunk->chunk_header.chunk_id & 0xC0) == 0x20) {    /* 01 */
                 processed_len = len;
-                result = eh_send_unrecognized_chunktype((unsigned char*)chunk,chunk_len);
+                result = eh_send_unrecognized_chunktype(asok, (unsigned char*)chunk,chunk_len);
                 event_logi(EXTERNAL_EVENT_X, "01: Unknown chunktype %u in rbundling.c",chunk->chunk_header.chunk_id);
             } else if ((chunk->chunk_header.chunk_id & 0xC0) == 0x80) {    /* 10 */
                 /* nothing */
                 event_logi(EXTERNAL_EVENT_X, "10: Unknown chunktype %u in rbundling.c",chunk->chunk_header.chunk_id);
             } else if ((chunk->chunk_header.chunk_id & 0xC0) == 0xC0) {    /* 11 */
                 event_logi(EXTERNAL_EVENT_X, "11: Unknown chunktype %u in rbundling.c", chunk->chunk_header.chunk_id);
-                result = eh_send_unrecognized_chunktype((unsigned char*)chunk,chunk_len);
+                result = eh_send_unrecognized_chunktype(asok, (unsigned char*)chunk,chunk_len);
             }
             break;
         }
@@ -520,21 +500,21 @@ gint rbu_rcvDatagram(guint address_index, guchar * datagram, guint len)
 
         if (data_chunk_received == TRUE) {
             /* update SACK structure and start SACK timer */
-            rxc_all_chunks_processed(TRUE);
+            rxc_all_chunks_processed(asok, TRUE);
         } else {
             /* update SACK structure and datagram counter */
-            rxc_all_chunks_processed(FALSE);   
+            rxc_all_chunks_processed(asok, FALSE);      		
         }
         /* optionally also add a SACK chunk, at least for every second datagram
          * see section 6.2, second paragraph
          */
         if (data_chunk_received == TRUE){
-            send_it = rxc_create_sack(&address_index, FALSE);
-            se_doNotifications();
-            if (send_it==TRUE) bu_sendAllChunks(&address_index);
+            send_it = rxc_create_sack(asok, addressKey, FALSE);
+            se_doNotifications(asok);
+            if (send_it==TRUE) bu_sendAllChunks(asok, addressKey);
         }
         /* if the association has already been removed, we cannot unlock it anymore */
-        bu_unlock_sender(&address_index);
+        bu_unlock_sender(asok, addressKey);
     }
 
     return 0;
