@@ -1,44 +1,41 @@
 /*
- * --------------------------------------------------------------------------
- *
- *           //=====   //===== ===//=== //===//  //       //   //===//
- *          //        //         //    //    // //       //   //    //
- *         //====//  //         //    //===//  //       //   //===<<
- *              //  //         //    //       //       //   //    //
- *       ======//  //=====    //    //       //=====  //   //===//
- *
- * -------------- An SCTP implementation according to RFC 4960 --------------
+ *  $Id: recvctrl.c,v 1.6 2004/01/13 16:13:14 ajung Exp $
  *
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
- * Copyright (C) 2001-2004 Andreas Jungmaier
- * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Acknowledgements:
- * Realized in co-operation between Siemens AG and the University of
- * Duisburg-Essen, Institute for Experimental Mathematics, Computer
- * Networking Technology group.
- * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany
- * (Förderkennzeichen 01AK045).
+ * Realized in co-operation between Siemens AG
+ * and University of Essen, Institute of Computer Networking Technology.
+ *
+ * Acknowledgement
+ * This work was partially funded by the Bundesministerium fr Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * There are two mailinglists available at http://www.sctp.de which should be
+ * used for any discussion related to this implementation.
+ *
+ * Contact: discussion@sctp.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
+ *
+ * This module creates SACK data structures, that may be used to
+ * indicate received and lost data to the association peer.
  *
- * Contact: sctp-discussion@sctp.de
- *          thomas.dreibholz@gmail.com
- *          tuexen@fh-muenster.de
- *          andreas.jungmaier@web.de
  */
 
 #include "recvctrl.h"
@@ -46,22 +43,19 @@
 #include "bundling.h"
 #include "distribution.h"
 #include "streamengine.h"
-#include "SCTP-control.h"
 
 #include <glib.h>
 #include <string.h>
+#include <assert.h>
 
 /**
  * this struct contains all necessary data for creating SACKs from received data chunks
  */
 typedef struct rxc_buffer_struct
 {
-    /*@{ */
-    /** */
+    /* @{ */
     void *sack_chunk;
-    /** */
     GList *frag_list;
-    /** */
     GList *dup_list;
     /** cumulative TSN acked */
     unsigned int ctsna;
@@ -70,28 +64,23 @@ typedef struct rxc_buffer_struct
     /** stores highest tsn received so far, taking care of wraps
         i.e. highest < lowest indicates a wrap */
     unsigned int highest;
-    /** */
     boolean contains_valid_sack;
-    /** */
     boolean timer_running;
     /** indicates whether a chunk was recvd that is truly new */
     boolean new_chunk_received;
     /** timer for delayed sacks */
     TimerID sack_timer;
     int datagrams_received;
-     /* either 1 (= sack each data chunk) or 2 (=sack every second chunk)*/
+    /* either 1 (= sack each data chunk) or 2 (=sack every second chunk)*/
     unsigned int sack_flag;
-    /** */
-    unsigned int last_address;
-    /** */
-    unsigned int my_association;
-    /** */
+    unsigned int lastAddressKey;
+    Association* myAssoc;
     unsigned int my_rwnd;
     /** delay for delayed ACK in msecs */
     unsigned int delay;
     /** number of dest addresses */
     unsigned int num_of_addresses;
-    /*@} */
+    /* @} */
 } rxc_buffer;
 
 
@@ -101,33 +90,28 @@ typedef struct rxc_buffer_struct
  * @param  remote_initial_TSN initial tsn of the peer
  * @return pointer to the newly created structure
  */
-void *rxc_new_recvctrl(unsigned int remote_initial_TSN, unsigned int number_of_destination_addresses, void* sctpInstance)
+void *rxc_new_recvctrl(Association * asok, unsigned int remote_initial_TSN)
 {
     rxc_buffer *tmp;
-/*
-    unsigned int count;
-*/
-    tmp = (rxc_buffer*)malloc(sizeof(rxc_buffer));
+
+    tmp = malloc(sizeof(rxc_buffer));
     if (!tmp) error_log(ERROR_FATAL, "Malloc failed");
 
     tmp->frag_list = NULL;
     tmp->dup_list = NULL;
-    tmp->num_of_addresses = number_of_destination_addresses;
     tmp->sack_chunk = malloc(sizeof(SCTP_sack_chunk));
-    tmp->ctsna = remote_initial_TSN - 1; /* as per section 4.1 */
-    tmp->lowest = remote_initial_TSN - 1;
+    tmp->ctsna   = remote_initial_TSN - 1; /* as per section 4.1 */
+    tmp->lowest  = remote_initial_TSN - 1;
     tmp->highest = remote_initial_TSN - 1;
-    tmp->contains_valid_sack = FALSE;
-    tmp->timer_running = FALSE;
-    tmp->datagrams_received = -1;
-    tmp->sack_flag = 2;
-    tmp->last_address = 0;
-    tmp->my_rwnd =  mdi_getDefaultMyRwnd();
-    tmp->delay =    mdi_getDefaultDelay(sctpInstance);
-    tmp->my_association = mdi_readAssociationID();
-    event_logi(VVERBOSE, "RecvControl : Association-ID== %d ", tmp->my_association);
-    if (tmp->my_association == 0)
-        error_log(ERROR_FATAL, "Association was not set, should be......");
+    tmp->contains_valid_sack    = FALSE;
+    tmp->timer_running          = FALSE;
+    tmp->datagrams_received     = -1;
+    tmp->sack_flag              = 2;
+    tmp->lastAddressKey         = 0;
+    tmp->my_rwnd =  mdi_getDefaultMyRwnd(asok->sctpInstance);
+    tmp->delay =    mdi_getDefaultDelay(asok->sctpInstance);
+    assert(asok);
+    tmp->myAssoc = asok;
     return (tmp);
 }
 
@@ -135,18 +119,12 @@ void *rxc_new_recvctrl(unsigned int remote_initial_TSN, unsigned int number_of_d
  * function deletes a rxc_buffer structure (when it is not needed anymore)
  * @param rxc_instance pointer to a rxc_buffer, that was previously created
  */
-void rxc_delete_recvctrl(void *rxc_instance)
+void rxc_delete_recvctrl(Association* asok)
 {
-    rxc_buffer *tmp;
-    tmp = (rxc_buffer *) rxc_instance;
+    rxc_buffer *tmp = (rxc_buffer *) asok->rx_control;
     event_log(INTERNAL_EVENT_0, "deleting receivecontrol");
     free(tmp->sack_chunk);
 
-    if (tmp->timer_running == TRUE) {
-        sctp_stopTimer(tmp->sack_timer);
-        tmp->timer_running = FALSE;
-    }
-
     g_list_foreach(tmp->frag_list, &free_list_element, NULL);
     g_list_free(tmp->frag_list);
     g_list_foreach(tmp->dup_list, &free_list_element, NULL);
@@ -191,7 +169,7 @@ boolean rxc_chunk_is_duplicate(rxc_buffer * rbuf, unsigned int chunk_tsn)
         if (between(frag->start_tsn, chunk_tsn, frag->stop_tsn))
             return TRUE;
         /* assuming an ordered list of fragments */
-        if (after(frag->stop_tsn, chunk_tsn))
+        if (frag->stop_tsn > chunk_tsn)
            return FALSE;
         temp = g_list_next(temp);
     }
@@ -273,7 +251,7 @@ void rxc_update_duplicates(rxc_buffer * rbuf, unsigned int ch_tsn)
         current = g_list_next(current);
     }
     /* its new - add it to the list */
-    match = (duplicate*)malloc(sizeof(duplicate));
+    match = malloc(sizeof(duplicate));
     match->duplicate_tsn = ch_tsn;
     rbuf->dup_list =  g_list_insert_sorted(rbuf->dup_list, match, (GCompareFunc) rxc_sort_duplicates);
 
@@ -360,7 +338,7 @@ boolean rxc_update_fragments(rxc_buffer * rbuf, unsigned int ch_tsn)
                     rbuf->new_chunk_received = TRUE;
                     return TRUE;
                 } else {    /* a fragment in between */
-                    new_frag = (fragment32*)malloc(sizeof(fragment32));
+                    new_frag = malloc(sizeof(fragment32));
                     new_frag->start_tsn = new_frag->stop_tsn = ch_tsn;
                     event_log(VVERBOSE, "Inserting new fragment....");
                     rbuf->frag_list = g_list_insert_sorted(rbuf->frag_list, new_frag, (GCompareFunc) rxc_sort_fragments);
@@ -423,7 +401,7 @@ boolean rxc_update_fragments(rxc_buffer * rbuf, unsigned int ch_tsn)
         return FALSE;
     } else {                    /* a new fragment altogether */
         current = g_list_last(rbuf->frag_list);
-        new_frag = (fragment32*)malloc(sizeof(fragment32));
+        new_frag = malloc(sizeof(fragment32));
         new_frag->start_tsn = ch_tsn;
         new_frag->stop_tsn = ch_tsn;
         rbuf->frag_list = g_list_append(rbuf->frag_list, new_frag);
@@ -440,31 +418,28 @@ boolean rxc_update_fragments(rxc_buffer * rbuf, unsigned int ch_tsn)
  * For now this function treats only one incoming data chunk' tsn
  * @param chunk the data chunk that was received by the bundling
  */
-int rxc_data_chunk_rx(SCTP_data_chunk * se_chk, unsigned int ad_idx)
+int rxc_data_chunk_rx(Association * asok, SCTP_data_chunk * se_chk, unsigned int addrKey)
 {
-    rxc_buffer *rxc;
+    rxc_buffer *rxc = NULL;
     unsigned int chunk_tsn;
     unsigned int chunk_len;
     unsigned int assoc_state;
     boolean result = FALSE;
     int bytesQueued = 0;
-    unsigned current_rwnd = 0;
+    unsigned int current_rwnd = 0;
 
     event_log(INTERNAL_EVENT_0, "Entering function rxc_data_chunk_rx");
-    rxc = (rxc_buffer *) mdi_readRX_control();
-    if (!rxc) {
-        error_log(ERROR_MAJOR, "rxc_buffer instance not set !");
-        return (-1);
-    }
-
 
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
+    assert(rxc);
     /* resetting it */
     rxc->new_chunk_received = FALSE;
-    rxc->last_address = ad_idx;
+    rxc->lastAddressKey = addrKey;
 
-    bytesQueued = se_getQueuedBytes();
+    /* FIXME : implement silly window syndrome here !!!!!!!!!!!! */
+    bytesQueued = se_getQueuedBytesRecvQueue(asok);
     if (bytesQueued < 0) bytesQueued = 0;
-    if ((unsigned int)bytesQueued > rxc->my_rwnd) {
+    if (bytesQueued > rxc->my_rwnd) {
         current_rwnd = 0;
     } else {
         current_rwnd = rxc->my_rwnd - bytesQueued;
@@ -481,22 +456,24 @@ int rxc_data_chunk_rx(SCTP_data_chunk * se_chk, unsigned int ad_idx)
 
     chunk_tsn = ntohl(se_chk->tsn);
     chunk_len = ntohs(se_chk->chunk_length);
-    assoc_state = sci_getState();
 
+    assoc_state = sci_getState(asok);
+    
     if ( (after(chunk_tsn, rxc->highest) && current_rwnd == 0) ||
          (assoc_state == SHUTDOWNRECEIVED) ||
          (assoc_state == SHUTDOWNACKSENT) ) {
-        /* drop chunk, if either: our rwnd is 0, or we are already shutting down */
+
         rxc->new_chunk_received = FALSE;
         return 1;
     }
+    /* TODO : do sanity checks on received chunk !!!!!!!!!! */
 
-    /* TODO :  Duplicates : see Note in section 6.2 :  */
-    /*  Note: When a datagram arrives with duplicate DATA chunk(s) and no new
-        DATA chunk(s), the receiver MUST immediately send a SACK with no
-        delay. Normally this will occur when the original SACK was lost, and
-        the peers RTO has expired. The duplicate TSN number(s) SHOULD be
-        reported in the SACK as duplicate.
+    /*
+     * Note: When a datagram arrives with duplicate DATA chunk(s) and no new
+     * DATA chunk(s), the receiver MUST immediately send a SACK with no
+     * delay. Normally this will occur when the original SACK was lost, and
+     * the peers RTO has expired. The duplicate TSN number(s) SHOULD be
+     * reported in the SACK as duplicate.
      */
     event_logii(VERBOSE, "rxc_data_chunk_rx : chunk_tsn==%u, chunk_len=%u", chunk_tsn, chunk_len);
     if (rxc_update_lowest(rxc, chunk_tsn) == TRUE) {
@@ -514,13 +491,9 @@ int rxc_data_chunk_rx(SCTP_data_chunk * se_chk, unsigned int ad_idx)
 
     event_logi(VVERBOSE, "rxc_data_chunk_rx: after rxc_bubbleup_ctsna, rxc->ctsna=%u", rxc->ctsna);
 
-    if (rxc->new_chunk_received == TRUE) {
-        if(se_recvDataChunk(se_chk, chunk_len, ad_idx) == SCTP_SUCCESS) {
-            /* resetting it */
-            rxc->new_chunk_received = FALSE;
-        }
-        /* else: ABORT has been sent and the association (possibly) removed in callback! */
-    }
+    if (rxc->new_chunk_received == TRUE) se_recvDataChunk(asok, se_chk, chunk_len);
+    /* resetting it */
+    rxc->new_chunk_received = FALSE;
     return 1;
 }
 
@@ -529,17 +502,17 @@ int rxc_data_chunk_rx(SCTP_data_chunk * se_chk, unsigned int ad_idx)
  * send SACK to bundling using bu_put_SACK_Chunk() function.
  * @return boolean to indicate, whether a SACK was generated, and should be sent !
  */
-boolean rxc_create_sack(unsigned int *destination_address, boolean force_sack)
+boolean rxc_create_sack(Association* asok, unsigned int dAddressKey, boolean forceSack)
 {
     rxc_buffer *rxc;
+    int result;
     unsigned int num_of_frags;
 
     event_logii(VVERBOSE,
-                "Entering rxc_create_sack(address==%u, force_sack==%s",
-                ((destination_address != NULL) ? *destination_address : 0),
-                ((force_sack == TRUE) ? "TRUE" : "FALSE"));
+                "Entering rxc_create_sack(address==%u, force_sack==%s", dAddressKey,
+                ((forceSack == TRUE) ? "TRUE" : "FALSE"));
 
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
         error_log(ERROR_MAJOR, "rxc_buffer instance not set !");
         return FALSE;
@@ -547,22 +520,22 @@ boolean rxc_create_sack(unsigned int *destination_address, boolean force_sack)
 
     if (rxc->contains_valid_sack == FALSE) {
         event_log(INTERNAL_EVENT_0, "SACK structure was not updated (should have been)");
-        rxc_all_chunks_processed(FALSE);
+        rxc_all_chunks_processed(asok, FALSE);
     }
 
     num_of_frags = g_list_length(rxc->frag_list);
 
     if (num_of_frags > 0)
-        rxc_send_sack_everytime();
+        rxc_send_sack_everytime(asok);
     else
-        rxc_send_sack_every_second_time();
+        rxc_send_sack_every_second_time(asok);
 
     /* send sacks along every second time, generally */
     /* some timers may want to have a SACK anyway */
     /* first sack is sent at once, since datagrams_received==-1 */
-    if (force_sack == TRUE) {
+    if (forceSack == TRUE) {
         rxc->lowest = rxc->ctsna;
-        bu_put_SACK_Chunk((SCTP_sack_chunk*)rxc->sack_chunk, destination_address);
+        result = bu_put_SACK_Chunk(asok, rxc->sack_chunk, dAddressKey);
         return TRUE;
     } else {
 
@@ -575,7 +548,7 @@ boolean rxc_create_sack(unsigned int *destination_address, boolean force_sack)
                 return FALSE;
         }
         rxc->lowest = rxc->ctsna;
-        bu_put_SACK_Chunk((SCTP_sack_chunk*)rxc->sack_chunk,destination_address);
+        result = bu_put_SACK_Chunk(asok, rxc->sack_chunk,dAddressKey);
         return TRUE;
     }
     return FALSE;
@@ -593,43 +566,35 @@ boolean rxc_create_sack(unsigned int *destination_address, boolean force_sack)
  */
 void rxc_sack_timer_cb(TimerID tid, void *assoc, void *dummy)
 {
-    unsigned short res;
+    int send_result;
 
-    rxc_buffer *rxc;
+    rxc_buffer *rxc = NULL;
     event_log(INTERNAL_EVENT_1, "Timer Callback Function activated -> initiate sending of a SACK");
-    res = mdi_setAssociationData(*(unsigned int *) assoc);
-    if (res == 1)
-        error_log(ERROR_MAJOR, " association does not exist !");
-    if (res == 2) {
-        error_log(ERROR_MAJOR, "Association was not cleared..... !!!");
-        /* failure treatment ? */
-    }
-    /* all should be well now */
-    rxc = (rxc_buffer *) mdi_readRX_control();
+
+    rxc = (rxc_buffer *) mdi_readRX_control((Association*)assoc);
     if (!rxc) {
         error_log(ERROR_MAJOR, "rxc_buffer instance not set !");
         return;
     }
     rxc->timer_running = FALSE;
     /* sending sack */
-    /* FIXME : maybe choose different address ??? */
-    rxc_create_sack(&rxc->last_address, TRUE);
-    bu_sendAllChunks(&rxc->last_address);
+    rxc_create_sack((Association*)assoc, rxc->lastAddressKey, TRUE);
+    send_result = bu_sendAllChunks((Association*)assoc, rxc->lastAddressKey);
 
-    mdi_clearAssociationData();
     return;
 }
 
+
 /**
  * function called by bundling when a SACK is actually sent, to stop
  * a possibly running  timer
  */
-void rxc_stop_sack_timer(void)
+void rxc_stop_sack_timer(Association* asok)
 {
-    rxc_buffer *rxc;
+    rxc_buffer *rxc = NULL;
     int result;
 
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
         error_log(ERROR_MINOR, "rxc_buffer instance not set !");
         return;
@@ -647,10 +612,11 @@ void rxc_stop_sack_timer(void)
     return;
 }
 
-boolean rxc_sack_timer_is_running(void)
+
+boolean rxc_sack_timer_is_running(Association* asok)
 {
     rxc_buffer *rxc;
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
         error_log(ERROR_MINOR, "rxc_buffer instance not set !");
         return FALSE;
@@ -661,29 +627,30 @@ boolean rxc_sack_timer_is_running(void)
 
 /**
  * called by bundling, after new data has been processed (so we may start building a sack chunk)
- * or by streamengine, when ULP has read some data, and we want to update the RWND.
  */
-void rxc_all_chunks_processed(boolean new_data_received)
+int rxc_all_chunks_processed(Association* asok, boolean new_data_received)
 {
+
     /* now go and create SACK structure from the array */
-    rxc_buffer *rxc=NULL;
-    SCTP_sack_chunk *sack=NULL;
+    rxc_buffer *rxc = NULL;
+    SCTP_sack_chunk *sack = NULL;
     unsigned short num_of_frags, num_of_dups;
     unsigned short len16, count, frag_start16, frag_stop16;
     unsigned int pos;
-    duplicate *dptr=NULL, d;
-    fragment32 *f32=NULL;
+    duplicate *dptr = NULL, d;
+    fragment32 *f32 = NULL;
     fragment chunk_frag;
     GList *temp=NULL;
     int bytesQueued = 0;
     unsigned current_rwnd = 0;
 
-    event_log(INTERNAL_EVENT_0, "Entering funtion rxc_all_chunks_processed ()");
+    event_logi(INTERNAL_EVENT_0, "Entering funtion rxc_all_chunks_processed (%s)",
+                                 (new_data_received == TRUE)?"TRUE":"FALSE");
 
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
         error_log(ERROR_MAJOR, "rxc_buffer instance not set !");
-        return;
+        return SCTP_MODULE_NOT_FOUND;
     }
 
     if (new_data_received == TRUE) rxc->datagrams_received++;
@@ -691,25 +658,25 @@ void rxc_all_chunks_processed(boolean new_data_received)
     num_of_frags = g_list_length(rxc->frag_list);
     num_of_dups  = g_list_length(rxc->dup_list);
 
-    /* limit size of SACK to 80 bytes plus fixed size chunk and chunk header */
-    /* FIXME : Limit number of Fragments/Duplicates according to ->PATH MTU<-  */
+    /* limit variable lnegth of SACK to 80 bytes plus chunk header/fixed part */
     if (num_of_frags > 10) num_of_frags = 10;
-    if (num_of_dups > 10)  num_of_dups = 10;
+    if (num_of_dups  > 10) num_of_dups = 10;
 
     event_logii(VVERBOSE, "len of frag_list==%u, len of dup_list==%u", num_of_frags, num_of_dups);
 
-    bytesQueued = se_getQueuedBytes();
+    /* TODO : implement silly window syndrome here !!!!!!!!!!!! */
+    bytesQueued = se_getQueuedBytesRecvQueue(asok);
     if (bytesQueued < 0) bytesQueued = 0;
-    if ((unsigned int)bytesQueued > rxc->my_rwnd) {
+    if (bytesQueued > rxc->my_rwnd) {
         current_rwnd = 0;
     } else {
         current_rwnd = rxc->my_rwnd - bytesQueued;
     }
+
     /* do SWS prevention */
     if (current_rwnd > 0 && current_rwnd <= 2 * MAX_SCTP_PDU) current_rwnd = 1;
 
-
-    sack = (SCTP_sack_chunk*)rxc->sack_chunk;
+    sack = rxc->sack_chunk;
     sack->chunk_header.chunk_id = CHUNK_SACK;
     sack->chunk_header.chunk_flags = 0;
     len16 = sizeof(SCTP_chunk_header) + (2 + num_of_dups) * sizeof(unsigned int) +
@@ -717,13 +684,13 @@ void rxc_all_chunks_processed(boolean new_data_received)
 
     sack->chunk_header.chunk_length = htons(len16);
     sack->cumulative_tsn_ack = htonl(rxc->ctsna);
-    /* FIXME : deduct size of data still in queue, that is waiting to be picked up by an ULP */
     sack->a_rwnd = htonl(current_rwnd);
     sack->num_of_fragments  = htons(num_of_frags);
     sack->num_of_duplicates = htons(num_of_dups);
     pos = 0L;
 
-    temp = g_list_first(rxc->frag_list); count = 0;
+    temp = g_list_first(rxc->frag_list);
+    count = 0;
     while ((temp != NULL) && (count < num_of_frags)) {
 
         f32 = (fragment32*)temp->data;
@@ -736,34 +703,37 @@ void rxc_all_chunks_processed(boolean new_data_received)
             break;
         }
         frag_start16 = (unsigned short) (f32->start_tsn - rxc->ctsna);
-        frag_stop16 = (unsigned short) (f32->stop_tsn - rxc->ctsna);
+        frag_stop16  = (unsigned short) (f32->stop_tsn - rxc->ctsna);
         event_logii(VVERBOSE, "frag_start16==%u, frag_stop16==%u", frag_start16, frag_stop16);
 
-        chunk_frag.start = htons((unsigned short)(f32->start_tsn - rxc->ctsna));
-        chunk_frag.stop = htons((unsigned short)(f32->stop_tsn - rxc->ctsna));
+        chunk_frag.start = htons(f32->start_tsn - rxc->ctsna);
+        chunk_frag.stop  = htons(f32->stop_tsn  - rxc->ctsna);
         event_logii(VVERBOSE, "chunk_frag.start=%u,chunk_frag.stop ==%u",
                                 ntohs(chunk_frag.start), ntohs(chunk_frag.stop));
         memcpy(&sack->fragments_and_dups[pos], &chunk_frag, sizeof(fragment));
         pos += sizeof(fragment);
-        temp = g_list_next(temp); count++;
+        temp = g_list_next(temp);
+        count++;
     }
 
-    temp = g_list_first(rxc->dup_list); count = 0;
+    temp = g_list_first(rxc->dup_list);
+    count = 0;
     while ((temp != NULL) && (count < num_of_dups)) {
         dptr = (duplicate*)temp->data;
         if (dptr) d.duplicate_tsn = htonl(dptr->duplicate_tsn);
         memcpy(&sack->fragments_and_dups[pos], &d, sizeof(duplicate));
         pos += sizeof(duplicate);
-        temp = g_list_next(temp); count++;
+        temp = g_list_next(temp);
+        count++;
     }
-    /* start sack_timer set to 200 msecs */
+    /* start sack_timer */
     if (rxc->timer_running != TRUE && new_data_received == TRUE) {
-        rxc->sack_timer = adl_startTimer(rxc->delay, &rxc_sack_timer_cb, TIMER_TYPE_SACK, &(rxc->my_association), NULL);
+        rxc->sack_timer = adl_startTimer(rxc->delay, &rxc_sack_timer_cb, TIMER_TYPE_SACK, rxc->myAssoc, NULL);
         event_log(INTERNAL_EVENT_0, "Started SACK Timer !");
         rxc->timer_running = TRUE;
     }
     rxc->contains_valid_sack = TRUE;
-    return;
+    return SCTP_SUCCESS;
 }
 
 
@@ -771,118 +741,124 @@ void rxc_all_chunks_processed(boolean new_data_received)
   Function starts a SACK timer after data has been read by the ULP, and the
   buffer is about to change...
  */
-int rxc_start_sack_timer(unsigned int oldQueueLen)
+int rxc_start_sack_timer(Association* asok, unsigned int oldQueueLen)
 {
-    rxc_buffer *rxc;
-    int bytesQueued = 0;
-
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    rxc_buffer *rxc = NULL;
+    int bytesQueued = 0, send_result = 0;
+    
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
         error_log(ERROR_MINOR, "rxc_buffer instance not set - returning 0");
-        return (-1);
+        return SCTP_MODULE_NOT_FOUND;
     }
 
-    bytesQueued = se_getQueuedBytes();
+    bytesQueued = se_getQueuedBytesRecvQueue(asok);
     if (bytesQueued < 0) bytesQueued = 0;
+    
     /* no new data received, but we want updated SACK to be sent */
-    rxc_all_chunks_processed(FALSE);
+    rxc_all_chunks_processed(asok, FALSE);
     if ((rxc->my_rwnd - oldQueueLen < 2 * MAX_SCTP_PDU) &&
         (rxc->my_rwnd - bytesQueued >= 2 * MAX_SCTP_PDU)) {
-        /* send SACK at once */
-        rxc_create_sack(&rxc->last_address, TRUE);
-        bu_sendAllChunks(&rxc->last_address);
-        rxc_stop_sack_timer();
+        /* send SACK at once, we have again space to accept more than 2 MTU of data */
+        rxc_create_sack(asok, rxc->lastAddressKey, TRUE);
+        send_result = bu_sendAllChunks(asok, rxc->lastAddressKey);
+        rxc_stop_sack_timer(asok);
     } else {    /* normal application read, no need to rush things */
         if (rxc->timer_running != TRUE) {
-            rxc->sack_timer = adl_startTimer(rxc->delay, &rxc_sack_timer_cb, TIMER_TYPE_SACK, &(rxc->my_association), NULL);
+            rxc->sack_timer = adl_startTimer(rxc->delay, &rxc_sack_timer_cb, TIMER_TYPE_SACK, rxc->myAssoc, NULL);
             event_log(INTERNAL_EVENT_0, "Started SACK Timer !");
             rxc->timer_running = TRUE;
         }
     }
-    return 0;
+    return SCTP_SUCCESS;
 }
 
 
 /**
-  @return my current receiver window (32 bit unsigned value)
+ * returns the maximum local receiver window, that was initially announced
+ * @return my current receiver window (32 bit unsigned value)
  */
-unsigned int rxc_get_local_receiver_window(void)
+int rxc_get_local_receiver_window(Association* asok)
 {
-    rxc_buffer *rxc;
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    int win;
+
+    rxc_buffer *rxc = NULL;
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
-        error_log(ERROR_MINOR, "rxc_buffer instance not set - returning 0");
-        return (0);
+        win =  mdi_getDefaultMyRwnd(asok->sctpInstance);
+        event_logi(VERBOSE, "function rxc_get_my_receiver_window() rxc not set, returning  %d", win);
+        return win;
     }
     event_logi(VERBOSE, "function rxc_get_my_receiver_window() returns %u", rxc->my_rwnd);
-    return rxc->my_rwnd;
+    return (int)rxc->my_rwnd;
+
 }
 
 
 /**
-  @return my current sack delay in msecs
+ * returns the maximum time in msecs, that a SACK is delayed
+ * @return my current sack delay in msecs
  */
-int rxc_get_sack_delay(void)
+int rxc_get_sack_delay(Association* asok)
 {
-    rxc_buffer *rxc;
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    rxc_buffer *rxc = NULL;
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
         error_log(ERROR_MINOR, "rxc_buffer instance not set - returning default");
-        return (-1);
+        return mdi_getDefaultDelay(asok->sctpInstance);
     }
     event_logi(VERBOSE, "function rxc_get_sack_delay() returns %u", rxc->delay);
     return ((int)rxc->delay);
 }
 
 /**
-  @return my current sack delay in msecs
+ * set a new value for the maximum time that a SACK is delayed (in msecs)
+ * @param new_delay    new delay time (in msecs)
+ * @return SCTP_MODULE_NOT_FOUND, or SCTP_SUCCESS
  */
-int rxc_set_sack_delay(unsigned int new_delay)
+int rxc_set_sack_delay(Association* asok, unsigned int new_delay)
 {
-    rxc_buffer *rxc;
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    rxc_buffer *rxc = NULL;
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
-        error_log(ERROR_MINOR, "rxc_buffer instance not set - returning default");
-        return (-1);
+        error_log(ERROR_MINOR, "rxc_buffer instance not set - returning");
+        return SCTP_MODULE_NOT_FOUND;
     }
     rxc->delay = new_delay;
     event_logi(VERBOSE, "Setting new sack delay  to %u msecs", rxc->delay);
-    return 0;
+    return SCTP_SUCCESS;
 }
 
+
 /**
- Set the size of my receiver window. This needs to reflect buffer sizes.
- Beware, this is really only a DUMMY function, too !
- @param  new local receiver window (32 bit unsigned value)
- @return 0 on success, else -1 on failure
+ * Set the value that will be announced as my receiver window. This needs to
+ * reflect buffer sizes. Beware, this changes only the value, not the buffer size.
+ * @param  new local receiver window (32 bit unsigned value)
+ * @return SCTP_SUCCESS on success, else SCTP_MODULE_NOT_FOUND on failure
  */
-int rxc_set_local_receiver_window(unsigned int new_window)
+int rxc_set_local_receiver_window(Association* asok, unsigned int new_window)
 {
-    rxc_buffer *rxc;
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    rxc_buffer *rxc = NULL;
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
         error_log(ERROR_MAJOR, "rxc_buffer instance not set !");
-        return (-1);
+        return SCTP_MODULE_NOT_FOUND;
     }
     event_logi(VERBOSE, "function rxc_set_my_receiver_window(%u)", new_window);
     rxc->my_rwnd = new_window;
-    return 0;
+    return SCTP_SUCCESS;
 }
 
 
 /**
-  Get the number of the current cumulative TSN, that we may ack
-  @return my current ctsna (32 bit unsigned value)
+ * Get the number of the current cumulative TSN, that we may ack
+ * @return my current ctsna (32 bit unsigned value)
  */
-unsigned int rxc_read_cummulativeTSNacked(void)
+unsigned int rxc_read_cummulativeTSNacked(Association* asok)
 {
-    rxc_buffer *rxc;
-
-    rxc = (rxc_buffer *) mdi_readRX_control();
-    if (!rxc) {
-        error_log(ERROR_MAJOR, "rxc_buffer instance not set !");
-        return (0);
-    }
+    rxc_buffer *rxc = NULL;
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
+    assert(rxc);
     return (rxc->ctsna);
 }
 
@@ -890,11 +866,10 @@ unsigned int rxc_read_cummulativeTSNacked(void)
  * Helper function called, when we have gap reports in incoming
  * SACK chunks....
  */
-void rxc_send_sack_everytime(void)
+void rxc_send_sack_everytime(Association* asok)
 {
-    rxc_buffer *rxc;
-
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    rxc_buffer *rxc = NULL;
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
         error_log(ERROR_MAJOR, "rxc_buffer instance not set !");
         return;
@@ -906,60 +881,65 @@ void rxc_send_sack_everytime(void)
  * Helper function called, when we have no gap reports in incoming
  * SACK chunks....
  */
-void rxc_send_sack_every_second_time(void)
+void rxc_send_sack_every_second_time(Association* asok)
 {
-    rxc_buffer *rxc;
-
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    rxc_buffer *rxc = NULL;
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
         error_log(ERROR_MAJOR, "rxc_buffer instance not set !");
         return;
     }
     rxc->sack_flag = 2;
-
 }
 
 
 /**
- function only called in a restart case.
- Beware : this has been largely untested !
- @param  new_remote_TSN new tsn value of peer that has restarted
+ * Function re-creates proper strucures for receive-control module in the case
+ * of a re-start. Does not change RXC pointer ! Needs testing...
+ * @param  new_remote_TSN   new tsn value of peer that has restarted
+ * @param  my_rwnd          value of my advertised receiver window
  */
-void rxc_restart_receivecontrol(unsigned int my_rwnd, unsigned int new_remote_TSN)
+void rxc_restart_receivecontrol(Association* asok, unsigned int my_rwnd, unsigned int new_remote_TSN)
 {
     rxc_buffer *rxc;
 
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
         error_log(ERROR_MAJOR, "rxc_buffer instance not set !");
         return;
     }
-    rxc_stop_sack_timer();
+    rxc_stop_sack_timer(asok);
+
+    g_list_foreach(rxc->dup_list, &free_list_element, NULL);
+    g_list_free(rxc->dup_list);
     g_list_foreach(rxc->frag_list, &free_list_element, NULL);
     g_list_free(rxc->frag_list);
-    rxc->ctsna = new_remote_TSN - 1;
-    rxc->lowest = new_remote_TSN - 1;
+    rxc->frag_list  = NULL;
+    rxc->dup_list   = NULL;
+
+    rxc->ctsna   = new_remote_TSN - 1;
+    rxc->lowest  = new_remote_TSN - 1;
     rxc->highest = new_remote_TSN - 1;
     /* initialize and set up lists */
 
-    rxc->frag_list = NULL;
-    rxc->dup_list = NULL;
     rxc->contains_valid_sack = FALSE;
     rxc->timer_running = FALSE;
     rxc->datagrams_received = -1;
     rxc->sack_flag = 2;
-    rxc->last_address = 0;
+    rxc->lastAddressKey = 0;
     rxc->my_rwnd = my_rwnd;
-    rxc->my_association = mdi_readAssociationID();
+    rxc->myAssoc = asok;
     return;
 }
 
 
-int rxc_process_forward_tsn(void* chunk)
+int rxc_process_forward_tsn(Association* asok, void* chunk)
 {
-    rxc_buffer *rxc=NULL;
+    rxc_buffer *rxc = NULL;
     unsigned int fw_tsn;
-    unsigned int chunk_len;
+    unsigned int chk_len;
+
+
     unsigned int lo, hi;
     fragment32 *frag=NULL;
     GList *current = NULL;
@@ -967,20 +947,20 @@ int rxc_process_forward_tsn(void* chunk)
     SCTP_forward_tsn_chunk* chk = (SCTP_forward_tsn_chunk*)chunk;
 
     fw_tsn = ntohl(chk->forward_tsn);
-    chunk_len = ntohs(chk->chunk_header.chunk_length);
+    chk_len = ntohs(chk->chunk_header.chunk_length);
 
-    event_logii(INTERNAL_EVENT_0, "rxc_process_forward_tsn: %u, len:%u",fw_tsn, chunk_len);
-    rxc = (rxc_buffer *) mdi_readRX_control();
+    event_logii(INTERNAL_EVENT_0, "rxc_process_forward_tsn: %u (len=%u)",fw_tsn, chk_len);
+    rxc = (rxc_buffer *) mdi_readRX_control(asok);
     if (!rxc) {
         error_log(ERROR_MAJOR, "rxc_buffer instance not set !");
-        return (-1);
+        return SCTP_MODULE_NOT_FOUND;
     }
     /* discard old FORWARD_TSN */
-    if (after(rxc->ctsna, fw_tsn) || fw_tsn==rxc->ctsna || mdi_supportsPRSCTP() == FALSE) {
-        event_logii(VERBOSE, "rxc_process_forward_tsn --> discard fw_tsn !! (fw_tsn %u <= ctsna %u)",fw_tsn, rxc->ctsna);
-        return 0;
+    if (after(rxc->ctsna, fw_tsn) || fw_tsn==rxc->ctsna || mdi_supportsPRSCTP(asok) == FALSE) {
+        event_logii(VERBOSE, "rxc_process_forward_tsn: fw_tsn %u <= ctsna %u, discarding !",fw_tsn, rxc->ctsna);
+        /* FIXME: shouldn't we send an error chunk in the latest case ? */
+        return SCTP_SUCCESS;
     }
-
     current = g_list_first(rxc->frag_list);
 
     /* -get first fragment
@@ -1014,7 +994,7 @@ int rxc_process_forward_tsn(void* chunk)
                 rxc->ctsna = fw_tsn;
             } else {
                 error_log(ERROR_FATAL, "rxc_process_forward_tsn: impossible conditon");
-                abort();
+                exit(-1);
             }
         }
         /* we are still here, take next fragment == first fragment */
@@ -1024,9 +1004,9 @@ int rxc_process_forward_tsn(void* chunk)
         rxc->ctsna = fw_tsn;
         event_logi(VERBOSE, "rxc_process_forward_tsn: case4: set ctsna => %u !",fw_tsn);
     }
-    se_deliver_unreliably(rxc->ctsna, chk);
+    se_deliver_unreliably(asok, rxc->ctsna, chk);
 
-    rxc_all_chunks_processed(TRUE);
+    rxc_all_chunks_processed(asok, TRUE);
 
     return 0;
 }
