@@ -1,47 +1,48 @@
 /*
- * --------------------------------------------------------------------------
- *
- *           //=====   //===== ===//=== //===//  //       //   //===//
- *          //        //         //    //    // //       //   //    //
- *         //====//  //         //    //===//  //       //   //===<<
- *              //  //         //    //       //       //   //    //
- *       ======//  //=====    //    //       //=====  //   //===//
- *
- * -------------- An SCTP implementation according to RFC 4960 --------------
+ *  $Id: sbundling.c,v 1.6 2003/10/10 15:26:20 rohde Exp $
  *
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
- * Copyright (C) 2001-2004 Andreas Jungmaier
- * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Acknowledgements:
- * Realized in co-operation between Siemens AG and the University of
- * Duisburg-Essen, Institute for Experimental Mathematics, Computer
- * Networking Technology group.
- * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany
- * (Förderkennzeichen 01AK045).
+ * Realized in co-operation between Siemens AG
+ * and University of Essen, Institute of Computer Networking Technology.
+ *
+ * Acknowledgement
+ * This work was partially funded by the Bundesministerium fr Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * Contact: sctp-discussion@sctp.de
- *          thomas.dreibholz@gmail.com
- *          tuexen@fh-muenster.de
- *          andreas.jungmaier@web.de
+ * There are two mailinglists available at http://www.sctp.de which should be
+ * used for any discussion related to this implementation.
+ *
+ * Contact: discussion@sctp.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
+ *
+ * Purpose: This module bundels chunks chunks to be send into UDP-datagramms. Chunks are accepted
+ *          with the putChunk function until sendChunk is called, which causes the transmission of
+ *          all chunks accumulated so far.
+ *          In the receive direction, this module gets UDP-dataparts from SCTP message-distribution,
+ *          which are disassambled (debundled) into chunks. Depending on the chunk-type, the chunk
+ *          are distributed to SCTP-control, RX-control, pathmanagement or reliable transfer.
  */
 
 #include <stdio.h>
+#include <assert.h>
 
 #include "bundling.h"
 #include "messages.h"
@@ -50,14 +51,15 @@
 #include "reltransfer.h"
 #include "errorhandler.h"
 
-#define TOTAL_SIZE(buf)		((buf)->ctrl_position+(buf)->sack_position+(buf)->data_position- 2*sizeof(SCTP_common_header))
-#define SACK_SIZE(buf)		((buf)->ctrl_position+(buf)->data_position- sizeof(SCTP_common_header))
+#define TOTAL_SIZE(buf)     ((buf)->ctrl_position+(buf)->sack_position+(buf)->data_position- 2*sizeof(SCTP_common_header))
+#define SACK_SIZE(buf)      ((buf)->ctrl_position+(buf)->data_position- sizeof(SCTP_common_header))
+
 /**
  * this struct contains all data belonging to a bundling module
  */
 typedef struct bundling_instance_struct
 {
-    /*@{ */
+    /* @{ */
     /** buffer for control chunks */
     guchar ctrl_buf[MAX_MTU_SIZE];
     /** buffer for sack chunks */
@@ -79,15 +81,12 @@ typedef struct bundling_instance_struct
     gboolean sack_in_buffer;
     /** status flag for correct sequence of actions */
     gboolean got_send_request;
-    /** */
     gboolean got_send_address;
-    /** */
     gboolean locked;
     /** did we receive a shutdown, either by ULP or peer ? */
     gboolean got_shutdown;
-    /** */
-    guint requested_destination;
-    /*@} */
+    unsigned int requested_destination;
+    /* @} */
 
 }
 bundling_instance;
@@ -101,7 +100,7 @@ static bundling_instance *global_buffer;
 
 void bu_init_bundling(void)
 {
-    global_buffer = (bundling_instance*)bu_new();
+    global_buffer = bu_new();
 }
 
 /**
@@ -113,7 +112,7 @@ gpointer bu_new(void)
     /* Alloc new bundling_instance data struct */
     bundling_instance *ptr;
 
-    ptr = (bundling_instance*)malloc(sizeof(bundling_instance));
+    ptr = malloc(sizeof(bundling_instance));
     if (!ptr) {
         error_log(ERROR_MAJOR, "Malloc failed");
         return 0;
@@ -136,24 +135,26 @@ gpointer bu_new(void)
  *
  * @param Pointer which was returned by bu_new()
  */
-void bu_delete(gpointer buPtr)
+void bu_delete(Association* asok)
 {
     event_log(INTERNAL_EVENT_0, "deleting bundling");
-    free(buPtr);
+    assert(asok);
+    assert(asok->bundling);
+    free(asok->bundling);
 }
 
 
 
 /**
  * Keep sender from sending data right away - wait after received chunks have
- * been diassembled completely.
+ * been disassembled completely.
  */
-void bu_lock_sender()
+void bu_lock_sender(Association* asok)
 {
     bundling_instance *bu_ptr;
     event_log(VERBOSE, "bu_lock_sender() was called... ");
 
-    bu_ptr = (bundling_instance *) mdi_readBundling();
+    bu_ptr = (bundling_instance *) mdi_readBundling(asok);
     if (!bu_ptr) {              /* Assume that no association exists, so we take the global bundling buffer */
         event_log(VERBOSE, "Setting global bundling buffer ");
         bu_ptr = global_buffer;
@@ -166,11 +167,11 @@ void bu_lock_sender()
  * Enable sending again - wait after received chunks have
  * been diassembled completely.
  */
-void bu_unlock_sender(guint* ad_idx)
+void bu_unlock_sender(Association* asok, guint addressKey)
 {
     bundling_instance *bu_ptr;
 
-    bu_ptr = (bundling_instance *) mdi_readBundling();
+    bu_ptr = (bundling_instance *) mdi_readBundling(asok);
     if (!bu_ptr) {              /* Assume that no association exists, so we take the global bundling buffer */
         event_log(VERBOSE, "Setting global bundling buffer ");
         bu_ptr = global_buffer;
@@ -179,7 +180,7 @@ void bu_unlock_sender(guint* ad_idx)
     event_logi(VERBOSE, "bu_unlock_sender() was called..and got %s send request -> processing",
        (bu_ptr->got_send_request == TRUE)?"A":"NO");
 
-    if (bu_ptr->got_send_request == TRUE) bu_sendAllChunks(ad_idx);
+    if (bu_ptr->got_send_request == TRUE) bu_sendAllChunks(asok, addressKey);
 
 }
 
@@ -190,31 +191,30 @@ void bu_unlock_sender(guint* ad_idx)
  * @param chunk pointer to chunk, that is to be put in the bundling buffer
  * @return error value, 0 on success, -1 on error
  */
-gint bu_put_SACK_Chunk(SCTP_sack_chunk * chunk, unsigned int * dest_index)
+gint bu_put_SACK_Chunk(Association* asok, SCTP_sack_chunk * chunk, unsigned int destKey)
 {
-    bundling_instance *bu_ptr;
+    gint result;
+    bundling_instance *bu_ptr = NULL;
     gboolean lock;
 
     event_log(INTERNAL_EVENT_0, "bu_put_SACK_Chunk() was called ");
 
-    bu_ptr = (bundling_instance *) mdi_readBundling();
+    bu_ptr = (bundling_instance *) mdi_readBundling(asok);
 
     if (!bu_ptr) {              /* Assume that no association exists, so we take the global bundling buffer */
         event_log(VERBOSE, "Copying SACK to global bundling buffer ");
         bu_ptr = global_buffer;
     }
 
-    if (SACK_SIZE(bu_ptr) + CHUNKP_LENGTH((SCTP_chunk_header *) chunk) >= MAX_SCTP_PDU) {
+    if (SACK_SIZE(bu_ptr) + CHUNKP_LENGTH((SCTP_chunk_header *) chunk) > pm_readAsocMTU(asok)) {
         lock = bu_ptr->locked;
-         event_logi(VERBOSE,
-                  "Chunk Length exceeded MAX_SCTP_PDU : sending chunk to address %u !",
-                    (dest_index==NULL)?0:*dest_index);
+        event_logi(VERBOSE, "Chunk Length exceeded SCTP MTU : sending chunk to address %u !", destKey);
         if (lock) bu_ptr->locked = FALSE;
-        bu_sendAllChunks(dest_index);
+        result = bu_sendAllChunks(asok, destKey);
         if (lock) bu_ptr->locked = TRUE;
-    } else if (dest_index != NULL) {
+    } else if (destKey != 0) {
         bu_ptr->got_send_address = TRUE;
-        bu_ptr->requested_destination = *dest_index;
+        bu_ptr->requested_destination = destKey;
     }
 
     if (bu_ptr->sack_in_buffer == TRUE) { /* multiple calls in between */
@@ -242,32 +242,32 @@ gint bu_put_SACK_Chunk(SCTP_sack_chunk * chunk, unsigned int * dest_index)
  * @param chunk pointer to chunk, that is to be put in the bundling buffer
  * @return TODO : error value, 0 on success
  */
-gint bu_put_Ctrl_Chunk(SCTP_simple_chunk * chunk,unsigned int * dest_index)
+gint bu_put_Ctrl_Chunk(Association* asok, SCTP_simple_chunk * chunk, unsigned int destKey)
 {
-    bundling_instance *bu_ptr;
+    gint result;
+    bundling_instance *bu_ptr = NULL;
     gint count;
     gboolean lock;
 
     event_log(INTERNAL_EVENT_0, "bu_put_Ctrl_Chunk() was called");
 
-    bu_ptr = (bundling_instance *) mdi_readBundling();
+    bu_ptr = (bundling_instance *) mdi_readBundling(asok);
 
     if (!bu_ptr) {              /* Assume that no association exists, so we take the global bundling buffer */
         event_log(VERBOSE, "Copying Control Chunk to global bundling buffer ");
         bu_ptr = global_buffer;
     }
 
-    if (TOTAL_SIZE(bu_ptr) + CHUNKP_LENGTH((SCTP_chunk_header *) chunk) >= MAX_SCTP_PDU) {
+    if (TOTAL_SIZE(bu_ptr) + CHUNKP_LENGTH((SCTP_chunk_header *) chunk) > pm_readAsocMTU(asok)) {
         lock = bu_ptr->locked;
-        event_logi(VERBOSE,
-                  "Chunk Length exceeded MAX_SCTP_PDU : sending chunk to address %u !",
-                    (dest_index==NULL)?0:*dest_index);
+        event_logiii(VERBOSE, "Chunk Length exceeded SCTP MTU(would be %u allowed: %u) : sending chunk to address %u !", TOTAL_SIZE(bu_ptr) + CHUNKP_LENGTH((SCTP_chunk_header *) chunk),pm_readAsocMTU(asok),destKey);
+                    
         if (lock) bu_ptr->locked = FALSE;
-        bu_sendAllChunks(dest_index);
+        result = bu_sendAllChunks(asok, destKey);
         if (lock) bu_ptr->locked = TRUE;
-    } else if (dest_index != NULL) {
+    } else if (destKey != 0) {
         bu_ptr->got_send_address = TRUE;
-        bu_ptr->requested_destination = *dest_index;
+        bu_ptr->requested_destination = destKey;
     }
 
     memcpy(&(bu_ptr->ctrl_buf[bu_ptr->ctrl_position]), chunk,
@@ -287,11 +287,11 @@ gint bu_put_Ctrl_Chunk(SCTP_simple_chunk * chunk,unsigned int * dest_index)
     return 0;
 }
 
-gboolean bu_userDataOutbound(void)
+gboolean bu_userDataOutbound(Association* asok)
 {
-    bundling_instance *bu_ptr;
+    bundling_instance *bu_ptr = NULL;
 
-    bu_ptr = (bundling_instance *) mdi_readBundling();
+    bu_ptr = (bundling_instance *) mdi_readBundling(asok);
     if (!bu_ptr) {              /* Assume that no association exists, so we take the global bundling buffer */
         event_log(VERBOSE, "Setting global bundling buffer ");
         bu_ptr = global_buffer;
@@ -307,32 +307,32 @@ gboolean bu_userDataOutbound(void)
  * @param chunk pointer to chunk, that is to be put in the bundling buffer
  * @return TODO : error value, 0 on success
  */
-gint bu_put_Data_Chunk(SCTP_simple_chunk * chunk,unsigned int * dest_index)
+gint bu_put_Data_Chunk(Association* asok, SCTP_simple_chunk * chunk,unsigned int destKey)
 {
-    bundling_instance *bu_ptr;
+    gint result;
+    bundling_instance *bu_ptr = NULL;
     gint count;
     gboolean lock;
 
     event_log(INTERNAL_EVENT_0, "bu_put_Data_Chunk() was called ");
 
-    bu_ptr = (bundling_instance *) mdi_readBundling();
+    bu_ptr = (bundling_instance *) mdi_readBundling(asok);
 
     if (!bu_ptr) {              /* Assume that no association exists, so we take the global bundling buffer */
         event_log(VERBOSE, "Copying data to global bundling buffer ");
         bu_ptr = global_buffer;
     }
 
-    if (TOTAL_SIZE(bu_ptr) + CHUNKP_LENGTH((SCTP_chunk_header *) chunk) >= MAX_SCTP_PDU) {
+    if (TOTAL_SIZE(bu_ptr) + CHUNKP_LENGTH((SCTP_chunk_header *) chunk) > pm_readAsocMTU(asok)) {
         lock = bu_ptr->locked;
-        event_logi(VERBOSE,
-                  "Chunk Length exceeded MAX_SCTP_PDU : sending chunk to address %u !",
-                    (dest_index==NULL)?0:*dest_index);
+        event_logi(VERBOSE, "Chunk Length exceeded SCTP MTU : sending chunk to address %u !", destKey);
+
         if (lock) bu_ptr->locked = FALSE;
-        bu_sendAllChunks(dest_index);
+        result = bu_sendAllChunks(asok, destKey);
         if (lock) bu_ptr->locked = TRUE;
-    } else if (dest_index != NULL) {
+    } else if (destKey != 0) {
         bu_ptr->got_send_address = TRUE;
-        bu_ptr->requested_destination = *dest_index;
+        bu_ptr->requested_destination = destKey;
     }
     memcpy(&(bu_ptr->data_buf[bu_ptr->data_position]), chunk,
            CHUNKP_LENGTH((SCTP_chunk_header *) chunk));
@@ -350,9 +350,8 @@ gint bu_put_Data_Chunk(SCTP_simple_chunk * chunk,unsigned int * dest_index)
                 CHUNKP_LENGTH((SCTP_chunk_header *) chunk), TOTAL_SIZE(bu_ptr));
 
     bu_ptr->data_in_buffer = TRUE;
-
-    /* if SACK is waiting, force sending it along */
-    if (rxc_sack_timer_is_running() == TRUE) rxc_create_sack(dest_index, TRUE);
+    /* if SACK is waiting, force sending it along - this may send the whole PDU at once !!!  */
+    if (rxc_sack_timer_is_running(asok) == TRUE) rxc_create_sack(asok, destKey, TRUE);
 
     return 0;
 }
@@ -368,14 +367,14 @@ gint bu_put_Data_Chunk(SCTP_simple_chunk * chunk,unsigned int * dest_index)
  *  @return                 Errorcode (0 for good case: length bytes sent; 1 or -1 for error)
  *  @param   ad_idx     pointer to address index or NULL if data is to be sent to default address
  */
-gint bu_sendAllChunks(guint * ad_idx)
+gint bu_sendAllChunks(Association* asok, unsigned int destKey)
 {
     gint result, send_len = 0;
     guchar *send_buffer = NULL;
-    bundling_instance *bu_ptr;
-    gshort idx = 0;
+    bundling_instance *bu_ptr = NULL;
+    unsigned int tmpKey = 0;
 
-    bu_ptr = (bundling_instance *) mdi_readBundling();
+    bu_ptr = (bundling_instance *) mdi_readBundling(asok);
 
     event_log(INTERNAL_EVENT_0, "bu_sendAllChunks() is being executed...");
 
@@ -385,87 +384,68 @@ gint bu_sendAllChunks(guint * ad_idx)
     }
     if (bu_ptr->locked == TRUE) {
         bu_ptr->got_send_request = TRUE;
-        if (ad_idx) {
+        if (destKey != 0) {
             bu_ptr->got_send_address = TRUE;
-            bu_ptr->requested_destination = *ad_idx;
+            bu_ptr->requested_destination = destKey;
         }
         event_log(INTERNAL_EVENT_0, "bu_sendAllChunks : sender is LOCKED ---> returning ");
-        return 1;
+        return SCTP_UNSPECIFIED_ERROR;
     }
 
     /* TODO : more intelligent path selection strategy */
     /*         should take into account PM_INACTIVE */
-    if (ad_idx != NULL) {
-        if (*ad_idx > 0xFFFF) {
-            error_log(ERROR_FATAL, "address_index too big !");
-        } else {
-            idx = (short) *ad_idx;
-        }
+    if (destKey != 0) {
+        tmpKey = destKey;
     } else {
         if (bu_ptr->got_send_address) {
-            idx = (short)bu_ptr->requested_destination;
+            tmpKey = bu_ptr->requested_destination;
+            /* this might also be 0 */
         } else {
-            idx = -1; /* use last from address */
+            tmpKey = 0; /* use last from address, or primary address */
         }
     }
 
-    event_logi(VVERBOSE, "bu_sendAllChunks : send to path %d ", idx);
+    event_logi(VVERBOSE, "bu_sendAllChunks : send to path key %u ", tmpKey);
 
     if (bu_ptr->sack_in_buffer)             send_buffer = bu_ptr->sack_buf;
     else if (bu_ptr->ctrl_chunk_in_buffer)  send_buffer = bu_ptr->ctrl_buf;
     else if (bu_ptr->data_in_buffer)        send_buffer = bu_ptr->data_buf;
     else {
         error_log(ERROR_MINOR, "Nothing to send, but bu_sendAllChunks was called !");
-        return 1;
+        return SCTP_SPECIFIC_FUNCTION_ERROR;
     }
 
     if (bu_ptr->sack_in_buffer) {
-        rxc_stop_sack_timer();
+        rxc_stop_sack_timer(asok);
         /* SACKs by default go to the last active address, from which data arrived */
         send_len = bu_ptr->sack_position; /* at least sizeof(SCTP_common_header) */
         /* at most pointing to the end of SACK chunk */
-        event_logi(VVERBOSE, "bu_sendAllChunks(sack) : send_len == %d ", send_len);
         if (bu_ptr->ctrl_chunk_in_buffer) {
-            memcpy(&send_buffer[send_len],
-                   &(bu_ptr->ctrl_buf[sizeof(SCTP_common_header)]),
+            memcpy(&send_buffer[send_len], &(bu_ptr->ctrl_buf[sizeof(SCTP_common_header)]),
                    (bu_ptr->ctrl_position - sizeof(SCTP_common_header)));
             send_len += bu_ptr->ctrl_position - sizeof(SCTP_common_header);
-            event_logi(VVERBOSE, "bu_sendAllChunks(sack+ctrl) : send_len == %d ", send_len);
         }
         if (bu_ptr->data_in_buffer) {
-            memcpy(&send_buffer[send_len],
-                   &(bu_ptr->data_buf[sizeof(SCTP_common_header)]),
+            memcpy(&send_buffer[send_len], &(bu_ptr->data_buf[sizeof(SCTP_common_header)]),
                    (bu_ptr->data_position - sizeof(SCTP_common_header)));
             send_len += bu_ptr->data_position - sizeof(SCTP_common_header);
-            event_logi(VVERBOSE, "bu_sendAllChunks(sack+data) : send_len == %d ", send_len);
         }
     } else if (bu_ptr->ctrl_chunk_in_buffer) {
         send_len = bu_ptr->ctrl_position;
-        event_logi(VVERBOSE, "bu_sendAllChunks(ctrl) : send_len == %d ", send_len);
         if (bu_ptr->data_in_buffer) {
-            memcpy(&send_buffer[send_len],
-                   &(bu_ptr->data_buf[sizeof(SCTP_common_header)]),
+            memcpy(&send_buffer[send_len], &(bu_ptr->data_buf[sizeof(SCTP_common_header)]),
                    (bu_ptr->data_position - sizeof(SCTP_common_header)));
             send_len += bu_ptr->data_position - sizeof(SCTP_common_header);
-            event_logi(VVERBOSE, "bu_sendAllChunks(ctrl+data) : send_len == %d ", send_len);
         }
 
     } else if (bu_ptr->data_in_buffer) send_len = bu_ptr->data_position;
 
-    event_logi(VVERBOSE, "bu_sendAllChunks(finally) : send_len == %d ", send_len);
+    if ((bu_ptr->data_in_buffer) && (tmpKey != 0)) pm_chunksSentOn(asok, tmpKey);
 
-    if (send_len > 1480) {
-        fprintf(stderr, "MTU definitely exceeded (%u) - aborting\n",send_len);
-        fprintf(stderr, "sack_position: %u, ctrl_position: %u, data_position: %u\n",
-            bu_ptr->sack_position,bu_ptr->ctrl_position,bu_ptr->data_position);
-        abort();
-    }
+    event_logii(VERBOSE, "bu_sendAllChunks() : sending message len==%u to adress key=%u", send_len, tmpKey);
+    assert(send_len <= 1480);
 
-    if ((bu_ptr->data_in_buffer) && (idx != -1)) pm_chunksSentOn(idx);
-
-    event_logii(VERBOSE, "bu_sendAllChunks() : sending message len==%u to adress idx=%d", send_len, idx);
-
-    result = mdi_send_message((SCTP_message *) send_buffer, send_len, idx);
+    result = mdi_send_message(asok, (SCTP_message *) send_buffer, send_len, tmpKey);
 
     event_logi(VVERBOSE, "bu_sendAllChunks(): result == %s ", (result==0)?"OKAY":"ERROR");
 
@@ -482,7 +462,3 @@ gint bu_sendAllChunks(guint * ad_idx)
 
     return result;
 }
-
-
-
-
