@@ -1,45 +1,47 @@
 /*
- * --------------------------------------------------------------------------
- *
- *           //=====   //===== ===//=== //===//  //       //   //===//
- *          //        //         //    //    // //       //   //    //
- *         //====//  //         //    //===//  //       //   //===<<
- *              //  //         //    //       //       //   //    //
- *       ======//  //=====    //    //       //=====  //   //===//
- *
- * -------------- An SCTP implementation according to RFC 4960 --------------
+ *  $Id: sctp.h,v 1.10 2004/01/13 16:13:14 ajung Exp $
  *
+ * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
- * Copyright (C) 2001-2004 Andreas Jungmaier
- * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Acknowledgements:
- * Realized in co-operation between Siemens AG and the University of
- * Duisburg-Essen, Institute for Experimental Mathematics, Computer
- * Networking Technology group.
- * This work was partially funded by the Bundesministerium fuer Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany
- * (Förderkennzeichen 01AK045).
+ * Realized in co-operation between Siemens AG
+ * and University of Essen, Institute of Computer Networking Technology.
+ *
+ * Acknowledgement
+ * This work was partially funded by the Bundesministerium fr Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * Contact: sctp-discussion@sctp.de
- *          thomas.dreibholz@gmail.com
- *          tuexen@fh-muenster.de
- *          andreas.jungmaier@web.de
+ * There are two mailinglists available at www.sctp.de which should be used for
+ * any discussion related to this implementation.
+ *
+ * Contact: discussion@sctp.de
+ *          Michael.Tuexen@icn.siemens.de
+ *          ajung@exp-math.uni-essen.de
+ *
+ * Purpose: This header-file defines the API to the SCTP library, and contains
+ *                      all functions and structures the library offers to an application.
+ *          Further comments on these functions can be found in the implemen-
+ *          tation of this interface in the files distribution.c, adaptation.c
+ *          and in the accompanying manual (see subdirectory manual)
+ * 
+ *  function prefix: sctp_         
  */
+           
 
 #ifndef SCTP_H
 #define SCTP_H
@@ -64,9 +66,10 @@
 extern "C" {
 #endif
 
+
 #define SCTP_MAJOR_VERSION      1
-#define SCTP_MINOR_VERSION      0
-#define SCTP_TINY_VERSION       8
+#define SCTP_MINOR_VERSION      1
+#define SCTP_TINY_VERSION       5
 
 /* the maximum length of an IP address string (IPv4 or IPv6, NULL terminated) */
 /* see RFC 1884 (mixed IPv6/Ipv4 addresses)   */
@@ -74,11 +77,25 @@ extern "C" {
 
 /** this parameter specifies the maximum number of addresses that an endpoint may have */
 #define SCTP_MAX_NUM_ADDRESSES      20
+                                
+/**
+ * reasonable sized SACK, SCTP and IP header + one data chunk should be less than MTU
+ * this is for ethernet..... ;-)
+ *
+ * these are also defined in "globals.h"
+ * if you chage them here, be sure to also change them there
+ */
+#ifndef SCTP_MAXIMUM_DATA_LENGTH
+    #define SCTP_MAXIMUM_DATA_LENGTH     1404
+#endif
 
-/* reasonable sized SACK, SCTP and IP header + one data chunk should be less than MTU */
-/* this is for ethernet..... ;-) */
-#define SCTP_MAXIMUM_DATA_LENGTH     1400
-/******************** Defines *********************************************************************/
+#ifndef SCTP_MAXIMUM_MESSAGE_LENGTH
+    #define SCTP_MAXIMUM_MESSAGE_LENGTH  32000
+#endif
+
+
+/******************** Defines 
+*********************************************************************/
 /* the possible 7 states of an association */
 #define SCTP_CLOSED                     0
 #define SCTP_COOKIE_WAIT                1
@@ -92,11 +109,17 @@ extern "C" {
 /* Pathstatus, used with networkstatus primitives */
 #define SCTP_PATH_OK                    0
 #define SCTP_PATH_UNREACHABLE           1
+/* result values used with asconfStatusNotif */
 #define SCTP_PATH_ADDED                 2
 #define SCTP_PATH_REMOVED               3
-#define SCTP_PATH_CONFIRMED             4
-#define SCTP_PATH_UNCONFIRMED           5
-
+#define SCTP_LOCAL_ADDRESS_ADDED        4
+#define SCTP_LOCAL_ADDRESS_REMOVED      5
+/* peer acked our ASCONF                */
+#define SCTP_ASCONF_SUCCEEDED           6
+/* peer did not like our ASCONF         */
+#define SCTP_ASCONF_FAILED              7
+#define SCTP_PATH_CONFIRMED             8
+#define SCTP_PATH_UNCONFIRMED           9
 
 /* for use in  sctp_changeHeartBeat */
 #define SCTP_HEARTBEAT_ON               1
@@ -110,18 +133,18 @@ extern "C" {
 #define SCTP_BUNDLING_DISABLED          1
 
 /* these are also for sctp_send() */
-#define SCTP_USE_PRIMARY                    -1
-#define SCTP_INFINITE_LIFETIME              0xFFFFFFFF
-#define SCTP_NO_RETRANSMISSION              0
-#define SCTP_SEND_RELIABLE                  SCTP_INFINITE_LIFETIME
-#define SCTP_NO_CONTEXT                     NULL
-#define SCTP_GENERIC_PAYLOAD_PROTOCOL_ID    0
+#define SCTP_USE_PRIMARY                0
+#define SCTP_SEND_RELIABLE              SCTP_INFINITE_LIFETIME
+#define SCTP_INFINITE_LIFETIME          0xFFFFFFFF
+#define SCTP_NO_RETRANSMISSION          0
+#define SCTP_NO_CONTEXT                 NULL
+#define SCTP_GENERIC_PAYLOAD_PROTOCOL_ID 0
 /* these are for sctp_receive() */
-#define SCTP_MSG_DEFAULT                    0x00
-#define SCTP_MSG_PEEK                       0x02
+#define SCTP_MSG_DEFAULT				0x00
+#define SCTP_MSG_PEEK					0x02
 
-#define SCTP_CHECKSUM_ALGORITHM_CRC32C      0x1
-#define SCTP_CHECKSUM_ALGORITHM_ADLER32     0x2
+#define SCTP_CHECKSUM_ALGORITHM_CRC32C   0x1
+#define SCTP_CHECKSUM_ALGORITHM_ADLER32  0x2
 
 /* Here are some error codes that are returned by some functions         */
 /* this list may be enhanced or become more extensive in future releases */
@@ -141,16 +164,19 @@ extern "C" {
 #define SCTP_WRONG_STATE                    -13
 #define SCTP_BUFFER_TOO_SMALL               -14
 #define SCTP_NO_CHUNKS_IN_QUEUE             -15
-#define SCTP_INSTANCE_IN_USE                -16
+#define SCTP_PEER_ERROR                     -16
+#define SCTP_MESSAGE_TOO_LONG               -17
+#define SCTP_FALSE_SIGNATURE                -18
 #define SCTP_SPECIFIC_FUNCTION_ERROR        1
 
-/* some constants for return codes from COMMUNICATION LOST or COMMUNICATION UP callbacks */
+/* some constants for return codes from COMMUNICATION LOST or COMMUNICATION UP callbacks 
+*/
 #define SCTP_COMM_LOST_ABORTED                   1
 #define SCTP_COMM_LOST_ENDPOINT_UNREACHABLE      2
 #define SCTP_COMM_LOST_EXCEEDED_RETRANSMISSIONS  3
 #define SCTP_COMM_LOST_NO_TCB                    4
 /* maybe some others............. */
-#define SCTP_COMM_LOST_INVALID_PARAMETER         8
+#define SCTP_COMM_LOST_ZERO_STREAMS              8
 /* called if peer does not recognize some of our parameters */
 #define SCTP_COMM_LOST_FAILURE                   9
 #define SCTP_SHUTDOWN_COMPLETE                  10
@@ -162,15 +188,14 @@ extern "C" {
 #define SCTP_SEND_QUEUE                         1
 
 
-/******************** Structure Definitions *******************************************************/
-
+/******************** Structure Definitions ********************************************/
 
 typedef
 /**
    This struct containes the pointers to ULP callback functions.
    Each SCTP-instance can have its own set of callback functions.
    The callback functions of each SCTP-instance can be found by
-   first reading the datastruct of an association from the list of
+   first reading the datastruct of an association from the list of 
    associations. The datastruct of the association contains the name
    of the SCTP instance to which it belongs. With the name of the SCTP-
    instance its datastruct can be read from the list of SCTP-instances.
@@ -189,7 +214,7 @@ struct SCTP_ulp_Callbacks
      *  @param 7 unordered flag (TRUE==1==unordered, FALSE==0==normal, numbered chunk)
      *  @param 8 pointer to ULP data
      */
-    void (*dataArriveNotif) (unsigned int, unsigned short, unsigned int, unsigned short, unsigned int, unsigned int, unsigned int,   void*);
+    void (*dataArriveNotif) (unsigned int, unsigned int, unsigned int, unsigned short, unsigned int, unsigned int, unsigned int,   void*);
     /**
      * indicates a send failure (chapter 10.2.B).
      *  @param 1 associationID
@@ -206,7 +231,7 @@ struct SCTP_ulp_Callbacks
      *  @param 3 newState
      *  @param 4 pointer to ULP data
      */
-    void (*networkStatusChangeNotif) (unsigned int, short, unsigned short, void*);
+    void (*networkStatusChangeNotif) (unsigned int, unsigned int, int, void*);
     /**
      * indicates that a association is established (chapter 10.2.D).
      *  @param 1 associationID
@@ -215,19 +240,21 @@ struct SCTP_ulp_Callbacks
      *  @param 4 number input streamns
      *  @param 5 number output streams
      *  @param 6 int  supportPRSCTP (0=FALSE, 1=TRUE)
-     *  @param 7 pointer to ULP data, usually NULL
+     *  @param 7 int  adaptationLayerIndicationLen (0==Not Set)
+     *  @param 8 void*  adaptationLayerIndication
+     *  @param 9 pointer to ULP data, usually NULL
      *  @return the callback is to return a pointer, that will be transparently returned with every callback
      */
     void* (*communicationUpNotif) (unsigned int, int, unsigned int,
                                    unsigned short, unsigned short,
-                                   int, void*);
+                                   int, int, void*, void*);
     /**
      * indicates that communication was lost to peer (chapter 10.2.E).
      *  @param 1 associationID
      *  @param 2 status, type of event
      *  @param 3 pointer to ULP data
      */
-    void (*communicationLostNotif) (unsigned int, unsigned short, void*);
+    void (*communicationLostNotif) (unsigned int, int, void*);
     /**
      * indicates that communication had an error. (chapter 10.2.F)
      * Currently not implemented !?
@@ -235,14 +262,8 @@ struct SCTP_ulp_Callbacks
      *  @param 2 status, type of error
      *  @param 3 pointer to ULP data
      */
-    void (*communicationErrorNotif) (unsigned int, unsigned short, void*);
-    /**
-     * indicates that a RESTART has occurred. (chapter 10.2.G)
-     *  @param 1 associationID
-     *  @param 2 pointer to ULP data
-     */
-    void (*restartNotif) (unsigned int, void*);
-    /**
+    void (*communicationErrorNotif) (unsigned int, int, void*);
+	/**
      * indicates that a SHUTDOWN has been received by the peer. Tells the
      * application to stop sending new data.
      *  @param 0 instanceID
@@ -250,6 +271,12 @@ struct SCTP_ulp_Callbacks
      *  @param 2 pointer to ULP data
      */
     void (*peerShutdownReceivedNotif) (unsigned int, void*);
+    /**
+     * indicates that a RESTART has occurred. (chapter 10.2.G)
+     *  @param 1 associationID
+     *  @param 2 pointer to ULP data
+     */
+    void (*restartNotif) (unsigned int, void*);
     /**
      * indicates that a SHUTDOWN has been COMPLETED. (chapter 10.2.H)
      *  @param 0 instanceID
@@ -267,15 +294,6 @@ struct SCTP_ulp_Callbacks
      *  @param 4 pointer to ULP data
      */
     void (*queueStatusChangeNotif) (unsigned int, int, int, int, void*);
-    /**
-     * indicates that a ASCONF request from the ULP has succeeded or failed.
-     *  @param 0 associationID
-     *  @param 1 correlation ID
-     *  @param 2 result (int, negative for error)
-     *  @param 3 pointer to a temporary, request specific structure (NULL if not needed)
-     *  @param 4 pointer to ULP data
-     */
-    void (*asconfStatusNotif) (unsigned int, unsigned int, int, void*, void*);
     /* @} */
 }SCTP_ulpCallbacks;
 
@@ -312,7 +330,6 @@ struct SCTP_Library_Parameters {
      */
     int supportADDIP;
 
-
 }SCTP_LibraryParameters;
 
 
@@ -326,10 +343,12 @@ typedef
  */
 struct SCTP_Instance_Parameters {
     /* @{ */
-    /* this is read-only (get) */
-    unsigned int   noOfLocalAddresses;
-    /* this is read-only (get) */
-    unsigned char  localAddressList[SCTP_MAX_NUM_ADDRESSES][SCTP_MAX_IP_LEN];
+    /** gives the number of local addresses configured for this instance
+     *  this is read-only (get) */
+    unsigned int noOfLocalAddresses;
+    /** gives the local addresses of this instance as strings
+     *  this is read-only (get) */
+    unsigned char localAddressList[SCTP_MAX_NUM_ADDRESSES][SCTP_MAX_IP_LEN];
     /** the initial round trip timeout */
     unsigned int rtoInitial;
     /** the minimum timeout value */
@@ -343,7 +362,7 @@ struct SCTP_Instance_Parameters {
     /**  (get/set) */
     unsigned short inStreams;
     /** does the instance by default signal unreliable streams (as a server) no==0, yes==1 */
-    unsigned int supportUnreliableStreams;
+    unsigned int supportPRSCTP;
     /** does the instance by default signal unreliable streams (as a server) no==0, yes==1 */
     unsigned int supportADDIP;
     /** maximum retransmissions per association */
@@ -358,7 +377,7 @@ struct SCTP_Instance_Parameters {
     unsigned int delay;
     /** per instance: for the IP type of service field. */
     unsigned char ipTos;
-    /** limit the number of chunks queued in the send queue */
+    /** limits the number of chunks queued in the send queue (0 for unlimited queue) */
     unsigned int maxSendQueue;
     /** currently unused, may limit the number of chunks queued in the receive queue later.
      *  Is this really needed ? The protocol limits the receive queue with
@@ -367,7 +386,7 @@ struct SCTP_Instance_Parameters {
     /**
      * maximum number of associations we want. Is this limit greater than 0,
      * implementation will automatically send ABORTs to incoming INITs, when
-     * there are that many associations !
+     * there are that many associations ! Still unsupported.....bear with us :-)
      */
     unsigned int maxNumberOfAssociations;
     /* @} */
@@ -385,9 +404,11 @@ struct SCTP_Association_Status
 {
     /* @{ */
     /** (get)  */
-    unsigned short state;
+    int state;
     /**  (get) */
-    unsigned short numberOfAddresses;
+    unsigned int numberOfDestinationPaths;
+    /**  (get) an array of keys */
+    unsigned int destinationPathIDs[SCTP_MAX_NUM_ADDRESSES];
     /**  (get) */
     unsigned char  primaryDestinationAddress[SCTP_MAX_IP_LEN];
     /**  (get) */
@@ -399,11 +420,11 @@ struct SCTP_Association_Status
     /**  (get) */
     unsigned short inStreams;
     /** does the assoc support unreliable streams  no==0, yes==1 */
-    unsigned int supportUnreliableStreams;
+    unsigned int supportPRSCTP;
     /** does the assoc support adding/deleting IP addresses no==0, yes==1 */
     unsigned int supportADDIP;
     /**  (get/set) */
-    unsigned short primaryAddressIndex;
+    unsigned int   primaryAddressKey;
     /**  (get) */
     unsigned int   currentReceiverWindowSize;
     /**  (get) */
@@ -434,11 +455,9 @@ struct SCTP_Association_Status
     unsigned int delay;
     /** (get/set) per instance: for the IP type of service field. */
     unsigned char ipTos;
-    /**  limit the number of chunks queued in the send queue */
+    /** limits the number of chunks queued in the send queue (0 for unlimited queue) */
     unsigned int maxSendQueue;
-    /** currently unused, may limit the number of chunks queued in the receive queue later.
-     *  Is this really needed ? The protocol limits the receive queue with
-     *  window advertisement of arwnd==0  */
+    /** currently unused */
     unsigned int maxRecvQueue;
     /* @} */
 } SCTP_AssociationStatus;
@@ -455,8 +474,14 @@ struct SCTP_Path_Status
     /* @{ */
     /**   */
     unsigned char destinationAddress[SCTP_MAX_IP_LEN];
+    /* this is the key corresponding to the path */
+    unsigned int pathKey;
+    /* this is the current index of this path                 */
+    /* this value may change if addresses are added, the path */
+    /* key remains the same over the lifetime of the assoc    */
+    int pathIndex;
     /**  SCTP_PATH_ACTIVE  0, SCTP_PATH_INACTIVE   1    */
-    short state;
+    int state;
     /** smoothed round trip time in msecs */
     unsigned int srtt;
     /** current rto value in msecs */
@@ -483,7 +508,7 @@ struct SCTP_Path_Status
 }SCTP_PathStatus;
 
 
-/******************** Function Definitions ********************************************************/
+/******************* Function Definitions ***********************************************/
 
 /**
  * Function that needs to be called in advance to all library calls.
@@ -494,7 +519,6 @@ struct SCTP_Path_Status
  */
 int sctp_initLibrary(void);
 
-
 /**
  * Function returns coded library version as result. This unsigned integer
  * contains the major version in the upper 16 bits, and the minor version in
@@ -505,25 +529,25 @@ unsigned int sctp_getLibraryVersion(void);
 
 
 int sctp_registerInstance(unsigned short localPort,
-                          unsigned short noOfInStreams,
-                          unsigned short noOfOutStreams,
-                          unsigned int   noOfLocalAddresses,
-                          unsigned char  localAddressList[SCTP_MAX_NUM_ADDRESSES][SCTP_MAX_IP_LEN],
-                          SCTP_ulpCallbacks ULPcallbackFunctions);
+                            unsigned short noOfInStreams,
+                            unsigned short noOfOutStreams,
+                            unsigned int   noOfLocalAddresses,
+                            unsigned char  localAddressList[][SCTP_MAX_IP_LEN],
+                            SCTP_ulpCallbacks ULPcallbackFunctions);
 
-int sctp_unregisterInstance(unsigned short instance_name);
+int sctp_unregisterInstance(int instanceName);
+
+int sctp_bindAddressToInstance(int SCTP_InstanceName, unsigned char newAddress[SCTP_MAX_IP_LEN]);
 
 
-
-
-unsigned int sctp_associate(unsigned int SCTP_InstanceName,
+unsigned int sctp_associate(int instanceName,
                             unsigned short noOfOutStreams,
                             unsigned char  destinationAddress[SCTP_MAX_IP_LEN],
                             unsigned short destinationPort,
                             void* ulp_data);
 
 
-unsigned int sctp_associatex(unsigned int SCTP_InstanceName,
+unsigned int sctp_associatex(int instanceName,
                              unsigned short noOfOutStreams,
                              unsigned char  destinationAddresses[][SCTP_MAX_IP_LEN],
                              unsigned int   noOfDestinationAddresses,
@@ -531,78 +555,78 @@ unsigned int sctp_associatex(unsigned int SCTP_InstanceName,
                              unsigned short destinationPort,
                              void* ulp_data);
 
-
+                            
 int sctp_shutdown(unsigned int associationID);
 
-int sctp_abort(unsigned int associationID);
+int sctp_abort(unsigned int associationID, int userDataLen, void* abortUserData);
 
-#ifndef SCTP_SOCKET_API
-#define sctp_send sctp_send_private
-#endif
-int sctp_send_private(unsigned int associationID,
-                      unsigned short streamID,
-                      unsigned char *buffer,
-                      unsigned int length,
-                      unsigned int protocolId,
-                      short path_id,         /* -1 for primary path, else address index to be taken */
-                      void * context,        /* SCTP_NO_CONTEXT */
-                      unsigned int lifetime, /* 0xFFFFFFFF-> infinite, 0->no retransmit, else msecs */
-                      int unorderedDelivery, /* use constants SCTP_ORDERED_DELIVERY, SCTP_UNORDERED_DELIVERY */
-                      int dontBundle);  /* use constants SCTP_BUNDLING_ENABLED, SCTP_BUNDLING_DISABLED */
+
+int sctp_send(unsigned int associationID,
+              unsigned short streamID,
+              unsigned char *buffer,
+              unsigned int length,
+              unsigned int protocolId,
+              unsigned int pathId,      /* 0 for primary path, else key of the address to be taken */
+              unsigned int lifetime,    /* 0xFFFFFFFF-> infinite, 0->no retransmit, else msecs */
+              void * context,           /* use e.g. SCTP_NO_CONTEXT */
+              int unorderedDelivery,    /* use constants SCTP_ORDERED_DELIVERY, SCTP_UNORDERED_DELIVERY */
+              int dontBundle);          /* use constants SCTP_BUNDLING_ENABLED, SCTP_BUNDLING_DISABLED */
 
 
 /*
  *  sctp_receive() now returns SCTP_SUCCESS if data was received okay,
- *  also reports back a tsn value (at least one belonging to a fragment)
  *  SCTP_NO_CHUNKS_IN_QUEUE if there was no new data to be received,
  *  SCTP_PARAMETER_PROBLEM,SCTP_ASSOC_NOT_FOUND, SCTP_MODULE_NOT_FOUND if there was an error
+ *  also reports back a tsn value (at least one belonging to a fragment)
  */
 int sctp_receive(unsigned int associationID, unsigned short streamID, unsigned char  *buffer,
                  unsigned int *length, unsigned short *streamSN, unsigned int * tsn, unsigned int flags);
 
 
-int sctp_receivefrom(unsigned int associationID, unsigned short streamID, unsigned char  *buffer,
-                     unsigned int *length, unsigned short *streamSN, unsigned int * tsn,
-                     unsigned int *addressIndex, unsigned int flags);
-
-
-
 /*----------------------------------------------------------------------------------------------*/
 /*  These are the new function for getting/setting parameters per instance, association or path */
 /*----------------------------------------------------------------------------------------------*/
 int sctp_setLibraryParameters(SCTP_LibraryParameters *params);
 int sctp_getLibraryParameters(SCTP_LibraryParameters *params);
 
-int sctp_setAssocDefaults(unsigned short SCTP_InstanceName, SCTP_InstanceParameters* params);
+int sctp_setAssocDefaults(int SCTP_InstanceName, SCTP_InstanceParameters* params);
 
-int sctp_getAssocDefaults(unsigned short SCTP_InstanceName, SCTP_InstanceParameters* params);
+int sctp_getAssocDefaults(int SCTP_InstanceName, SCTP_InstanceParameters* params);
 
 
 int sctp_getAssocStatus(unsigned int associationID, SCTP_AssociationStatus* status);
 
 int sctp_setAssocStatus(unsigned int associationID, SCTP_AssociationStatus* new_status);
 
-int sctp_getPathStatus(unsigned int associationID, short path_id, SCTP_PathStatus* status);
-int sctp_setPathStatus(unsigned int associationID, short path_id, SCTP_PathStatus *new_status);
+int sctp_getPathStatus(unsigned int associationID, unsigned int pathId, SCTP_PathStatus* status);
+int sctp_setPathStatus(unsigned int associationID, unsigned int pathId, SCTP_PathStatus* new_status);
 /*----------------------------------------------------------------------------------------------*/
+
 /*
  * These _could_ be build up from the above functions, but for the sake of a
- * complete API according to RFC 4960, section 10, we have the following
+ * complete API according to RFC 2960, section 10, we have the following
  * six functions  here, explicitly
  */
+int sctp_setPrimary(unsigned int associationID, unsigned int pathId);
 
-short sctp_setPrimary(unsigned int associationID, short path_id);
-short sctp_getPrimary(unsigned int associationID);
+unsigned int sctp_getPrimary(unsigned int associationID);
 
-int sctp_getSrttReport(unsigned int associationID, short path_id);
-int sctp_changeHeartBeat(unsigned int associationID,
-                         short path_id, int heartbeatON, unsigned int timeIntervall);
+int sctp_getSrttReport(unsigned int associationID, unsigned int pathId);
+
+/**
+ *  use SCTP_HEARTBEAT_ON or SCTP_HEARTBEAT_OFF, respectively to turn the
+ *  HB on or off.
+ */
+int sctp_changeHeartBeat(unsigned int   associationID,
+                         unsigned int   pathId,
+                         int            heartbeatON,
+                         unsigned int   timeIntervall);
 
 
-int sctp_requestHeartbeat(unsigned int associationID, short path_id);
+int sctp_requestHeartbeat(unsigned int associationID, unsigned int pathId);
 
 int sctp_setFailureThreshold(unsigned int associationID,
-                             unsigned short pathMaxRetransmissions);
+                             unsigned int pathMaxRetransmissions);
 
 /*
  * this function can be used to get an instance ID of an association
@@ -613,7 +637,7 @@ int sctp_setFailureThreshold(unsigned int associationID,
  * @return 0 for success, 1 if assoc does not exists, -1 for error (i.e.
  *           library not initialized, pointer Null, etc.
  */
-int sctp_getInstanceID(unsigned int associationID, unsigned short* instanceID);
+int sctp_getInstanceID(unsigned int associationID, int* instanceID);
 /*----------------------------------------------------------------------------------------------*/
 
 int sctp_receiveUnsent(unsigned int associationID, unsigned char *buffer, unsigned int *length,
@@ -643,8 +667,6 @@ typedef void (*sctp_socketCallback) (int, unsigned char *, int, unsigned char[]
    on a user file-descriptor
    Params: 1. file-descriptor
    Params: 2. received events mask
-   Params: 3. pointer to registered events mask. It may be changed by the callback function.
-   Params: 4. user data
 */
 typedef void (*sctp_userCallback) (int, short int, short int*, void*);
 
@@ -657,7 +679,6 @@ typedef void (*sctp_userCallback) (int, short int, short int*, void*);
  * @param  scf  callback funtion that is called when data has arrived
  * @return new UDP socket file descriptor, or -1 if error ocurred
  */
-#ifndef WIN32
 int sctp_registerUdpCallback(unsigned char me[],
                              unsigned short my_port,
                              sctp_socketCallback scf);
@@ -666,14 +687,6 @@ int sctp_unregisterUdpCallback(int udp_sfd);
 
 int sctp_sendUdpData(int sfd, unsigned char* buf, int length,
                      unsigned char destination[], unsigned short dest_port);
-#endif
-
-typedef void (*sctp_StdinCallback) (char*, int);
-
-int sctp_registerStdinCallback(sctp_StdinCallback sdf,
-                                 char* buffer, int length);
-
-int sctp_unregisterStdinCallback();
 
 /**
  * this function registers a callback function for catching
@@ -683,10 +696,9 @@ int sctp_unregisterStdinCallback();
  * @param  scf  callback funtion that is called (when return is hit)
  * @return 0, or -1 if error ocurred (i.e. already used)
  */
-#ifndef WIN32
 int sctp_registerUserCallback(int fd, sctp_userCallback sdf, void* userData, short int eventMask);
 int sctp_unregisterUserCallback(int fd);
-#endif
+
 
 /* Defines the callback function that is called when an timer expires.
    Params: 1. ID of timer
@@ -709,7 +721,7 @@ typedef void (*sctp_timerCallback) (unsigned int, void *, void *);
  *      @author     ajung
  */
 unsigned int sctp_startTimer(unsigned int seconds , unsigned int microseconds,
-                        sctp_timerCallback timer_cb, void *param1, void *param2);
+                             sctp_timerCallback timer_cb, void *param1, void *param2);
 
 
 /**
@@ -737,10 +749,12 @@ int sctp_eventLoop(void);
 
 int sctp_extendedEventLoop(void (*lock)(void* data), void (*unlock)(void* data), void* data);
 
-/**
- *  these next funtions are unused. They should either be implemented, or removed :-)
- *  Maybe we should ask Thomas...
- */
+/* ------------------------------------- AS_CONF --------------------------------------------*/
+
+int sctp_setRemotePrimary(unsigned int associationID, unsigned char address[SCTP_MAX_IP_LEN]);
+
+/* ------------------------------------- AS_CONF --------------------------------------------*/
+
 int sctp_getNumberOfLocalAddresses(unsigned int associationID);
 
 unsigned char* sctp_getLocalAddress(unsigned int associationID, int index);
@@ -750,25 +764,14 @@ unsigned char* sctp_getLocalAddress(unsigned int associationID, int index);
 /* buffer points to a buffer that includes SCTP header.
    This header will be overwritten by the function, and
    insert correct data for an existing association */
-int sctp_sendRawData(unsigned int associationID, short path_id,
+int sctp_sendRawData(unsigned int associationID, unsigned int pathId,
                      unsigned char *buffer, unsigned int length);
 
 #endif
 
-#ifdef TD_DEBUG
-#warning Using memory allocation debugging functions!
-#include <sys/types.h>
-void* my_calloc(size_t nmemb, size_t size);
-void* my_malloc(size_t size);
-void my_free(void* p);
-#define calloc my_calloc
-#define malloc my_malloc
-#define free my_free
-#endif
-
 #ifdef __cplusplus
 }
 #endif
 
 
-#endif
+#endif  /* SCTP_H */
